@@ -38,6 +38,7 @@
  * SUCH DAMAGE.  
  */
 
+#include "config.h"
 #include "afflib.h"
 #include "afflib_i.h"
 
@@ -46,8 +47,14 @@
 #include "vnode_afm.h"
 #include "vnode_aff.h"
 #include "vnode_afd.h"
+#ifdef UNIX
 #include "vnode_evf.h"
 #include "vnode_evd.h"
+#endif
+
+#ifdef USE_S3
+#include "vnode_s3.h"
+#endif
 
 #include <fcntl.h>
 #include <assert.h>
@@ -57,11 +64,16 @@
 // vnode implementations.
 // order matters
 struct af_vnode *af_vnode_array[] = {
+#ifdef USE_S3
+    &vnode_s3,				// must be first for s3:// interpertation
+#endif
     &vnode_afd, 
     &vnode_afm,				// must be before aff
     &vnode_aff,
+#ifdef UNIX
     &vnode_evd,
     &vnode_evf,
+#endif
     &vnode_split_raw,			// must be before raw
     &vnode_raw,				// greedy; must be last
     0};
@@ -95,6 +107,8 @@ int	af_last_seg(AFFILE *af,char *last_segname,int last_segname_len,int64 *pos)
  ****************************************************************/
 
 static int aff_initialized = 0;
+int af_cache_debug = 0;
+FILE *af_trace = 0;
 
 void af_initialize()
 {
@@ -104,7 +118,13 @@ void af_initialize()
     assert(sizeof(struct af_head)==8);
     assert(sizeof(struct af_segment_head)==16);
     assert(sizeof(struct af_segment_tail)==8);
+    const char *val = getenv(AFFLIB_CACHE_DEBUG);
+    if(val) af_cache_debug = atoi(val);
+
+    val = getenv(AFFLIB_TRACE);
+    if(val) af_trace = stderr;
     aff_initialized = 1;
+    
 }
 
 
@@ -115,21 +135,20 @@ void af_initialize()
  * -1 if failure.
  */
 
-int af_identify_type(const char *filename)
+int af_identify_file_type(const char *filename,int exists)
 {
-    if(access(filename,R_OK)!=0) return AF_IDENTIFY_NOEXIST;
     for(int i = 0; af_vnode_array[i]; i++){
-	if( (*af_vnode_array[i]->identify)(filename)==1 ){
+	if( (*af_vnode_array[i]->identify)(filename,exists)==1 ){
 	    return (af_vnode_array[i]->type);
 	}
     }
-    return AF_IDENTIFY_ERR;
+    return exists ? AF_IDENTIFY_NOEXIST : AF_IDENTIFY_ERR;
 }
 
-const char *af_identify_name(const char *filename)
+const char *af_identify_file_name(const char *filename,int exists)
 {
     for(int i = 0; af_vnode_array[i]; i++){
-	if( (*af_vnode_array[i]->identify)(filename)==1 ){
+	if( (*af_vnode_array[i]->identify)(filename,exists)==1 ){
 	    return (af_vnode_array[i]->name);
 	}
     }
@@ -155,10 +174,15 @@ AFFILE *af_open_with(const char *filename,int flags,int mode, struct af_vnode *v
     af->fname	  = strdup(filename);	                // remember file name
     af->exists    = (access(filename,R_OK) == 0);	// does the file exist?
     af->openmode  = mode;
-    af->pagenum   = -1;
     af->image_sectorsize = 512;		// default size
-    af->badflag   = (unsigned char *)malloc(af->image_sectorsize);
     af->error_reporter = aff_warnx;
+    af->badflag   = (unsigned char *)malloc(af->image_sectorsize);
+
+    /* Right now just set up the cache by hand */
+    const char *cache_pages = getenv(AFFLIB_CACHE_PAGES);
+    if(cache_pages) af->num_pbufs = atoi(cache_pages);
+    if(af->num_pbufs<1) af->num_pbufs = AFFLIB_CACHE_PAGES_DEFAULT; // default valuen
+    af->pbcache   = (struct aff_pagebuf *)calloc(af->num_pbufs,sizeof(struct aff_pagebuf));
 
     /* Try opening it! */
     if((*af->v->open)(af)){		
@@ -167,6 +191,7 @@ AFFILE *af_open_with(const char *filename,int flags,int mode, struct af_vnode *v
 	free(af);
 	return 0;
     }
+    af_read_sizes(af);			// set up the metadata
     return af;
 }
 
@@ -179,11 +204,13 @@ AFFILE *af_open(const char *filename,int flags,int mode)
 	errno = EINVAL;
 	return 0;			// this flag not supported
     }
+    int exists = (flags & O_CREAT) ? 0 : 1; // file most exist if O_CREAT not specified
+
 
     /* Figure out it's format, then hand off to the correct subsystem. */
     for(int i = 0; af_vnode_array[i]; i++){
 	/* Check to see if the implementation identifies the file */
-	if( (*af_vnode_array[i]->identify)(filename)==1 ){
+	if( (*af_vnode_array[i]->identify)(filename,exists)==1 ){
 	    return af_open_with(filename,flags,mode,af_vnode_array[i]);
 	}
     }
@@ -204,6 +231,7 @@ AFFILE *af_freopen(FILE *file)
     return af;
 }
 
+#ifdef UNIX
 /* Open a regular file as an affile */
 AFFILE *af_popen(const char *command,const char *type)
 {
@@ -216,6 +244,7 @@ AFFILE *af_popen(const char *command,const char *type)
     af->fname     = strdup(command);
     return af;
 }
+#endif
 
 
 /* Close the image */
@@ -223,21 +252,31 @@ int af_close(AFFILE *af)
 {
     int ret = 0;
 
-    if(af->writing){
-	ret = af_purge(af);			// empty any buffers in memory
-    }
+    af_cache_flush(af);			// flush the cache (if writing)
 
     if(af->writing && af->image_size != af->image_size_in_file){
-	af_update_segq(af,AF_IMAGESIZE,(int64)af->image_size,1);
+	af_update_segq(af,AF_IMAGESIZE,(int64)af->image_size);
 	af->image_size_in_file = af->image_size;
     }
-    (*af->v->close)(af);
-    if(af->pagebuf){
-	memset(af->pagebuf,0,af->image_pagesize); // clean object reuse
-	free(af->pagebuf);
+    if(getenv(AFFLIB_CACHE_STATS)){
+	fputc('\n',stderr);
+	af_stats(af,stderr);
     }
-    if(af->fname) free(af->fname);
-    if(af->badflag) free(af->badflag);
+
+    (*af->v->close)(af);
+    /* Clear out the cache */
+    if(af->pbcache){
+	for(int i=0;i<af->num_pbufs;i++){
+	    struct aff_pagebuf *p = &af->pbcache[i];
+	    if(p->pagebuf){
+		memset(p->pagebuf,0,af->image_pagesize); // clean object reuse
+		free(p->pagebuf);
+	    }
+	}
+	free(af->pbcache);
+    }
+    if(af->fname)	free(af->fname);
+    if(af->badflag)	free(af->badflag);
     memset(af,0,sizeof(*af));		// clean object reuse
     free(af);
     return ret;
@@ -245,10 +284,9 @@ int af_close(AFFILE *af)
 
 
 /* Seek in the virtual file */
-uint64 af_seek(AFFILE *af,uint64 pos,int whence)
+uint64 af_seek(AFFILE *af,int64 pos,int whence)
 {
-    if(af->logfile) fprintf(af->logfile,"af_seek(%p,%" I64d",%d)\n",af,pos,whence);
-
+    if(af_trace) fprintf(af_trace,"af_seek(%p,%"I64d",%d)\n",af,pos,whence);
     uint64 new_pos=0;
     switch(whence){
     case SEEK_SET:
@@ -323,7 +361,7 @@ const char *af_ext(const char *str)
     return str;
 }
 
-int af_ext_is(const char *filename,const char *ext)
+bool af_ext_is(const char *filename,const char *ext)
 {
     return strcasecmp(af_ext(filename),ext)==0;
 }
@@ -379,13 +417,15 @@ int af_rewind_seg(AFFILE *af)
 }
 
 int af_update_seg(AFFILE *af, const char *name,
-		  unsigned long arg,const void *value,unsigned int vallen,int append)
+		  unsigned long arg,const void *value,unsigned int vallen)
 {
     if(af->v->update_seg==0){
 	errno = ENOTSUP;
 	return -1;	// not supported by this file system
     }
-    return (*af->v->update_seg)(af,name,arg,value,vallen,append);
+    int r = (*af->v->update_seg)(af,name,arg,value,vallen);
+    if(r==0) af->bytes_written += vallen;
+    return r;
 }
 
 
@@ -416,6 +456,19 @@ int af_has_pages(AFFILE *af)
 }
 
 
+void af_stats(AFFILE *af,FILE *f)
+{
+    fprintf(f,"AFSTATS for %s\n",af_filename(af));
+    fprintf(f,"Pages read: %"I64u"\n",af->pages_read);
+    fprintf(f,"Pages written: %"I64u"\n",af->pages_written);
+    fprintf(f,"Pages compressed: %"I64u"\n",af->pages_compressed);
+    fprintf(f,"Pages decompressed: %"I64u"\n",af->pages_decompressed);
+    fprintf(f,"Cache hits: %"I64u"\n",af->cache_hits);
+    fprintf(f,"Cache misses: %"I64u"\n",af->cache_misses);
+    fprintf(f,"Bytes copied: %"I64u"\n",af->bytes_memcpy);
+    
+}
+
 #ifdef NO_ERR
 #include <stdarg.h>
 void err(int eval,const char *fmt,...)
@@ -425,6 +478,16 @@ void err(int eval,const char *fmt,...)
   vfprintf(stderr,fmt,ap);
   fprintf(stderr,": %s\n",strerror(errno));
   va_end(ap);
+  exit(eval);
+}
+void errx(int eval,const char *fmt,...)
+{
+  va_list ap;
+  va_start(ap,fmt);
+  vfprintf(stderr,fmt,ap);
+  fprintf(stderr,"%s\n",strerror(errno));
+  va_end(ap);
+  exit(eval);
 }
 #endif
 
