@@ -53,22 +53,43 @@
 #include <string.h>
 #include <stdlib.h>
 #include <zlib.h>
-#include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
 #include <assert.h>
 #include <errno.h>
 #include <openssl/rand.h>
+#include <openssl/md5.h>
 
 /* Additional includes for specific platforms */
 
 #ifdef WIN32
-#include "unix4win32.h"
+#define NO_ERR
+extern "C" {
+#include "getopt_win32.h"
+}
+#include <malloc.h>
+#include <windows.h>
+#include <io.h>
 #include <winsock.h>			// htonl()
+#include <direct.h>
+
+
+#define snprintf _snprintf
+#define strcasecmp _stricmp
+#define mkdir(path,mode) _mkdir(path)
+#define ENOTSUP 65536		/* made up number */
+#define random() rand()
+typedef int mode_t;
+#define MAXPATHLEN 1024
+#ifndef F_OK
+#define F_OK 00
+#define R_OK 04
+#endif
 #endif
 
 #ifdef UNIX
+#include <sys/param.h>
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <dirent.h>
@@ -86,18 +107,15 @@
 #ifdef linux
 #define POINTER_FMT "%p"
 #include <err.h>
-#define NO_WARNX
 #endif
 
 #ifdef sun
 #define NO_ERR
-#define NO_WARNX
 #include <alloca.h>
 #endif
 
 #ifdef __CYGWIN__
 #define NO_ERR
-#define NO_WARNX
 #endif
 
 
@@ -117,10 +135,7 @@
 
 #ifdef NO_ERR
 void err(int eval, const char *fmt, ...);
-#endif
-
-#ifdef NO_WARNX
-void warnx(const char *fmt, ...);
+void errx(int eval, const char *fmt, ...);
 #endif
 
 
@@ -128,29 +143,6 @@ void warnx(const char *fmt, ...);
  *** AFFLIB internal stuff follows.
  ****************************************************************/
 
-/* The AFF STREAM VNODE */
-struct af_vnode {
-    int type;				// numeric vnode type
-    int flag;				// file system flag type
-    char *name;
-    int (*identify)(const char *fname);	// returns 1 if file system is identified by implementation
-    int (*open)(AFFILE *af);
-    int (*close)(AFFILE *af);
-    int (*vstat)(AFFILE *af,struct af_vnode_info *);	// returns info about the vnode image file
-    int (*get_seg)(AFFILE *af,const char *name,unsigned long *arg, unsigned char *data,size_t *datalen);
-    int	(*get_next_seg)(AFFILE *af,char *segname,size_t segname_len,
-			unsigned long *arg, unsigned char *data, size_t *datalen);
-    int (*rewind_seg)(AFFILE *af);
-    int (*update_seg)(AFFILE *af,const char *name,unsigned long arg,
-		      const void *value,unsigned int vallen,int append);
-    int (*del_seg)(AFFILE *af,const char *name);
-    int (*read)(AFFILE *af,unsigned char *buf,uint64 offset,int count);
-    int (*write)(AFFILE *af,unsigned char *buf,uint64 offset,int count);
-};
-
-#define AF_VNODE_TYPE_PRIMITIVE 0x01	// single-file impelemntation
-#define AF_VNODE_TYPE_COMPOUND  0x02	// multi-file impelemntation
-
 struct af_vnode_info {
     int64 imagesize;			// size of this image
     int   pagesize;			// what is the natural page size?
@@ -164,6 +156,30 @@ struct af_vnode_info {
 };
 
 
+/* The AFF STREAM VNODE */
+struct af_vnode {
+    int type;				// numeric vnode type
+    int flag;				// file system flag type
+    char *name;
+    int (*identify)(const char *fname,int exists);	// returns 1 if file system is identified by implementation; 
+    int (*open)(AFFILE *af);
+    int (*close)(AFFILE *af);
+    int (*vstat)(AFFILE *af,struct af_vnode_info *);	// returns info about the vnode image file
+    int (*get_seg)(AFFILE *af,const char *name,unsigned long *arg, unsigned char *data,size_t *datalen);
+    int	(*get_next_seg)(AFFILE *af,char *segname,size_t segname_len,
+			unsigned long *arg, unsigned char *data, size_t *datalen);
+    int (*rewind_seg)(AFFILE *af);
+    int (*update_seg)(AFFILE *af,const char *name,unsigned long arg,
+		      const void *value,unsigned int vallen);
+    int (*del_seg)(AFFILE *af,const char *name);
+    int (*read)(AFFILE *af,unsigned char *buf,uint64 offset,int count);
+    int (*write)(AFFILE *af,unsigned char *buf,uint64 offset,int count);
+};
+
+#define AF_VNODE_TYPE_PRIMITIVE 0x01	// single-file implemntation
+#define AF_VNODE_TYPE_COMPOUND  0x02	// multi-file implemntation
+#define AF_VNODE_TYPE_RELIABLE  0x04	// writes are reliable; no need to verify them.
+
 int af_vstat(AFFILE *af,struct af_vnode_info *vni); // does the stat
 
 /* The header for an AFF file. All binary numbers are stored in network byte order. */
@@ -209,6 +225,7 @@ struct af_toc_mem {
 
 void af_initialize();			// initialize the AFFLIB
                                         // automatically called by af_open()
+void aff_warnx(const char *fmt,...);
 AFFILE *af_open_with(const char *filename,int flags,int mode, struct af_vnode *v);
 extern struct af_vnode *af_vnode_array[]; // array of filesystems; last is a "0"
 
@@ -239,13 +256,13 @@ const char *af_hexbuf(char *dst,int dst_len,const unsigned char *bin,int bytes,i
  * Returns 0 if successful, -1 if error.
  */
 
-int	af_figure_media(int fd,struct af_figure_media_buf *);
 struct af_figure_media_buf {
     int version;
     int sector_size;
     uint64 total_sectors;
     int max_read_blocks;
 };
+int	af_figure_media(int fd,struct af_figure_media_buf *);
 
 /****************************************************************
  *** Lowest-level routines for manipulating the AFF File...
@@ -294,14 +311,25 @@ int	af_update_page(AFFILE *af,int64 pagenum,unsigned char *data,int datalen);
 int	af_page_size(AFFILE *af);	// returns page size, or -1
 int	af_get_page_raw(AFFILE *af,int64 pagenum,unsigned long *arg,unsigned char *data,size_t *bytes);
 int	af_get_page(AFFILE *af,int64 pagenum,unsigned char *data,size_t *bytes);
-int	af_purge(AFFILE *af);		// write buffers to disk
+
+extern  int af_cache_debug;
+void	af_cache_writethrough(AFFILE *af,int64 pagenum,
+			      const unsigned char *buf,int bufflen);
+int	af_cache_flush(AFFILE *af);		// write buffers to disk
+struct aff_pagebuf *af_cache_alloc(AFFILE *af,int64 pagenum);
+
 
 /* afflib_util.cpp
  */
-unsigned long long af_decode_q(unsigned char buf[8]); // return buf[8] into an unsigned quad
-#ifndef __FreeBSD__
-void strlcpy(char *dest,const char *src,int dest_size);
-void strlcat(char *dest,const char *src,int dest_size);
+uint64  af_decode_q(unsigned char buf[8]); // return buf[8] into an unsigned quad
+const char *af_commas(char buf[64],int64 val);
+
+#ifndef HAVE_STRLCPY
+size_t strlcpy(char *dest,const char *src,size_t dest_size);
+#endif
+
+#ifndef HAVE_STRLCAT
+size_t strlcat(char *dest,const char *src,size_t dest_size);
 #endif
 
 
@@ -315,10 +343,16 @@ void strlcat(char *dest,const char *src,int dest_size);
 
 int	af_toc_free(AFFILE *af);
 void	af_toc_print(AFFILE *af);
-void	af_toc_append(AFFILE *af,const char *segname,int64 offset);
+int	af_toc_append(AFFILE *af,const char *segname,int64 offset);
 int	af_toc_build(AFFILE *af);	// build by scanning the AFFILE
 struct af_toc_mem *af_toc(AFFILE *af,const char *segname);
 void af_toc_del(AFFILE *af,const char *segname);
 void af_toc_insert(AFFILE *af,const char *segname,int64 offset);
 
+/* lzma_glue.cpp:
+ * For the LZMA compression engine
+ */
+int lzma_compress(unsigned char *dest,size_t *destLen, const unsigned char *data,size_t datalen,int level);
+int lzma_uncompress(unsigned char *buf,size_t *buflen, const unsigned char *cbuf,size_t cbuf_size);
+
 #endif
