@@ -4,6 +4,7 @@
  * Functions for the manipulation of AFF files...
  */
 
+#include "config.h"
 #include "afflib.h"
 #include "afflib_i.h"
 #include "vnode_aff.h"
@@ -246,7 +247,7 @@ static int aff_rewind_seg(AFFILE *af)
  */
 
 int aff_update_seg(AFFILE *af, const char *name,
-		    unsigned long arg,const void *value,unsigned int vallen,int append)
+		    unsigned long arg,const void *value,unsigned int vallen)
 {
     char   next_segment_name[AF_MAX_NAME_LEN];
     size_t next_segsize = 0;
@@ -262,7 +263,7 @@ int aff_update_seg(AFFILE *af, const char *name,
     struct af_toc_mem *adm = af_toc(af,name);
        
 #ifdef DEBUG
-    fprintf(stderr,"aff_update_seg(name=%s,arg=%lu,vallen=%u,append=%d)\n",name,arg,vallen,append);
+    fprintf(stderr,"aff_update_seg(name=%s,arg=%lu,vallen=%u)\n",name,arg,vallen);
 #endif
 
     /* Is the last segment an ignore? */
@@ -318,7 +319,6 @@ int aff_update_seg(AFFILE *af, const char *name,
 		}
 	    }
 
-	    append = 1;			// be sure to add the data to the end!
 	    break;			// and exit this loop
 	}
 
@@ -331,30 +331,29 @@ int aff_update_seg(AFFILE *af, const char *name,
 	}
 	fseeko(af->aseg,next_segsize,SEEK_CUR); // skip this segment
     }
-    if(append){				// if appending, just write to the end
-	/* See if there is a place where this would fit. */
-	//fprintf(stderr,"size_closest=%d\n",size_closest);
-	if(size_closest>0){
-	    /* Yes. Put it here and put a new AF_IGNORE in the space left-over
-	     * TODO: If the following space is also an AF_IGNORE, then combine the two.
-	     */
+
+    /* Ready to write */
+    //fprintf(stderr,"size_closest=%d\n",size_closest);
+    if(size_closest>0){
+	/* Yes. Put it here and put a new AF_IGNORE in the space left-over
+	 * TODO: If the following space is also an AF_IGNORE, then combine the two.
+	 */
 #ifdef DEBUG
-	    fprintf(stderr,"size_closest=%d\n",size_closest);
-	    fprintf(stderr,"loc_closest=%qd\n",loc_closest);
-	    fprintf(stderr,"*** Squeezing it in at %qd. name=%s. vallen=%d\n",loc_closest,name,vallen);
+	fprintf(stderr,"size_closest=%d\n",size_closest);
+	fprintf(stderr,"loc_closest=%qd\n",loc_closest);
+	fprintf(stderr,"*** Squeezing it in at %qd. name=%s. vallen=%d\n",loc_closest,name,vallen);
 #endif
-
-	    fseeko(af->aseg,loc_closest,SEEK_SET); // move to the location
-	    aff_write_seg(af,name,arg,value,vallen); // write the new segment
-
-	    int newsize = size_closest - vallen - aff_ignore_overhead() - strlen(name);
-
-	    aff_write_ignore(af,newsize); // write the smaller ignore
-	    return 0;
-	}
-	return aff_write_seg(af,name,arg,value,vallen);
+	
+	fseeko(af->aseg,loc_closest,SEEK_SET); // move to the location
+	aff_write_seg(af,name,arg,value,vallen); // write the new segment
+	
+	int newsize = size_closest - vallen - aff_ignore_overhead() - strlen(name);
+	
+	aff_write_ignore(af,newsize); // write the smaller ignore
+	return 0;
     }
-    return -2;				// couldn't find segment
+    /* Just write to the end of the file */
+    return aff_write_seg(af,name,arg,value,vallen); // just write at the end
 }
 
 
@@ -368,20 +367,23 @@ int aff_update_seg(AFFILE *af, const char *name,
 
 static int aff_del_seg(AFFILE *af,const char *segname)
 {
-    char last_segname[AF_MAX_NAME_LEN];
-    int64 last_pos;
-
     af_toc_del(af,segname);		// remove it from the directory
 
     /* Find out if the last segment is the one we are deleting;
      * If so, we can just truncate the file.
+     * Note: we can't do this on Windows because _chsize(), 
+     * the equivillent of ftruncate(), takes a 32-bit argument.
      */
+#ifndef WIN32
+    char last_segname[AF_MAX_NAME_LEN];
+    int64 last_pos;
     af_last_seg(af,last_segname,sizeof(last_segname),&last_pos);
     if(strcmp(segname,last_segname)==0){
 	fflush(af->aseg);		// flush any ouput
 	ftruncate(fileno(af->aseg),last_pos); // make the file shorter
 	return 0;
     }
+#endif
 
     size_t datasize=0,segsize=0;
     if(aff_find_seg(af,segname,0,&datasize,&segsize)!=0){
@@ -405,11 +407,12 @@ int af_make_badflag(AFFILE *af)
 {
     RAND_pseudo_bytes(af->badflag,af->image_sectorsize);
     strcpy((char *)af->badflag,"BAD SECTOR");
+    af->badflag_set = 1;
 
-    if(af_update_seg(af,AF_BADFLAG,0,af->badflag,af->image_sectorsize,1)){
+    if(af_update_seg(af,AF_BADFLAG,0,af->badflag,af->image_sectorsize)){
 	return -1;
     }
-    if(af_update_segq(af,AF_BADSECTORS,0,1)){
+    if(af_update_segq(af,AF_BADSECTORS,0)){
 	return -1;
     }
     return 0;
@@ -427,14 +430,14 @@ static int aff_create(AFFILE *af)
     af_toc_build(af);	             // build the toc (will be pretty small)
     af_make_badflag(af);	     // writes the flag for bad blocks
     
-    char *version = xstr(AFFLIB_VERSION);
-    af_update_seg(af,AF_AFFLIB_VERSION,0,version,strlen(version),1);
+    char *version = xstr(PACKAGE_VERSION);
+    af_update_seg(af,AF_AFFLIB_VERSION,0,version,strlen(version));
     
-#ifdef HAS_GETPROGNAME
+#ifdef HAVE_GETPROGNAME
     const char *progname = getprogname();
-    if(af_update_seg(af,AF_CREATOR,0,progname,strlen(progname),1)) return -1;
+    if(af_update_seg(af,AF_CREATOR,0,progname,strlen(progname))) return -1;
 #endif
-    if(af_update_seg(af,AF_AFF_FILE_TYPE,0,"AFF",3,1)) return -1;
+    if(af_update_seg(af,AF_AFF_FILE_TYPE,0,"AFF",3)) return -1;
 
     return 0;
 }
@@ -452,7 +455,7 @@ static int aff_open(AFFILE *af)
     if(fd<0){				// couldn't open
 	return -1;
     }
-    af->compression_type     = AF_COMPRESSION_ZLIB; // default
+    af->compression_type     = AF_COMPRESSION_ALG_ZLIB; // default
     af->compression_level    = Z_DEFAULT_COMPRESSION;
 
     /* Open the FILE  for the AFFILE */
@@ -496,8 +499,7 @@ static int aff_open(AFFILE *af)
     }
 
     /* File has been validated */
-    af_toc_build(af);			// build the TOC
-    af_read_sizes(af);			// read all of the relevant sizes
+    if(af_toc_build(af)) return -1;	// build the TOC
     return 0;				// everything must be okay.
 }
 
@@ -512,8 +514,9 @@ static bool last4_is_aff(const char *filename)
 }
 
 /* Return 1 if a file is an AFF file */
-static int aff_identify_file(const char *filename)
+static int aff_identify_file(const char *filename,int exists)
 {
+    if(exists && access(filename,R_OK)!=0) return 0;	// needs to exist and it doesn't
     int fd = open(filename,O_RDONLY | O_BINARY);
     if(fd>0){
 	int len = strlen(AF_HEADER)+1;	
@@ -546,12 +549,6 @@ static int aff_close(AFFILE *af)
 }
 
 
-#ifdef __FreeBSD__
-#include <sys/param.h>
-#include <sys/mount.h>
-#define HAS_STATFS
-#endif
-
 static int aff_vstat(AFFILE *af,struct af_vnode_info *vni)
 {
     memset(vni,0,sizeof(*vni));		// clear it
