@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2006/09/01 15:23:03 $
+** $Date: 2006/11/09 14:50:35 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2004-2005 Brian Carrier.  All rights reserved
@@ -10,10 +10,6 @@
 #ifndef _TSK_OS_H
 #define _TSK_OS_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
     /*
      * Solaris 2.x. Build for large files when dealing with filesystems > 2GB.
      * With the 32-bit file model, needs pread() to access filesystems > 2GB.
@@ -146,6 +142,9 @@ extern "C" {
 #define _UNICODE
 #endif
 #define WIN32_LEAN_AND_MEAN	/* somewhat limit Win32 pollution */
+
+
+
 #include <windows.h>
 #include <tchar.h>
 #include <io.h>
@@ -153,17 +152,19 @@ extern "C" {
 #define _CRT_SECURE_NO_DEPRECATE	1
 
 #include "intrin.h"
-    typedef unsigned __int8 uint8_t;
-    typedef __int8 int8_t;
-    typedef unsigned __int16 uint16_t;
-    typedef __int16 int16_t;
-    typedef unsigned __int32 uint32_t;
-    typedef __int32 int32_t;
-    typedef unsigned __int64 uint64_t;
-    typedef __int64 int64_t;
-    typedef int mode_t;
-    typedef uint16_t gid_t;
-    typedef uint16_t uid_t;
+
+
+typedef unsigned __int8 uint8_t;
+typedef __int8 int8_t;
+typedef unsigned __int16 uint16_t;
+typedef __int16 int16_t;
+typedef unsigned __int32 uint32_t;
+typedef __int32 int32_t;
+typedef unsigned __int64 uint64_t;
+typedef __int64 int64_t;
+typedef int mode_t;
+typedef uint16_t gid_t;
+typedef uint16_t uid_t;
 
 #define strtoull	strtoul
 //#define open(filename, oflag) _open(filename, oflag|_O_BINARY, 0)
@@ -178,9 +179,9 @@ extern "C" {
 
 #if !defined(_SYS_INT_TYPES_H)
 #if defined (_LP64) || defined (_I32LPx)
-    typedef unsigned long uintptr_t;
+typedef unsigned long uintptr_t;
 #else
-    typedef unsigned int uintptr_t;
+typedef unsigned int uintptr_t;
 #endif
 #endif
 
@@ -190,6 +191,8 @@ extern "C" {
 
 #ifdef TSK_WIN32
 #define MAIN _tmain
+typedef TCHAR TSK_TCHAR;
+#define _TSK_T	_T
 #define TSTAT _tstat
 #define STAT_STR    _stat64i32
 //#define TSTRNCPY(d, s, l)    _tcscpy_s(d, l, s)
@@ -210,8 +213,8 @@ extern "C" {
 #else
 
 #define MAIN main
-#define _T(x)	x
-    typedef char TCHAR;
+#define _TSK_T(x)	x
+typedef char TSK_TCHAR;
 #define TSTAT stat
 #define STAT_STR    stat
 //#define TSTRNCPY(d,s,l)    strncpy(d,s,l)
@@ -239,7 +242,4 @@ extern "C" {
 #error "This operating system is not supported"
 #endif
 
-#ifdef __cplusplus
-}
-#endif
 #endif
