@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  * 
- * $Date: 2006/08/25 18:28:05 $
+ * $Date: 2006/12/07 16:25:39 $
  */
 #include <stdlib.h>
 #include <stdio.h>
@@ -18,3 +18,13 @@ print_version(FILE * hFile)
 #endif
     return;
 }
+
+char *
+tskGetVersion()
+{
+#ifdef VER
+    return VER;
+#else
+    return "0.0";
+#endif
+}
