@@ -2,7 +2,7 @@
 ** dcalc
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/09/20 20:16:01 $
 **
 ** Calculates the corresponding block number between 'dls' and 'dd' images
 ** when given an 'dls' block number, it determines the block number it
@@ -26,13 +26,13 @@
 #include <locale.h>
 #include "fs_tools.h"
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 static void
 usage()
 {
     TFPRINTF(stderr,
-	_T
+	_TSK_T
 	("usage: %s [-dsu unit_addr] [-vV] [-f fstype] [-i imgtype] [-o imgoffset] image [images]\n"),
 	progname);
     tsk_fprintf(stderr, "Slowly calculates the opposite block number\n");
@@ -58,12 +58,12 @@ usage()
 
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
-    TCHAR *fstype = NULL;
-    TCHAR *imgtype = NULL;
+    TSK_TCHAR *fstype = NULL;
+    TSK_TCHAR *imgtype = NULL;
     int ch;
-    TCHAR *cp;
+    TSK_TCHAR *cp;
     uint8_t type = 0;
     FS_INFO *fs;
     IMG_INFO *img;
@@ -74,71 +74,72 @@ MAIN(int argc, TCHAR ** argv)
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, _T("d:f:i:o:s:u:vV"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("d:f:i:o:s:u:vV"))) > 0) {
 	switch (ch) {
-	case _T('?'):
+	case _TSK_T('?'):
 	default:
-	    TFPRINTF(stderr, _T("Invalid argument: %s\n"), argv[optind]);
+	    TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+		argv[optind]);
 	    usage();
 
-	case _T('d'):
+	case _TSK_T('d'):
 	    type |= DCALC_DD;
 	    count = TSTRTOULL(optarg, &cp, 0);
 	    if (*cp || cp == optarg) {
-		TFPRINTF(stderr, _T("Invalid address: %s\n"), optarg);
+		TFPRINTF(stderr, _TSK_T("Invalid address: %s\n"), optarg);
 		usage();
 	    }
 	    set = 1;
 	    break;
 
-	case _T('f'):
+	case _TSK_T('f'):
 	    fstype = optarg;
-	    if (TSTRCMP(fstype, _T("list")) == 0) {
+	    if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
 		fs_print_types(stderr);
 		exit(1);
 	    }
 	    break;
 
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 	    break;
 
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
 
-	case _T('s'):
+	case _TSK_T('s'):
 	    type |= DCALC_SLACK;
 	    count = TSTRTOULL(optarg, &cp, 0);
 	    if (*cp || cp == optarg) {
-		TFPRINTF(stderr, _T("Invalid address: %s\n"), optarg);
+		TFPRINTF(stderr, _TSK_T("Invalid address: %s\n"), optarg);
 		usage();
 	    }
 	    set = 1;
 	    break;
 
-	case _T('u'):
+	case _TSK_T('u'):
 	    type |= DCALC_DLS;
 	    count = TSTRTOULL(optarg, &cp, 0);
 	    if (*cp || cp == optarg) {
-		TFPRINTF(stderr, _T("Invalid address: %s\n"), optarg);
+		TFPRINTF(stderr, _TSK_T("Invalid address: %s\n"), optarg);
 		usage();
 	    }
 	    set = 1;
 	    break;
 
-	case _T('v'):
+	case _TSK_T('v'):
 	    verbose++;
 	    break;
 
-	case _T('V'):
+	case _TSK_T('V'):
 	    print_version(stdout);
 	    exit(0);
 	}
@@ -163,7 +164,7 @@ MAIN(int argc, TCHAR ** argv)
 
     if ((img =
 	    img_open(imgtype, argc - optind,
-		(const TCHAR **) &argv[optind])) == NULL) {
+		(const TSK_TCHAR **) &argv[optind])) == NULL) {
 	tsk_error_print(stderr);
 	exit(1);
     }
