@@ -2,7 +2,7 @@
 ** dcalc
 ** The Sleuth Kit 
 **
-** $Date: 2006/09/01 15:23:03 $
+** $Date: 2006/12/05 21:39:52 $
 **
 ** Calculates the corresponding block number between 'dls' and 'dd' images
 ** when given an 'dls' block number, it determines the block number it
@@ -146,9 +146,13 @@ count_slack_inode_act(FS_INFO * fs, FS_INODE * fs_inode,
 
     /* For NTFS we go through each non-resident attribute */
     else {
-	FS_DATA *fs_data = fs_inode->attr;
+	FS_DATA *fs_data;
 
-	while ((fs_data) && (fs_data->flags & FS_DATA_INUSE)) {
+	for (fs_data = fs_inode->attr;
+	    fs_data != NULL; fs_data = fs_data->next) {
+
+	    if ((fs_data->flags & FS_DATA_INUSE) == 0)
+		continue;
 
 	    if (fs_data->flags & FS_DATA_NONRES) {
 		flen = fs_data->size;
@@ -162,8 +166,6 @@ count_slack_inode_act(FS_INFO * fs, FS_INODE * fs_inode,
 		    tsk_error_reset();
 		}
 	    }
-
-	    fs_data = fs_data->next;
 	}
     }
     return WALK_CONT;
@@ -196,8 +198,7 @@ fs_dcalc(FS_INFO * fs, uint8_t lclflags, DADDR_T cnt)
     }
     else if (lclflags == DCALC_SLACK) {
 	if (fs->inode_walk(fs, fs->first_inum, fs->last_inum,
-		(FS_FLAG_META_ALLOC | FS_FLAG_META_USED |
-		    FS_FLAG_META_LINK), count_slack_inode_act, NULL))
+		FS_FLAG_META_ALLOC, count_slack_inode_act, NULL))
 	    return -1;
     }
 
