@@ -2,7 +2,7 @@
 ** dcat
 ** The  Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/09/20 20:16:01 $
 **
 ** Given an image , block number, and size, display the contents
 ** of the block to stdout.
@@ -28,13 +28,13 @@
 #define DLS_TYPE "dls"
 #define RAW_STR "raw"
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 void
 usage()
 {
     TFPRINTF(stderr,
-	_T
+	_TSK_T
 	("usage: %s [-ahsvVw] [-f fstype] [-i imgtype] [-o imgoffset] [-u usize] image [images] unit_addr [num]\n"),
 	progname);
     tsk_fprintf(stderr, "\t-a: displays in all ASCII \n");
@@ -60,13 +60,13 @@ usage()
 
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
     FS_INFO *fs = NULL;
     IMG_INFO *img;
     DADDR_T addr = 0;
-    TCHAR *fstype = NULL;
-    TCHAR *cp, *imgtype = NULL;
+    TSK_TCHAR *fstype = NULL;
+    TSK_TCHAR *cp, *imgtype = NULL;
     DADDR_T read_num_units;	/* Number of data units */
     int usize = 0;		/* Length of each data unit */
     int ch;
@@ -77,61 +77,63 @@ MAIN(int argc, TCHAR ** argv)
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, _T("af:hi:o:su:vVw"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("af:hi:o:su:vVw"))) > 0) {
 	switch (ch) {
-	case _T('a'):
+	case _TSK_T('a'):
 	    format |= DCAT_ASCII;
 	    break;
-	case _T('f'):
+	case _TSK_T('f'):
 	    fstype = optarg;
-	    if (TSTRCMP(fstype, _T(DLS_TYPE)) == 0)
-		fstype = _T(RAW_STR);
-	    if (TSTRCMP(fstype, _T("list")) == 0) {
+	    if (TSTRCMP(fstype, _TSK_T(DLS_TYPE)) == 0)
+		fstype = _TSK_T(RAW_STR);
+	    if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
 		tsk_fprintf(stderr, "\t%s (Unallocated Space)\n",
 		    DLS_TYPE);
 		fs_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('h'):
+	case _TSK_T('h'):
 	    format |= DCAT_HEX;
 	    break;
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('s'):
+	case _TSK_T('s'):
 	    format |= DCAT_STAT;
 	    break;
-	case _T('u'):
+	case _TSK_T('u'):
 	    usize = TSTRTOUL(optarg, &cp, 0);
 	    if (*cp || cp == optarg) {
-		TFPRINTF(stderr, _T("Invalid block size: %s\n"), optarg);
+		TFPRINTF(stderr, _TSK_T("Invalid block size: %s\n"),
+		    optarg);
 		usage();
 	    }
 	    break;
-	case _T('v'):
+	case _TSK_T('v'):
 	    verbose++;
 	    break;
-	case _T('V'):
+	case _TSK_T('V'):
 	    print_version(stdout);
 	    exit(0);
 	    break;
-	case _T('w'):
+	case _TSK_T('w'):
 	    format |= DCAT_HTML;
 	    break;
-	case _T('?'):
+	case _TSK_T('?'):
 	default:
-	    TFPRINTF(stderr, _T("Invalid argument: %s\n"), argv[optind]);
+	    TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+		argv[optind]);
 	    usage();
 	}
     }
@@ -167,7 +169,7 @@ MAIN(int argc, TCHAR ** argv)
     if (format & DCAT_STAT) {
 	if ((img =
 		img_open(imgtype, argc - optind,
-		    (const TCHAR **) &argv[optind])) == NULL) {
+		    (const TSK_TCHAR **) &argv[optind])) == NULL) {
 	    tsk_error_print(stderr);
 	    exit(1);
 	}
@@ -180,14 +182,14 @@ MAIN(int argc, TCHAR ** argv)
 	    /* Not a number, so it is the image name and we do not have a length */
 	    addr = TSTRTOULL(argv[argc - 1], &cp, 0);
 	    if (*cp || cp == argv[argc - 1]) {
-		TFPRINTF(stderr, _T("Invalid block address: %s\n"),
+		TFPRINTF(stderr, _TSK_T("Invalid block address: %s\n"),
 		    argv[argc - 1]);
 		usage();
 	    }
 
 	    if ((img =
 		    img_open(imgtype, argc - optind - 1,
-			(const TCHAR **) &argv[optind])) == NULL) {
+			(const TSK_TCHAR **) &argv[optind])) == NULL) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
@@ -197,7 +199,8 @@ MAIN(int argc, TCHAR ** argv)
 	    /* We got a number, so take the length as well while we are at it */
 	    read_num_units = TSTRTOULL(argv[argc - 1], &cp, 0);
 	    if (*cp || cp == argv[argc - 1]) {
-		TFPRINTF(stderr, _T("Invalid size: %s\n"), argv[argc - 1]);
+		TFPRINTF(stderr, _TSK_T("Invalid size: %s\n"),
+		    argv[argc - 1]);
 		usage();
 	    }
 	    else if (read_num_units <= 0) {
@@ -208,7 +211,7 @@ MAIN(int argc, TCHAR ** argv)
 
 	    if ((img =
 		    img_open(imgtype, argc - optind - 2,
-			(const TCHAR **) &argv[optind])) == NULL) {
+			(const TSK_TCHAR **) &argv[optind])) == NULL) {
 
 		tsk_error_print(stderr);
 		exit(1);
