@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/09/21 18:23:12 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -28,7 +28,7 @@
 #include <locale.h>
 #include "fs_tools.h"
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 /* usage - explain and terminate */
 
@@ -36,13 +36,15 @@ static void
 usage()
 {
     TFPRINTF(stderr,
-	_T
-	("usage: %s [-belvV] [-f fstype] [-i imgtype] [-o imgoffset] image [images] [start-stop]\n"),
+	_TSK_T
+	("usage: %s [-aAbelvV] [-f fstype] [-i imgtype] [-o imgoffset] image [images] [start-stop]\n"),
 	progname);
     tsk_fprintf(stderr, "\t-b: no block padding\n");
     tsk_fprintf(stderr, "\t-e: every block\n");
     tsk_fprintf(stderr,
 	"\t-l: print details in time machine list format\n");
+    tsk_fprintf(stderr, "\t-a: Display allocated blocks\n");
+    tsk_fprintf(stderr, "\t-A: Display unallocated blocks\n");
     tsk_fprintf(stderr,
 	"\t-f fstype: File system type (use '-f list' for supported types)\n");
     tsk_fprintf(stderr,
@@ -65,10 +67,10 @@ usage()
 /* main - open file system, list block info */
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
-    TCHAR *fstype = NULL;
-    TCHAR *imgtype = NULL, *cp, *dash;
+    TSK_TCHAR *fstype = NULL;
+    TSK_TCHAR *imgtype = NULL, *cp, *dash;
     FS_INFO *fs;
     IMG_INFO *img;
     DADDR_T bstart = 0, blast = 0;
@@ -83,48 +85,55 @@ MAIN(int argc, TCHAR ** argv)
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, _T("bef:i:lo:svV"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("aAbef:i:lo:svV"))) > 0) {
 	switch (ch) {
-	case _T('?'):
+	case _TSK_T('?'):
 	default:
-	    TFPRINTF(stderr, _T("Invalid argument: %s\n"), argv[optind]);
+	    TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+		argv[optind]);
 	    usage();
-	case _T('b'):
-	    flags &= ~FS_FLAG_DATA_ALIGN;
-	    break;
-	case _T('e'):
+	case _TSK_T('a'):
 	    flags |= FS_FLAG_DATA_ALLOC;
 	    break;
-	case _T('f'):
+	case _TSK_T('A'):
+	    flags |= FS_FLAG_DATA_UNALLOC;
+	    break;
+	case _TSK_T('b'):
+	    flags &= ~FS_FLAG_DATA_ALIGN;
+	    break;
+	case _TSK_T('e'):
+	    flags |= (FS_FLAG_DATA_ALLOC | FS_FLAG_DATA_UNALLOC);
+	    break;
+	case _TSK_T('f'):
 	    fstype = optarg;
-	    if (TSTRCMP(fstype, _T("list")) == 0) {
+	    if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
 		fs_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('l'):
+	case _TSK_T('l'):
 	    lclflags = DLS_LIST;
 	    break;
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('s'):
+	case _TSK_T('s'):
 	    lclflags |= DLS_SLACK;
 	    break;
-	case _T('v'):
+	case _TSK_T('v'):
 	    verbose++;
 	    break;
-	case _T('V'):
+	case _TSK_T('V'):
 	    print_version(stdout);
 	    exit(0);
 	}
@@ -148,7 +157,7 @@ MAIN(int argc, TCHAR ** argv)
 	/* There should be no other arguments */
 	img =
 	    img_open(imgtype, argc - optind,
-	    (const TCHAR **) &argv[optind]);
+	    (const TSK_TCHAR **) &argv[optind]);
 
 	if (img == NULL) {
 	    tsk_error_print(stderr);
@@ -166,11 +175,11 @@ MAIN(int argc, TCHAR ** argv)
     else {
 
 	/* We need to determine if the block range was given */
-	if ((dash = TSTRCHR(argv[argc - 1], _T('-'))) == NULL) {
+	if ((dash = TSTRCHR(argv[argc - 1], _TSK_T('-'))) == NULL) {
 	    /* No dash in arg - therefore it is an image file name */
 	    if ((img =
 		    img_open(imgtype, argc - optind,
-			(const TCHAR **) &argv[optind])) == NULL) {
+			(const TSK_TCHAR **) &argv[optind])) == NULL) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
@@ -184,10 +193,10 @@ MAIN(int argc, TCHAR ** argv)
 	    bstart = TSTRTOULL(argv[argc - 1], &cp, 0);
 	    if (*cp || cp == argv[argc - 1]) {
 		/* Not a number - consider it a file name */
-		*dash = _T('-');
+		*dash = _TSK_T('-');
 		if ((img =
 			img_open(imgtype, argc - optind,
-			    (const TCHAR **) &argv[optind])) == NULL) {
+			    (const TSK_TCHAR **) &argv[optind])) == NULL) {
 		    tsk_error_print(stderr);
 		    exit(1);
 		}
@@ -201,10 +210,11 @@ MAIN(int argc, TCHAR ** argv)
 		if (*cp || cp == dash) {
 		    /* Not a number - consider it a file name */
 		    dash--;
-		    *dash = _T('-');
+		    *dash = _TSK_T('-');
 		    if ((img =
 			    img_open(imgtype, argc - optind,
-				(const TCHAR **) &argv[optind])) == NULL) {
+				(const TSK_TCHAR **) &argv[optind])) ==
+			NULL) {
 			tsk_error_print(stderr);
 			exit(1);
 		    }
@@ -217,7 +227,8 @@ MAIN(int argc, TCHAR ** argv)
 		    /* It was a block range, so do not include it in the open */
 		    if ((img =
 			    img_open(imgtype, argc - optind - 1,
-				(const TCHAR **) &argv[optind])) == NULL) {
+				(const TSK_TCHAR **) &argv[optind])) ==
+			NULL) {
 			tsk_error_print(stderr);
 			exit(1);
 		    }
