@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/12/05 21:39:52 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -44,11 +44,11 @@ ext2fs_group_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
      * Sanity check
      */
     if (grp_num < 0 || grp_num >= ext2fs->groups_count) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_group_load: invalid cylinder group number: %"
 	    PRI_EXT2GRP "", grp_num);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -74,8 +74,8 @@ ext2fs_group_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
 	sizeof(ext2fs_gd), offs);
     if (cnt != sizeof(ext2fs_gd)) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "ext2fs_group_load: Group descriptor %" PRI_EXT2GRP " at %"
@@ -146,11 +146,11 @@ ext2fs_bmap_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
      */
     if (getu32(fs->endian,
 	    ext2fs->grp_buf->bg_block_bitmap) > fs->last_block) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_BLK_NUM;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_bmap_load: Block too large for image: %" PRIu32
 	    "", getu32(fs->endian, ext2fs->grp_buf->bg_block_bitmap));
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -160,8 +160,8 @@ ext2fs_bmap_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
 
     if (cnt != ext2fs->fs_info.block_size) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "ext2fs_bmap_load: Bitmap group %" PRI_EXT2GRP " at %"
@@ -214,6 +214,7 @@ ext2fs_imap_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
      */
     if (getu32(fs->endian,
 	    ext2fs->grp_buf->bg_inode_bitmap) > fs->last_block) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_BLK_NUM;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_imap_load: Block too large for image: %" PRIu32
@@ -227,8 +228,8 @@ ext2fs_imap_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
 
     if (cnt != ext2fs->fs_info.block_size) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "ext2fs_imap_load: Inode bitmap %" PRI_EXT2GRP " at %"
@@ -263,10 +264,10 @@ ext2fs_dinode_load(EXT2FS_INFO * ext2fs, INUM_T inum)
      * Sanity check.
      */
     if ((inum < fs->first_inum) || (inum > fs->last_inum)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_INODE_NUM;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_dinode_load: address: %" PRIuINUM, inum);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -309,8 +310,8 @@ ext2fs_dinode_load(EXT2FS_INFO * ext2fs, INUM_T inum)
     cnt = fs_read_random(fs, (char *) dino, ext2fs->inode_size, addr);
     if (cnt != ext2fs->inode_size) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "ext2fs_dinode_load: Inode %" PRIuINUM
@@ -355,10 +356,10 @@ ext2fs_dinode_copy(EXT2FS_INFO * ext2fs, FS_INODE * fs_inode)
     INUM_T ibase = 0;
 
     if (ext2fs->dino_buf == NULL) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_dinode_copy: dino_buf is NULL");
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -475,8 +476,8 @@ ext2fs_dinode_copy(EXT2FS_INFO * ext2fs, FS_INODE * fs_inode)
 
 		if (cnt != fs->block_size) {
 		    if (cnt != -1) {
+			tsk_error_reset();
 			tsk_errno = TSK_ERR_FS_READ;
-			tsk_errstr[0] = '\0';
 		    }
 		    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			"ext2fs_dinode_copy: symlink destination from %"
@@ -523,9 +524,6 @@ ext2fs_dinode_copy(EXT2FS_INFO * ext2fs, FS_INODE * fs_inode)
     fs_inode->flags = (isset(ext2fs->imap_buf, ext2fs->dino_inum - ibase) ?
 	FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 
-    fs_inode->flags |= (fs_inode->nlink ?
-	FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
-
     /*
      * Apply the used/unused restriction.
      */
@@ -546,6 +544,9 @@ ext2fs_inode_lookup(FS_INFO * fs, INUM_T inum)
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     FS_INODE *fs_inode;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if ((fs_inode = fs_inode_alloc(EXT2FS_NDADDR, EXT2FS_NIADDR)) == NULL)
 	return NULL;
 
@@ -562,11 +563,23 @@ ext2fs_inode_lookup(FS_INFO * fs, INUM_T inum)
     return (fs_inode);
 }
 
+
+static uint8_t
+inode_walk_dent_orphan_act(FS_INFO * fs, FS_DENT * fs_dent, int flags,
+    void *ptr)
+{
+    if ((fs_dent->fsi) && (fs_dent->fsi->flags & FS_FLAG_META_UNALLOC)) {
+	if (tsk_list_add(&fs->list_inum_named, fs_dent->fsi->addr))
+	    return WALK_STOP;
+    }
+    return WALK_CONT;
+}
+
+
 /* ext2fs_inode_walk - inode iterator 
  *
  * flags used: FS_FLAG_META_USED, FS_FLAG_META_UNUSED,
- *  FS_FLAG_META_LINK, FS_FLAG_META_UNLINK,
- *  FS_FLAG_META_ALLOC, FS_FLAG_META_UNALLOC
+ *  FS_FLAG_META_ALLOC, FS_FLAG_META_UNALLOC, FS_FLAG_META_ORPHAN
  *
  *  Return 1 on error and 0 on success
 */
@@ -584,26 +597,66 @@ ext2fs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
     int myflags;
 
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     /*
      * Sanity checks.
      */
     if (start_inum < fs->first_inum || start_inum > fs->last_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: start inode: %" PRIuINUM "", myname, start_inum);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
     if (end_inum < fs->first_inum || end_inum > fs->last_inum
 	|| end_inum < start_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: end inode: %" PRIuINUM "", myname, end_inum);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
+    /* If ORPHAN is wanted, then make sure that the flags are correct */
+    if (flags & FS_FLAG_META_ORPHAN) {
+	flags |= FS_FLAG_META_UNALLOC;
+	flags &= ~FS_FLAG_META_ALLOC;
+    }
+    else if (((flags & FS_FLAG_META_ALLOC) == 0) &&
+	((flags & FS_FLAG_META_UNALLOC) == 0)) {
+	flags |= (FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC);
+    }
+
+    /* If neither of the USED or UNUSED flags are set, then set them
+     * both
+     */
+    if (((flags & FS_FLAG_META_USED) == 0) &&
+	((flags & FS_FLAG_META_UNUSED) == 0)) {
+	flags |= (FS_FLAG_META_USED | FS_FLAG_META_UNUSED);
+    }
+
+
+
+    /* If we are looking for orphan files and have not yet filled
+     * in the list of unalloc inodes that are pointed to, then fill
+     * in the list
+     */
+    if ((flags & FS_FLAG_META_ORPHAN) && (fs->list_inum_named == NULL)) {
+
+	if (ext2fs_dent_walk(fs, fs->root_inum,
+		FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC |
+		FS_FLAG_NAME_RECURSE, inode_walk_dent_orphan_act, NULL)) {
+	    strncat(tsk_errstr2,
+		" - ext2fs_inode_walk: identifying inodes allocated by file names",
+		TSK_ERRSTR_L);
+	    return 1;
+	}
+
+    }
+
     if ((fs_inode = fs_inode_alloc(EXT2FS_NDADDR, EXT2FS_NIADDR)) == NULL)
 	return 1;
 
@@ -650,25 +703,26 @@ ext2fs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 	    return 1;
 	}
 
-	/*
-	 * Apply the linked/unlinked restriction.
-	 */
-	myflags |= (getu16(fs->endian, ext2fs->dino_buf->i_nlink) ?
-	    FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
-
-	if ((flags & myflags) != myflags)
-	    continue;
 
 	/*
 	 * Apply the used/unused restriction.
 	 */
-
 	myflags |= (getu32(fs->endian, ext2fs->dino_buf->i_ctime) ?
 	    FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
 
 	if ((flags & myflags) != myflags)
 	    continue;
 
+	/* If we want only orphans, then check if this
+	 * inode is in the seen list
+	 */
+	if ((myflags & FS_FLAG_META_UNALLOC) &&
+	    (flags & FS_FLAG_META_ORPHAN) &&
+	    (tsk_list_find(fs->list_inum_named, inum))) {
+	    continue;
+	}
+
+
 	/*
 	 * Fill in a file system-independent inode structure and pass control
 	 * to the application.
@@ -718,25 +772,35 @@ ext2fs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
     DADDR_T dmin = 0;		/* first block after inodes */
     int myflags;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     /*
      * Sanity checks.
      */
     if (start_blk < fs->first_block || start_blk > fs->last_block) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: start block: %" PRIuDADDR, myname, start_blk);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
     if (end_blk < fs->first_block || end_blk > fs->last_block
 	|| end_blk < start_blk) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: end block: %" PRIuDADDR, myname, end_blk);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
+    /* Sanity check on flags -- make sure at least one ALLOC is set */
+    if (((flags & FS_FLAG_DATA_ALLOC) == 0) &&
+	((flags & FS_FLAG_DATA_UNALLOC) == 0)) {
+	flags |= (FS_FLAG_DATA_ALLOC | FS_FLAG_DATA_UNALLOC);
+    }
+
+
     if ((data_buf = data_buf_alloc(fs->block_size)) == NULL) {
 	return 1;
     }
@@ -771,8 +835,8 @@ ext2fs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
 
 		if (cnt != fs->block_size) {
 		    if (cnt != -1) {
+			tsk_error_reset();
 			tsk_errno = TSK_ERR_FS_READ;
-			tsk_errstr[0] = '\0';
 		    }
 		    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			"ext2fs_block_walk: metadata block %"
@@ -879,8 +943,8 @@ ext2fs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
 
 	    if (cnt != fs->block_size) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "ext2fs_block_walk: block %" PRIuDADDR, addr);
@@ -932,6 +996,7 @@ ext2fs_file_walk_direct(FS_INFO * fs, DATA_BUF * buf[],
     read_count = (length < buf[0]->size ? (size_t) length : buf[0]->size);
 
     if (addr > fs->last_block) {
+	tsk_error_reset();
 	if (flags & FS_FLAG_FILE_RECOVER)
 	    tsk_errno = TSK_ERR_FS_RECOVER;
 	else
@@ -939,7 +1004,6 @@ ext2fs_file_walk_direct(FS_INFO * fs, DATA_BUF * buf[],
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2: Invalid direct address (too large): %"
 	    PRIuDADDR "", (DADDR_T) addr);
-	tsk_errstr2[0] = '\0';
 	return -1;
     }
 
@@ -953,6 +1017,8 @@ ext2fs_file_walk_direct(FS_INFO * fs, DATA_BUF * buf[],
 	    if ((flags & FS_FLAG_FILE_AONLY) == 0)
 		memset(buf[0]->data, 0, (size_t) read_count);
 
+	    myflags |= (FS_FLAG_DATA_SPARSE | FS_FLAG_DATA_ALLOC);
+
 	    retval =
 		action(fs, addr, buf[0]->data, (unsigned int) read_count,
 		myflags, ptr);
@@ -970,8 +1036,8 @@ ext2fs_file_walk_direct(FS_INFO * fs, DATA_BUF * buf[],
 	    cnt = fs_read_block(fs, buf[0], rounded_size, addr);
 	    if (cnt != (SSIZE_T) rounded_size) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "ext2fs_file_walk_direct: block %" PRIuDADDR,
@@ -1013,6 +1079,7 @@ ext2fs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 	    level, addr);
 
     if (addr > fs->last_block) {
+	tsk_error_reset();
 	if (flags & FS_FLAG_FILE_RECOVER)
 	    tsk_errno = TSK_ERR_FS_RECOVER;
 	else
@@ -1020,7 +1087,6 @@ ext2fs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs: Indirect block address too large: %"
 	    PRIuDADDR, (DADDR_T) addr);
-	tsk_errstr2[0] = '\0';
 	return -1;
 
     }
@@ -1036,8 +1102,8 @@ ext2fs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 	cnt = fs_read_block(fs, buf[level], buf[level]->size, addr);
 	if (cnt != buf[level]->size) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_FS_READ;
-		tsk_errstr[0] = '\0';
 	    }
 	    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		"ext2fs_file_walk_indir: Block %" PRIuDADDR,
@@ -1068,6 +1134,7 @@ ext2fs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 	OFF_T prevcnt = todo_count;
 
 	if (getu32(fs->endian, (uint8_t *) & iaddr[n]) > fs->last_block) {
+	    tsk_error_reset();
 	    if (flags & FS_FLAG_FILE_RECOVER)
 		tsk_errno = TSK_ERR_FS_RECOVER;
 	    else
@@ -1075,7 +1142,6 @@ ext2fs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"ext2fs: Address in indirect block too large: %"
 		PRIu32, getu32(fs->endian, (uint8_t *) & iaddr[n]));
-	    tsk_errstr2[0] = '\0';
 	    return -1;
 	}
 
@@ -1126,6 +1192,9 @@ ext2fs_file_walk(FS_INFO * fs, FS_INODE * inode, uint32_t type,
     SSIZE_T read_b = 0;
 
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if (verbose)
 	tsk_fprintf(stderr,
 	    "ext2fs_file_walk: Processing file %" PRIuINUM "\n",
@@ -1219,10 +1288,10 @@ ext2fs_file_walk(FS_INFO * fs, FS_INODE * inode, uint32_t type,
 static uint8_t
 ext2fs_fscheck(FS_INFO * fs, FILE * hFile)
 {
+    tsk_error_reset();
     tsk_errno = TSK_ERR_FS_FUNC;
     snprintf(tsk_errstr, TSK_ERRSTR_L,
 	"fscheck not implemented yet for Ext3");
-    tsk_errstr2[0] = '\0';
     return 1;
 }
 
@@ -1239,6 +1308,9 @@ ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
     time_t tmptime;
 
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     tsk_fprintf(hFile, "FILE SYSTEM INFORMATION\n");
     tsk_fprintf(hFile, "--------------------------------------------\n");
 
@@ -1673,7 +1745,8 @@ print_addr_act(FS_INFO * fs, DADDR_T addr, char *buf,
     EXT2FS_PRINT_ADDR *print = (EXT2FS_PRINT_ADDR *) ptr;
 
     if (flags & FS_FLAG_DATA_CONT) {
-	int i, s;
+	int i;
+	size_t s;
 	/* cycle through the blocks if they exist */
 	for (i = 0, s = size; s > 0; s -= fs->block_size, i++) {
 
@@ -1709,6 +1782,9 @@ ext2fs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
     char ls[12];
     EXT2FS_PRINT_ADDR print;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if ((fs_inode = ext2fs_inode_lookup(fs, inum)) == NULL) {
 	strncat(tsk_errstr2, " - istat",
 	    TSK_ERRSTR_L - strlen(tsk_errstr2));
@@ -1804,8 +1880,8 @@ ext2fs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
 
 	if (cnt != fs->block_size) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_FS_READ;
-		tsk_errstr[0] = '\0';
 	    }
 	    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		"ext2fs_istat: ACL block %" PRIu32,
@@ -2088,6 +2164,11 @@ ext2fs_close(FS_INFO * fs)
     if (ext2fs->imap_buf != NULL)
 	free((char *) ext2fs->imap_buf);
 
+    if (fs->list_inum_named) {
+	tsk_list_free(fs->list_inum_named);
+	fs->list_inum_named = NULL;
+    }
+
     free(ext2fs);
 }
 
@@ -2105,11 +2186,14 @@ ext2fs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype,
     FS_INFO *fs;
     SSIZE_T cnt;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if ((ftype & FSMASK) != EXTxFS_TYPE) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Invalid FS Type in ext2fs_open");
-	tsk_errstr2[0] = '\0';
 	return NULL;
     }
 
@@ -2136,8 +2220,8 @@ ext2fs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype,
 	fs_read_random(fs, (char *) ext2fs->fs, len, (OFF_T) EXT2FS_SBOFF);
     if (cnt != len) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L, "ext2fs_open: superblock");
 	free(ext2fs->fs);
@@ -2151,10 +2235,10 @@ ext2fs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype,
     if (fs_guessu16(fs, ext2fs->fs->s_magic, EXT2FS_FS_MAGIC)) {
 	free(ext2fs->fs);
 	free(ext2fs);
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "not an EXTxFS file system (magic)");
-	tsk_errstr2[0] = '\0';
 	return NULL;
     }
 
@@ -2196,10 +2280,10 @@ ext2fs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype,
     if (fs->inum_count < 10) {
 	free(ext2fs->fs);
 	free(ext2fs);
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Not an EXTxFS file system (inum count)");
-	tsk_errstr2[0] = '\0';
 	return NULL;
     }
 
@@ -2227,10 +2311,10 @@ ext2fs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype,
 	getu32(fs->endian, ext2fs->fs->s_log_frag_size)) {
 	free(ext2fs->fs);
 	free(ext2fs);
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_FUNC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "This file system has fragments that are a different size than blocks, which is not currently supported\nContact brian with details of the system that created this image");
-	tsk_errstr2[0] = '\0';
 	return NULL;
     }
 
@@ -2288,6 +2372,7 @@ ext2fs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype,
     ext2fs->grp_buf = NULL;
     ext2fs->grp_num = 0xffffffff;
 
+    fs->list_inum_named = NULL;
 
     /*
      * Print some stats.
