@@ -3,7 +3,7 @@
 ** ext2fs_journal
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/11/29 22:02:09 $
 **
 ** Journaling code for EXT3FS image
 **
@@ -42,6 +42,7 @@ load_sb_action(FS_INFO * fs, DADDR_T addr, char *buf, size_t size,
     EXT2FS_JINFO *jinfo = ext2fs->jinfo;
 
     if (size < 1024) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_FUNC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "FS block size is less than 1024, not supported in journal yet");
@@ -51,6 +52,7 @@ load_sb_action(FS_INFO * fs, DADDR_T addr, char *buf, size_t size,
     sb = (ext2fs_journ_sb *) buf;
 
     if (big_getu32(sb->magic) != EXT2_JMAGIC) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Journal inode %" PRIuINUM
@@ -78,7 +80,11 @@ ext2fs_jopen(FS_INFO * fs, INUM_T inum)
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     EXT2FS_JINFO *jinfo;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if (!fs) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L, "ext2fs_jopen: fs is null");
 	return 1;
@@ -100,6 +106,7 @@ ext2fs_jopen(FS_INFO * fs, INUM_T inum)
 
     if (fs->file_walk(fs, jinfo->fs_inode, 0, 0, FS_FLAG_FILE_NOID,
 	    load_sb_action, NULL)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_FWALK;
 	snprintf(tsk_errstr, TSK_ERRSTR_L, "Error loading ext3 journal");
 	fs_inode_free(jinfo->fs_inode);
@@ -132,8 +139,12 @@ ext2fs_jentry_walk(FS_INFO * fs, int flags, FS_JENTRY_WALK_FN action,
     DADDR_T i;
     int b_desc_seen = 0;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
 
     if (jinfo == NULL) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_jentry_walk: journal is not open");
@@ -141,6 +152,7 @@ ext2fs_jentry_walk(FS_INFO * fs, int flags, FS_JENTRY_WALK_FN action,
     }
 
     if (jinfo->fs_inode->size != (jinfo->last_block + 1) * jinfo->bsize) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_jentry_walk: journal file size is different from \nsize reported in journal super block");
@@ -161,6 +173,7 @@ ext2fs_jentry_walk(FS_INFO * fs, int flags, FS_JENTRY_WALK_FN action,
     }
 
     if (buf1.left > 0) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_FWALK;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_jentry_walk: Buffer not fully copied");
@@ -420,7 +433,11 @@ ext2fs_jblk_walk(FS_INFO * fs, DADDR_T start, DADDR_T end, int flags,
     DADDR_T i;
     ext2fs_journ_head *head;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if (jinfo == NULL) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_jblk_walk: journal is not open");
@@ -428,6 +445,7 @@ ext2fs_jblk_walk(FS_INFO * fs, DADDR_T start, DADDR_T end, int flags,
     }
 
     if (jinfo->last_block < end) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_jblk_walk: end is too large ");
@@ -435,6 +453,7 @@ ext2fs_jblk_walk(FS_INFO * fs, DADDR_T start, DADDR_T end, int flags,
     }
 
     if (start != end) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_blk_walk: only start == end is currently supported");
@@ -442,6 +461,7 @@ ext2fs_jblk_walk(FS_INFO * fs, DADDR_T start, DADDR_T end, int flags,
     }
 
     if (jinfo->fs_inode->size != (jinfo->last_block + 1) * jinfo->bsize) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_FUNC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_jblk_walk: journal file size is different from size reported in journal super block");
@@ -465,6 +485,7 @@ ext2fs_jblk_walk(FS_INFO * fs, DADDR_T start, DADDR_T end, int flags,
     }
 
     if (buf1.left > 0) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_FWALK;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_jblk_walk: Buffer not fully copied");
@@ -544,6 +565,7 @@ ext2fs_jblk_walk(FS_INFO * fs, DADDR_T start, DADDR_T end, int flags,
     }
 
     if (fwrite(&journ[end * jinfo->bsize], jinfo->bsize, 1, stdout) != 1) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WRITE;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ext2fs_jblk_walk: error writing buffer block");
