@@ -2,7 +2,7 @@
 ** fatfs_dent
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/12/07 22:02:51 $
 **
 ** Human interface Layer support for the FAT file system
 **
@@ -90,8 +90,8 @@ typedef struct {
 } FATFS_DINFO;
 
 
-static uint8_t fatfs_dent_walk_lcl(FS_INFO *, FATFS_DINFO *, INUM_T, int,
-    FS_DENT_WALK_FN, void *);
+static uint8_t fatfs_dent_walk_lcl(FS_INFO *, FATFS_DINFO *, TSK_LIST **,
+    INUM_T, int, FS_DENT_WALK_FN, void *);
 
 
 /**************************************************************************
@@ -130,6 +130,7 @@ find_parent(FATFS_INFO * fatfs, FS_DENT * fs_dent)
 {
     FS_INFO *fs = (FS_INFO *) & fatfs->fs_info;
     FATFS_DINFO dinfo;
+    TSK_LIST *list_seen = NULL;
 
     memset(&dinfo, 0, sizeof(FATFS_DINFO));
 
@@ -156,6 +157,7 @@ find_parent(FATFS_INFO * fatfs, FS_DENT * fs_dent)
     }
 
     if ((fs_dent->fsi->mode & FS_INODE_FMT) != FS_INODE_DIR) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "fatfs_find_parent called on a non-directory");
@@ -167,12 +169,15 @@ find_parent(FATFS_INFO * fatfs, FS_DENT * fs_dent)
      * same first sector 
      */
 
-    if (fatfs_dent_walk_lcl(fs, &dinfo, fs->root_inum,
+    if (fatfs_dent_walk_lcl(fs, &dinfo, &list_seen, fs->root_inum,
 	    FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_RECURSE,
 	    find_parent_act, (void *) &dinfo)) {
+	tsk_list_free(list_seen);
+	list_seen = NULL;
 	return 0;
     }
-
+    tsk_list_free(list_seen);
+    list_seen = NULL;
 
     if (verbose)
 	tsk_fprintf(stderr,
@@ -199,7 +204,7 @@ find_parent(FATFS_INFO * fatfs, FS_DENT * fs_dent)
 */
 static int8_t
 fatfs_dent_parse_buf(FATFS_INFO * fatfs, FATFS_DINFO * dinfo,
-    char *buf, OFF_T len,
+    TSK_LIST ** list_seen, char *buf, OFF_T len,
     DADDR_T * addrs, int flags, FS_DENT_WALK_FN action, void *ptr)
 {
     unsigned int idx, sidx;
@@ -212,10 +217,10 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, FATFS_DINFO * dinfo,
     FATFS_LFN lfninfo;
 
     if (buf == NULL) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "fatfs_dent_parse_buf: buffer is NULL");
-	tsk_errstr2[0] = '\0';
 	return -1;
     }
 
@@ -238,6 +243,7 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, FATFS_DINFO * dinfo,
 	/* Get the base inode for this sector */
 	ibase = FATFS_SECT_2_INODE(fatfs, addrs[sidx]);
 	if (ibase > fs->last_inum) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_ARG;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"fatfs_parse: inode address is too large");
@@ -372,6 +378,7 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, FATFS_DINFO * dinfo,
 			    FATFS_MAXNAMLEN_UTF8), lenientConversion);
 
 		    if (retVal != conversionOK) {
+			tsk_error_reset();
 			tsk_errno = TSK_ERR_FS_UNICODE;
 			snprintf(tsk_errstr, TSK_ERRSTR_L,
 			    "fatfs_parse: Error converting FAT LFN to UTF8: %d",
@@ -543,59 +550,68 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, FATFS_DINFO * dinfo,
 		uint8_t back_recdel = 0;
 		int depth_added = 0;
 
-
-		/* append our name */
-		if ((dinfo->depth < MAX_DEPTH) &&
-		    (DIR_STRSZ >
-			strlen(dinfo->dirs) + strlen(fs_dent->name))) {
-		    dinfo->didx[dinfo->depth] =
-			&dinfo->dirs[strlen(dinfo->dirs)];
-		    strncpy(dinfo->didx[dinfo->depth], fs_dent->name,
-			DIR_STRSZ - strlen(dinfo->dirs));
-		    strncat(dinfo->dirs, "/", DIR_STRSZ);
-		    depth_added = 1;
-		}
-
-		/* save the .. inode value */
-		back_p = dinfo->pardir_inode;
-		dinfo->pardir_inode = dinfo->curdir_inode;
-		dinfo->depth++;
-
-
-		/* This will prevent errors from being generated from the invalid
-		 * deleted files.  save the current setting and set it to del */
-		if (myflags & FS_FLAG_NAME_ALLOC) {
-		    back_recdel = dinfo->recdel;
-		    dinfo->recdel = 1;
-		}
-
-		if (fatfs_dent_walk_lcl(&(fatfs->fs_info), dinfo,
-			fs_dent->inode, flags, action, ptr)) {
-		    /* If the directory could not be loaded,
-		     *                  * then move on */
-		    if (verbose) {
-			tsk_fprintf(stderr,
-			    "fatfs_dent_parse: error reading directory: %"
-			    PRIuINUM "\n", fs_dent->inode);
-			tsk_error_print(stderr);
+		/* Make sure we do not get into an infinite loop */
+		if (0 == tsk_list_find(*list_seen, fs_dent->inode)) {
+		    if (tsk_list_add(list_seen, fs_dent->inode)) {
+			fs_dent_free(fs_dent);
+			return -1;
 		    }
-		    tsk_error_reset();
-		}
 
-		dinfo->depth--;
-		dinfo->curdir_inode = dinfo->pardir_inode;
-		dinfo->pardir_inode = back_p;
+		    /* append our name */
+		    if ((dinfo->depth < MAX_DEPTH) &&
+			(DIR_STRSZ >
+			    strlen(dinfo->dirs) + strlen(fs_dent->name))) {
+			dinfo->didx[dinfo->depth] =
+			    &dinfo->dirs[strlen(dinfo->dirs)];
+			strncpy(dinfo->didx[dinfo->depth], fs_dent->name,
+			    DIR_STRSZ - strlen(dinfo->dirs));
+			strncat(dinfo->dirs, "/", DIR_STRSZ);
+			depth_added = 1;
+		    }
 
-		if (depth_added)
-		    *dinfo->didx[dinfo->depth] = '\0';
+		    /* save the .. inode value */
+		    back_p = dinfo->pardir_inode;
+		    dinfo->pardir_inode = dinfo->curdir_inode;
+		    dinfo->depth++;
 
-		/* Restore the recursion setting */
-		if (myflags & FS_FLAG_NAME_ALLOC) {
-		    dinfo->recdel = back_recdel;
+
+		    /* This will prevent errors from being generated from the invalid
+		     * deleted files.  save the current setting and set it to del */
+		    if (myflags & FS_FLAG_NAME_ALLOC) {
+			back_recdel = dinfo->recdel;
+			dinfo->recdel = 1;
+		    }
+
+		    if (fatfs_dent_walk_lcl(&(fatfs->fs_info), dinfo,
+			    list_seen, fs_dent->inode, flags, action,
+			    ptr)) {
+			/* If the directory could not be loaded,
+			 *                  * then move on */
+			if (verbose) {
+			    tsk_fprintf(stderr,
+				"fatfs_dent_parse: error reading directory: %"
+				PRIuINUM "\n", fs_dent->inode);
+			    tsk_error_print(stderr);
+			}
+			tsk_error_reset();
+		    }
+
+		    dinfo->depth--;
+		    dinfo->curdir_inode = dinfo->pardir_inode;
+		    dinfo->pardir_inode = back_p;
+
+		    if (depth_added)
+			*dinfo->didx[dinfo->depth] = '\0';
+
+		    /* Restore the recursion setting */
+		    if (myflags & FS_FLAG_NAME_ALLOC) {
+			dinfo->recdel = back_recdel;
+		    }
 		}
 	    }
 	}
     }
+    fs_dent_free(fs_dent);
 
     return 0;
 }
@@ -687,15 +703,33 @@ fatfs_dent_walk(FS_INFO * fs, INUM_T inode, int flags,
     FS_DENT_WALK_FN action, void *ptr)
 {
     FATFS_DINFO dinfo;
+    TSK_LIST *list_seen = NULL;
+    uint8_t retval;
+
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
+    /* Sanity check on flags -- make sure at least one ALLOC is set */
+    if (((flags & FS_FLAG_NAME_ALLOC) == 0) &&
+	((flags & FS_FLAG_NAME_UNALLOC) == 0)) {
+	flags |= (FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC);
+    }
 
     memset(&dinfo, 0, sizeof(FATFS_DINFO));
-    return fatfs_dent_walk_lcl(fs, &dinfo, inode, flags, action, ptr);
+    retval =
+	fatfs_dent_walk_lcl(fs, &dinfo, &list_seen, inode, flags, action,
+	ptr);
+
+    tsk_list_free(list_seen);
+    list_seen = NULL;
+    return retval;
 }
 
 
 static uint8_t
-fatfs_dent_walk_lcl(FS_INFO * fs, FATFS_DINFO * dinfo, INUM_T inode,
-    int flags, FS_DENT_WALK_FN action, void *ptr)
+fatfs_dent_walk_lcl(FS_INFO * fs, FATFS_DINFO * dinfo,
+    TSK_LIST ** list_seen, INUM_T inode, int flags, FS_DENT_WALK_FN action,
+    void *ptr)
 {
     OFF_T size, len;
     FS_INODE *fs_inode;
@@ -706,6 +740,7 @@ fatfs_dent_walk_lcl(FS_INFO * fs, FATFS_DINFO * dinfo, INUM_T inode,
     int retval;
 
     if ((inode < fs->first_inum) || (inode > fs->last_inum)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "fatfs_dent_walk: invalid inode value: %" PRIuINUM "\n",
@@ -715,6 +750,7 @@ fatfs_dent_walk_lcl(FS_INFO * fs, FATFS_DINFO * dinfo, INUM_T inode,
 
     fs_inode = fs->inode_lookup(fs, inode);
     if (!fs_inode) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_INODE_NUM;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "fatfs_dent_walk: %" PRIuINUM " is not a valid inode", inode);
@@ -768,11 +804,10 @@ fatfs_dent_walk_lcl(FS_INFO * fs, FATFS_DINFO * dinfo, INUM_T inode,
 
     /* We did not copy the entire directory, which occurs if an error occured */
     if (load.dirleft > 0) {
-
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_FWALK;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "fatfs_dent_walk: Error reading directory %" PRIuINUM, inode);
-	tsk_errstr2[0] = '\0';
 
 	/* Free the local buffers */
 	free(load.curaddrbuf);
@@ -785,8 +820,8 @@ fatfs_dent_walk_lcl(FS_INFO * fs, FATFS_DINFO * dinfo, INUM_T inode,
     /* Reset the local pointer because we could have done a realloc */
     addrbuf = load.curaddrbuf;
     retval =
-	fatfs_dent_parse_buf(fatfs, dinfo, dirbuf, len, addrbuf, flags,
-	action, ptr);
+	fatfs_dent_parse_buf(fatfs, dinfo, list_seen, dirbuf, len, addrbuf,
+	flags, action, ptr);
 
 
     fs_inode_free(fs_inode);
