@@ -2,7 +2,7 @@
 ** ffind  (file find)
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/09/20 20:16:01 $
 **
 ** Find the file that uses the specified inode (including deleted files)
 ** 
@@ -23,13 +23,13 @@
 #include <locale.h>
 #include "fs_tools.h"
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 void
 usage()
 {
     TFPRINTF(stderr,
-	_T
+	_TSK_T
 	("usage: %s [-aduvV] [-f fstype] [-i imgtype] [-o imgoffset] image [images] inode\n"),
 	progname);
     tsk_fprintf(stderr, "\t-a: Find all occurrences\n");
@@ -49,10 +49,10 @@ usage()
 
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
-    TCHAR *fstype = NULL;
-    TCHAR *imgtype = NULL;
+    TSK_TCHAR *fstype = NULL;
+    TSK_TCHAR *imgtype = NULL;
     int flags = FS_FLAG_NAME_RECURSE;
     int ch;
     FS_INFO *fs;
@@ -68,46 +68,47 @@ MAIN(int argc, TCHAR ** argv)
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, _T("adf:i:o:uvV"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("adf:i:o:uvV"))) > 0) {
 	switch (ch) {
-	case _T('a'):
+	case _TSK_T('a'):
 	    localflags |= FFIND_ALL;
 	    break;
-	case _T('d'):
+	case _TSK_T('d'):
 	    flags |= FS_FLAG_NAME_UNALLOC;
 	    break;
-	case _T('f'):
+	case _TSK_T('f'):
 	    fstype = optarg;
-	    if (TSTRCMP(fstype, _T("list")) == 0) {
+	    if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
 		fs_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('u'):
+	case _TSK_T('u'):
 	    flags |= FS_FLAG_NAME_ALLOC;
 	    break;
-	case _T('v'):
+	case _TSK_T('v'):
 	    verbose++;
 	    break;
-	case _T('V'):
+	case _TSK_T('V'):
 	    print_version(stdout);
 	    exit(0);
-	case _T('?'):
+	case _TSK_T('?'):
 	default:
-	    TFPRINTF(stderr, _T("Invalid argument: %s\n"), argv[optind]);
+	    TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+		argv[optind]);
 	    usage();
 	}
     }
@@ -128,7 +129,7 @@ MAIN(int argc, TCHAR ** argv)
 
     /* Get the inode */
     if (parse_inum(argv[argc - 1], &inode, &type, &id, &id_used)) {
-	TFPRINTF(stderr, _T("Invalid inode: %s\n"), argv[argc - 1]);
+	TFPRINTF(stderr, _TSK_T("Invalid inode: %s\n"), argv[argc - 1]);
 	usage();
     }
     if (id_used == 0)
@@ -137,7 +138,7 @@ MAIN(int argc, TCHAR ** argv)
     /* open image */
     if ((img =
 	    img_open(imgtype, argc - optind - 1,
-		(const TCHAR **) &argv[optind])) == NULL) {
+		(const TSK_TCHAR **) &argv[optind])) == NULL) {
 	tsk_error_print(stderr);
 	exit(1);
     }
