@@ -2,7 +2,7 @@
 ** ffind  (file find)
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/29 17:13:11 $
+** $Date: 2006/09/21 16:47:06 $
 **
 ** Find the file that uses the specified inode (including deleted files)
 ** 
@@ -99,7 +99,7 @@ fs_ffind(FS_INFO * fs, uint8_t lclflags, INUM_T inode_a, uint32_t type,
 		fs_inode_free(fs_inode);
 	}
 	else {
-	    tsk_printf("inode not currently used\n");
+	    tsk_printf("File name not found for inode\n");
 	}
     }
     return 0;
