@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/12/05 21:39:52 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -43,11 +43,11 @@ ffs_group_load(FFS_INFO * ffs, FFS_GRPNUM_T grp_num)
      * Sanity check
      */
     if (grp_num < 0 || grp_num >= ffs->groups_count) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ffs_group_load: invalid cylinder group number: %"
 	    PRI_FFSGRP "", grp_num);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -67,8 +67,8 @@ ffs_group_load(FFS_INFO * ffs, FFS_GRPNUM_T grp_num)
 	cnt = fs_read_block(fs, ffs->grp_buf, ffs->grp_buf->size, addr);
 	if (cnt != ffs->grp_buf->size) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_FS_READ;
-		tsk_errstr[0] = '\0';
 	    }
 	    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		"ffs_group_load: Group %" PRI_FFSGRP " at %" PRIuDADDR,
@@ -98,10 +98,10 @@ ffs_dinode_load(FFS_INFO * ffs, INUM_T inum)
      * Sanity check.
      */
     if (inum < fs->first_inum || inum > fs->last_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_INODE_NUM;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ffs_dinode_load: address: %" PRIuINUM, inum);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -161,8 +161,8 @@ ffs_dinode_load(FFS_INFO * ffs, INUM_T inum)
 		    (fs, ffs->itbl_buf, ffs->itbl_buf->size, addr);
 		if (cnt != ffs->itbl_buf->size) {
 		    if (cnt != -1) {
+			tsk_error_reset();
 			tsk_errno = TSK_ERR_FS_READ;
-			tsk_errstr[0] = '\0';
 		    }
 		    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			"ffs_dinode_load: FFS2 inode table at %"
@@ -195,8 +195,8 @@ ffs_dinode_load(FFS_INFO * ffs, INUM_T inum)
 		addr);
 	    if (cnt != ffs->itbl_buf->size) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "ffs_dinode_load: FFS1 inode table at %"
@@ -338,8 +338,8 @@ ffs_dinode_copy(FFS_INFO * ffs, FS_INODE * fs_inode)
 			fs_inode->direct_addr[i]);
 		    if (cnt != fs->block_size) {
 			if (cnt != -1) {
+			    tsk_error_reset();
 			    tsk_errno = TSK_ERR_FS_READ;
-			    tsk_errstr[0] = '\0';
 			}
 			snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			    "ffs_dinode_copy: FFS1A symlink dest at %"
@@ -459,8 +459,8 @@ ffs_dinode_copy(FFS_INFO * ffs, FS_INODE * fs_inode)
 			fs_inode->direct_addr[i]);
 		    if (cnt != fs->block_size) {
 			if (cnt != -1) {
+			    tsk_error_reset();
 			    tsk_errno = TSK_ERR_FS_READ;
-			    tsk_errstr[0] = '\0';
 			}
 			snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			    "ffs_dinode_copy: FFS1B symlink dest at %"
@@ -578,8 +578,8 @@ ffs_dinode_copy(FFS_INFO * ffs, FS_INODE * fs_inode)
 			fs_inode->direct_addr[i]);
 		    if (cnt != fs->block_size) {
 			if (cnt != -1) {
+			    tsk_error_reset();
 			    tsk_errno = TSK_ERR_FS_READ;
-			    tsk_errstr[0] = '\0';
 			}
 			snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			    "ffs_dinode_copy: FFS2 symlink dest at %"
@@ -600,10 +600,10 @@ ffs_dinode_copy(FFS_INFO * ffs, FS_INODE * fs_inode)
 	}			/* end of symlink */
     }
     else {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ffs_dinode_copy: Unknown FFS Type");
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -623,10 +623,6 @@ ffs_dinode_copy(FFS_INFO * ffs, FS_INODE * fs_inode)
     fs_inode->flags = (isset(inosused, ffs->dino_inum - ibase) ?
 	FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 
-    /* link flag */
-    fs_inode->flags |= (fs_inode->nlink ?
-	FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
-
     /* used/unused */
     fs_inode->flags |= (fs_inode->ctime ?
 	FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
@@ -647,6 +643,9 @@ ffs_inode_lookup(FS_INFO * fs, INUM_T inum)
     FFS_INFO *ffs = (FFS_INFO *) fs;
     FS_INODE *fs_inode;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     /* Lookup the inode and store it in ffs */
     if (ffs_dinode_load(ffs, inum))
 	return NULL;
@@ -672,11 +671,23 @@ ffs_inode_lookup(FS_INFO * fs, INUM_T inum)
  *
  **************************************************************************/
 
+
+static uint8_t
+inode_walk_dent_orphan_act(FS_INFO * fs, FS_DENT * fs_dent, int flags,
+    void *ptr)
+{
+    if ((fs_dent->fsi) && (fs_dent->fsi->flags & FS_FLAG_META_UNALLOC)) {
+	if (tsk_list_add(&fs->list_inum_named, fs_dent->fsi->addr))
+	    return WALK_STOP;
+    }
+    return WALK_CONT;
+}
+
+
 /* ffs_inode_walk - inode iterator 
  *
  * flags used: FS_FLAG_META_USED, FS_FLAG_META_UNUSED, 
- *  FS_FLAG_META_LINK, FS_FLAG_META_UNLINK, 
- *  FS_FLAG_META_ALLOC, FS_FLAG_META_UNALLOC
+ *  FS_FLAG_META_ALLOC, FS_FLAG_META_UNALLOC, FS_FLAG_META_ORPHAN
  *
  *  return 1 on error and 0 on success
  */
@@ -694,26 +705,65 @@ ffs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum, int flags,
     int myflags;
     INUM_T ibase = 0;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
 
     /*
      * Sanity checks.
      */
     if (start_inum < fs->first_inum || start_inum > fs->last_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: Start inode: %" PRIuINUM "", myname, start_inum);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
     else if (end_inum < fs->first_inum || end_inum > fs->last_inum
 	|| end_inum < start_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: End inode: %" PRIuINUM "", myname, end_inum);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
+    /* If ORPHAN is wanted, then make sure that the flags are correct */
+    if (flags & FS_FLAG_META_ORPHAN) {
+	flags |= FS_FLAG_META_UNALLOC;
+	flags &= ~FS_FLAG_META_ALLOC;
+    }
+
+    else if (((flags & FS_FLAG_META_ALLOC) == 0) &&
+	((flags & FS_FLAG_META_UNALLOC) == 0)) {
+	flags |= (FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC);
+    }
+
+    /* If neither of the USED or UNUSED flags are set, then set them
+     * both
+     */
+    if (((flags & FS_FLAG_META_USED) == 0) &&
+	((flags & FS_FLAG_META_UNUSED) == 0)) {
+	flags |= (FS_FLAG_META_USED | FS_FLAG_META_UNUSED);
+    }
+
+
+
+    /* If we are looking for orphan files and have not yet filled
+     * in the list of unalloc inodes that are pointed to, then fill
+     * in the list
+     * */
+    if ((flags & FS_FLAG_META_ORPHAN) && (fs->list_inum_named == NULL)) {
+
+	if (ffs_dent_walk(fs, fs->root_inum,
+		FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC |
+		FS_FLAG_NAME_RECURSE, inode_walk_dent_orphan_act, NULL)) {
+	    strncat(tsk_errstr2,
+		" - ffs_inode_walk: identifying inodes allocated by file names",
+		TSK_ERRSTR_L);
+	    return 1;
+	}
+    }
+
     if ((fs_inode = fs_inode_alloc(FFS_NDADDR, FFS_NIADDR)) == NULL)
 	return 1;
 
@@ -761,14 +811,6 @@ ffs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum, int flags,
 	    /* both inode forms are the same for the required fields */
 	    ffs_inode1 *in1 = (ffs_inode1 *) ffs->dino_buf;
 
-	    /*
-	     * Apply the linked/unlinked restriction.
-	     */
-	    myflags |= (gets16(fs->endian, in1->di_nlink) ?
-		FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
-	    if ((flags & myflags) != myflags)
-		continue;
-
 	    /*
 	     * Apply the used/unused restriction.
 	     */
@@ -780,14 +822,6 @@ ffs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum, int flags,
 	else {
 	    ffs_inode2 *in2 = (ffs_inode2 *) ffs->dino_buf;
 
-	    /*
-	     * Apply the linked/unlinked restriction.
-	     */
-	    myflags |= (gets16(fs->endian, in2->di_nlink) ?
-		FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
-	    if ((flags & myflags) != myflags)
-		continue;
-
 	    /*
 	     * Apply the used/unused restriction.
 	     */
@@ -797,6 +831,16 @@ ffs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum, int flags,
 		continue;
 	}
 
+	/* If we want only orphans, then check if this
+	 * inode is in the seen list
+	 */
+	if ((myflags & FS_FLAG_META_UNALLOC) &&
+	    (flags & FS_FLAG_META_ORPHAN) &&
+	    (tsk_list_find(fs->list_inum_named, inum))) {
+	    continue;
+	}
+
+
 	/*
 	 * Fill in a file system-independent inode structure and pass control
 	 * to the application.
@@ -860,34 +904,44 @@ ffs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk, int flags,
     int frags;
     char *null_block = NULL;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     /*
      * Sanity checks.
      */
     if (start_blk < fs->first_block || start_blk > fs->last_block) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: Start block: %" PRIuDADDR "", myname, start_blk);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
     if (end_blk < fs->first_block || end_blk > fs->last_block
 	|| end_blk < start_blk) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: End block: %" PRIuDADDR "", myname, end_blk);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
     if ((flags & FS_FLAG_DATA_ALIGN) && (start_blk % ffs->ffsbsize_f) != 0) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: start block is not block-aligned", myname);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
+    /* Sanity check on flags -- make sure at least one ALLOC is set */
+    if (((flags & FS_FLAG_DATA_ALLOC) == 0) &&
+	((flags & FS_FLAG_DATA_UNALLOC) == 0)) {
+	flags |= (FS_FLAG_DATA_ALLOC | FS_FLAG_DATA_UNALLOC);
+    }
+
+
     /*
      * Other initialization.
      */
@@ -1009,8 +1063,8 @@ ffs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk, int flags,
 			addr);
 		    if (cnt != fs->block_size * frags) {
 			if (cnt != -1) {
+			    tsk_error_reset();
 			    tsk_errno = TSK_ERR_FS_READ;
-			    tsk_errstr[0] = '\0';
 			}
 			snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			    "ffs_block_walk: Block %" PRIuDADDR, addr);
@@ -1069,6 +1123,7 @@ ffs_file_walk_direct(FS_INFO * fs, DATA_BUF * buf,
     read_count = (length < buf->size ? (size_t) length : buf->size);
 
     if (addr > fs->last_block) {
+	tsk_error_reset();
 	if (flags & FS_FLAG_FILE_RECOVER)
 	    tsk_errno = TSK_ERR_FS_RECOVER;
 	else
@@ -1076,7 +1131,6 @@ ffs_file_walk_direct(FS_INFO * fs, DATA_BUF * buf,
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ffs: Invalid direct address (too large): %"
 	    PRIuDADDR "", addr);
-	tsk_errstr2[0] = '\0';
 	return -1;
     }
 
@@ -1093,7 +1147,9 @@ ffs_file_walk_direct(FS_INFO * fs, DATA_BUF * buf,
     if (addr == 0) {
 	if (0 == (flags & FS_FLAG_FILE_NOSPARSE)) {
 	    int retval;
-	    myflags = FS_FLAG_DATA_CONT;
+	    myflags =
+		FS_FLAG_DATA_CONT | FS_FLAG_DATA_ALLOC |
+		FS_FLAG_DATA_SPARSE;
 
 	    if ((flags & FS_FLAG_FILE_AONLY) == 0)
 		memset(buf->data, 0, (size_t) read_count);
@@ -1116,8 +1172,8 @@ ffs_file_walk_direct(FS_INFO * fs, DATA_BUF * buf,
 		addr);
 	    if (cnt != (SSIZE_T) roundup(read_count, FFS_DEV_BSIZE)) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "ffs_file_walk_direct: Block %" PRIuDADDR, addr);
@@ -1154,13 +1210,13 @@ ffs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 	    level, addr);
 
     if (addr > fs->last_block) {
+	tsk_error_reset();
 	if (flags & FS_FLAG_FILE_RECOVER)
 	    tsk_errno = TSK_ERR_FS_RECOVER;
 	else
 	    tsk_errno = TSK_ERR_FS_INODE_INT;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ffs: Indirect block address too large: %" PRIuDADDR "", addr);
-	tsk_errstr2[0] = '\0';
 	return -1;
     }
 
@@ -1175,8 +1231,8 @@ ffs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 	cnt = fs_read_block(fs, buf[level], buf[level]->size, addr);
 	if (cnt != buf[level]->size) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_FS_READ;
-		tsk_errstr[0] = '\0';
 	    }
 	    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		"ffs_file_walk_indir: Block %" PRIuDADDR, addr);
@@ -1213,6 +1269,7 @@ ffs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 
 	    if (getu32(fs->endian,
 		    (uint8_t *) & iaddr[n]) > fs->last_block) {
+		tsk_error_reset();
 		if (flags & FS_FLAG_FILE_RECOVER)
 		    tsk_errno = TSK_ERR_FS_RECOVER;
 		else
@@ -1248,6 +1305,7 @@ ffs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 
 	    if (getu64(fs->endian,
 		    (uint8_t *) & iaddr[n]) > fs->last_block) {
+		tsk_error_reset();
 		if (flags & FS_FLAG_FILE_RECOVER)
 		    tsk_errno = TSK_ERR_FS_RECOVER;
 		else
@@ -1256,7 +1314,6 @@ ffs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 		    "ffs: Address in indirect block too large: %"
 		    PRIuDADDR "", getu64(fs->endian,
 			(uint8_t *) & iaddr[n]));
-		tsk_errstr2[0] = '\0';
 		return -1;
 	    }
 
@@ -1307,6 +1364,9 @@ ffs_file_walk(FS_INFO * fs, FS_INODE * inode, uint32_t type, uint16_t id,
     int n;
     int level;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if (verbose)
 	tsk_fprintf(stderr,
 	    "ffs_file_walk: Processing file %" PRIuINUM "\n", inode->addr);
@@ -1400,10 +1460,10 @@ ffs_file_walk(FS_INFO * fs, FS_INODE * inode, uint32_t type, uint16_t id,
 static uint8_t
 ffs_fscheck(FS_INFO * fs, FILE * hFile)
 {
+    tsk_error_reset();
     tsk_errno = TSK_ERR_FS_FUNC;
     snprintf(tsk_errstr, TSK_ERRSTR_L,
 	"fscheck not implemented for ffs yet");
-    tsk_errstr2[0] = '\0';
     return 1;
 }
 
@@ -1424,6 +1484,9 @@ ffs_fsstat(FS_INFO * fs, FILE * hFile)
     ffs_sb2 *sb2 = ffs->fs.sb2;
     int flags;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     tsk_fprintf(hFile, "FILE SYSTEM INFORMATION\n");
     tsk_fprintf(hFile, "--------------------------------------------\n");
 
@@ -1551,8 +1614,8 @@ ffs_fsstat(FS_INFO * fs, FILE * hFile)
 
 	    if (cnt != getu32(fs->endian, sb1->cg_ssize_b)) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "ffs_fsstat: FFS1 group descriptor at %"
@@ -1566,8 +1629,8 @@ ffs_fsstat(FS_INFO * fs, FILE * hFile)
 		(DADDR_T) getu64(fs->endian, sb2->cg_saddr));
 	    if (cnt != getu32(fs->endian, sb2->cg_ssize_b)) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "ffs_fsstat: FFS2 group descriptor at %"
@@ -1742,7 +1805,8 @@ print_addr_act(FS_INFO * fs, DADDR_T addr, char *buf,
     FFS_PRINT_ADDR *print = (FFS_PRINT_ADDR *) ptr;
 
     if (flags & FS_FLAG_DATA_CONT) {
-	int i, s;
+	int i;
+	size_t s;
 	/* cycle through the fragments if they exist */
 	for (i = 0, s = size; s > 0; s -= fs->block_size, i++) {
 
@@ -1780,6 +1844,9 @@ ffs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
     char ls[12];
     FFS_PRINT_ADDR print;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     fs_inode = ffs_inode_lookup(fs, inum);
     if (fs_inode == NULL)
 	return 1;
@@ -1861,8 +1928,8 @@ ffs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
 		    getu64(fs->endian, in->di_extb[0]));
 		if (cnt != ffs->ffsbsize_b) {
 		    if (cnt != -1) {
+			tsk_error_reset();
 			tsk_errno = TSK_ERR_FS_READ;
-			tsk_errstr[0] = '\0';
 		    }
 		    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			"ffs_istat: FFS2 extended attribute 0 at %"
@@ -1900,8 +1967,8 @@ ffs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
 		    getu64(fs->endian, in->di_extb[1]));
 		if (cnt != ffs->ffsbsize_b) {
 		    if (cnt != -1) {
+			tsk_error_reset();
 			tsk_errno = TSK_ERR_FS_FUNC;
-			tsk_errstr[0] = '\0';
 		    }
 		    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			"ffs_istat: FFS2 extended attribute 1 at %"
@@ -1983,9 +2050,9 @@ ffs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
 uint8_t
 ffs_jopen(FS_INFO * fs, INUM_T inum)
 {
+    tsk_error_reset();
     tsk_errno = TSK_ERR_FS_FUNC;
     snprintf(tsk_errstr, TSK_ERRSTR_L, "UFS does not have a journal");
-    tsk_errstr2[0] = '\0';
     return 1;
 }
 
@@ -1993,9 +2060,9 @@ uint8_t
 ffs_jentry_walk(FS_INFO * fs, int flags, FS_JENTRY_WALK_FN action,
     void *ptr)
 {
+    tsk_error_reset();
     tsk_errno = TSK_ERR_FS_FUNC;
     snprintf(tsk_errstr, TSK_ERRSTR_L, "UFS does not have a journal");
-    tsk_errstr2[0] = '\0';
     return 1;
 }
 
@@ -2004,9 +2071,9 @@ uint8_t
 ffs_jblk_walk(FS_INFO * fs, DADDR_T start, DADDR_T end, int flags,
     FS_JBLK_WALK_FN action, void *ptr)
 {
+    tsk_error_reset();
     tsk_errno = TSK_ERR_FS_FUNC;
     snprintf(tsk_errstr, TSK_ERRSTR_L, "UFS does not have a journal");
-    tsk_errstr2[0] = '\0';
     return 1;
 }
 
@@ -2027,6 +2094,11 @@ ffs_close(FS_INFO * fs)
     if (ffs->dino_buf)
 	free(ffs->dino_buf);
 
+    if (fs->list_inum_named) {
+	tsk_list_free(fs->list_inum_named);
+	fs->list_inum_named = NULL;
+    }
+
     free((char *) ffs->fs.sb1);
     free(ffs);
 }
@@ -2042,10 +2114,13 @@ ffs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype)
     FS_INFO *fs;
     SSIZE_T cnt;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if ((ftype & FSMASK) != FFS_TYPE) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L, "Invalid FS Type in ffs_open");
-	tsk_errstr2[0] = '\0';
 	return NULL;
     }
 
@@ -2078,8 +2153,8 @@ ffs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype)
 	(fs, (char *) ffs->fs.sb2, sizeof(ffs_sb2), (OFF_T) UFS2_SBOFF);
     if (cnt != sizeof(ffs_sb2)) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_READ;
-	    tsk_errstr2[0] = '\0';
 	}
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: Superblock at %" PRIuDADDR, myname, (OFF_T) UFS2_SBOFF);
@@ -2095,8 +2170,8 @@ ffs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype)
 	    (OFF_T) UFS2_SBOFF2);
 	if (cnt != sizeof(ffs_sb2)) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_FS_READ;
-		tsk_errstr[0] = '\0';
 	    }
 	    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		"%s: Superblock at %" PRIuDADDR,
@@ -2112,8 +2187,8 @@ ffs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype)
 		(fs, (char *) ffs->fs.sb1, len, (OFF_T) UFS1_SBOFF);
 	    if (cnt != len) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "%s: Superblock at %" PRIuDADDR,
@@ -2125,9 +2200,9 @@ ffs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype)
 	    if (fs_guessu32(fs, ffs->fs.sb1->magic, UFS1_FS_MAGIC)) {
 		free(ffs->fs.sb1);
 		free(ffs);
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_FS_MAGIC;
 		snprintf(tsk_errstr, TSK_ERRSTR_L, "No UFS Magic Found");
-		tsk_errstr2[0] = '\0';
 		return NULL;
 	    }
 	    else {
@@ -2172,20 +2247,20 @@ ffs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype)
     if ((fs->block_size % 512) || (ffs->ffsbsize_b % 512)) {
 	free(ffs->fs.sb1);
 	free(ffs);
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Not a UFS FS (invalid fragment or block size)");
-	tsk_errstr2[0] = '\0';
 	return NULL;
     }
 
     if ((ffs->ffsbsize_b / fs->block_size) != ffs->ffsbsize_f) {
 	free(ffs->fs.sb1);
 	free(ffs);
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Not a UFS FS (frag / block size mismatch)");
-	tsk_errstr2[0] = '\0';
 	return NULL;
     }
 
@@ -2231,6 +2306,8 @@ ffs_open(IMG_INFO * img_info, SSIZE_T offset, uint8_t ftype)
     fs->jopen = ffs_jopen;
     fs->journ_inum = 0;
 
+    fs->list_inum_named = NULL;
+
     /*
      * Print some stats.
      */
