@@ -2,7 +2,7 @@
 ** ffs_dent
 ** The  Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/11/29 22:02:10 $
 **
 ** File name layer for a FFS/UFS image 
 **
@@ -43,8 +43,8 @@ typedef struct {
 
 } FFS_DINFO;
 
-static uint8_t ffs_dent_walk_lcl(FS_INFO *, FFS_DINFO *, INUM_T, int,
-    FS_DENT_WALK_FN, void *);
+static uint8_t ffs_dent_walk_lcl(FS_INFO *, FFS_DINFO *, TSK_LIST **,
+    INUM_T, int, FS_DENT_WALK_FN, void *);
 
 
 /* 
@@ -95,10 +95,10 @@ ffs_dent_copy(FFS_INFO * ffs, FFS_DINFO * dinfo, char *ffs_dent,
 	fs_dent->ent_type = FS_DENT_UNDEF;
     }
     else {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ffs_dent_copy: Unknown FS type");
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -146,8 +146,9 @@ ffs_dent_copy(FFS_INFO * ffs, FFS_DINFO * dinfo, char *ffs_dent,
 ** return 0 on success, -1 on error, and 1 to stop 
 */
 static int
-ffs_dent_parse_block(FFS_INFO * ffs, FFS_DINFO * dinfo, char *buf,
-    unsigned int len, int flags, FS_DENT_WALK_FN action, void *ptr)
+ffs_dent_parse_block(FFS_INFO * ffs, FFS_DINFO * dinfo,
+    TSK_LIST ** list_seen, char *buf, unsigned int len, int flags,
+    FS_DENT_WALK_FN action, void *ptr)
 {
     unsigned int idx;
     unsigned int inode = 0, dellen = 0, reclen = 0;
@@ -271,36 +272,44 @@ ffs_dent_parse_block(FFS_INFO * ffs, FFS_DINFO * dinfo, char *buf,
 	    ((fs_dent->fsi->mode & FS_INODE_FMT) == FS_INODE_DIR)) {
 	    int depth_added = 0;
 
-	    /* save the path */
-	    if ((dinfo->depth < MAX_DEPTH) &&
-		(DIR_STRSZ >
-		    strlen(dinfo->dirs) + strlen(fs_dent->name))) {
-		dinfo->didx[dinfo->depth] =
-		    &dinfo->dirs[strlen(dinfo->dirs)];
-		strncpy(dinfo->didx[dinfo->depth], fs_dent->name,
-		    DIR_STRSZ - strlen(dinfo->dirs));
-		strncat(dinfo->dirs, "/", DIR_STRSZ);
-		depth_added = 1;
-	    }
-	    dinfo->depth++;
-
-	    /* Call ourselves again */
-	    if (ffs_dent_walk_lcl(&(ffs->fs_info), dinfo, fs_dent->inode,
-		    flags, action, ptr)) {
-		/* If the directory could not be loaded, 
-		 * then move on */
-		if (verbose) {
-		    tsk_fprintf(stderr,
-			"ffs_dent_parse_block: error reading directory: %"
-			PRIuINUM "\n", fs_dent->inode);
-		    tsk_error_print(stderr);
+	    /* Make sure we do not get into an infinite loop */
+	    if (0 == tsk_list_find(*list_seen, fs_dent->inode)) {
+		if (tsk_list_add(list_seen, fs_dent->inode)) {
+		    fs_dent_free(fs_dent);
+		    return -1;
 		}
-		tsk_error_reset();
-	    }
 
-	    dinfo->depth--;
-	    if (depth_added)
-		*dinfo->didx[dinfo->depth] = '\0';
+		/* save the path */
+		if ((dinfo->depth < MAX_DEPTH) &&
+		    (DIR_STRSZ >
+			strlen(dinfo->dirs) + strlen(fs_dent->name))) {
+		    dinfo->didx[dinfo->depth] =
+			&dinfo->dirs[strlen(dinfo->dirs)];
+		    strncpy(dinfo->didx[dinfo->depth], fs_dent->name,
+			DIR_STRSZ - strlen(dinfo->dirs));
+		    strncat(dinfo->dirs, "/", DIR_STRSZ);
+		    depth_added = 1;
+		}
+		dinfo->depth++;
+
+		/* Call ourselves again */
+		if (ffs_dent_walk_lcl(&(ffs->fs_info), dinfo, list_seen,
+			fs_dent->inode, flags, action, ptr)) {
+		    /* If the directory could not be loaded, 
+		     * then move on */
+		    if (verbose) {
+			tsk_fprintf(stderr,
+			    "ffs_dent_parse_block: error reading directory: %"
+			    PRIuINUM "\n", fs_dent->inode);
+			tsk_error_print(stderr);
+		    }
+		    tsk_error_reset();
+		}
+
+		dinfo->depth--;
+		if (depth_added)
+		    *dinfo->didx[dinfo->depth] = '\0';
+	    }
 	}
 
     }				/* end for size */
@@ -326,15 +335,32 @@ ffs_dent_walk(FS_INFO * fs, INUM_T inode, int flags,
     FS_DENT_WALK_FN action, void *ptr)
 {
     FFS_DINFO dinfo;
+    TSK_LIST *list_seen = NULL;
+    int retval;
+
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     memset(&dinfo, 0, sizeof(FFS_DINFO));
+    /* Sanity check on flags -- make sure at least one ALLOC is set */
+    if (((flags & FS_FLAG_NAME_ALLOC) == 0) &&
+	((flags & FS_FLAG_NAME_UNALLOC) == 0)) {
+	flags |= (FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC);
+    }
+
+    retval =
+	ffs_dent_walk_lcl(fs, &dinfo, &list_seen, inode, flags, action,
+	ptr);
+    tsk_list_free(list_seen);
+    list_seen = NULL;
+    return retval;
 
-    return ffs_dent_walk_lcl(fs, &dinfo, inode, flags, action, ptr);
 }
 
 /* Return 0 on success and 1 on error */
 static uint8_t
-ffs_dent_walk_lcl(FS_INFO * fs, FFS_DINFO * dinfo, INUM_T inode, int flags,
-    FS_DENT_WALK_FN action, void *ptr)
+ffs_dent_walk_lcl(FS_INFO * fs, FFS_DINFO * dinfo, TSK_LIST ** list_seen,
+    INUM_T inode, int flags, FS_DENT_WALK_FN action, void *ptr)
 {
     OFF_T size;
     FS_INODE *fs_inode;
@@ -345,10 +371,10 @@ ffs_dent_walk_lcl(FS_INFO * fs, FFS_DINFO * dinfo, INUM_T inode, int flags,
     int retval = 0;
 
     if (inode < fs->first_inum || inode > fs->last_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ffs_dent_walk_lcl: Invalid inode value: %" PRIuINUM, inode);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -385,10 +411,10 @@ ffs_dent_walk_lcl(FS_INFO * fs, FFS_DINFO * dinfo, INUM_T inode, int flags,
 
     /* Not all of the directory was copied, so we return */
     if (load_file.left > 0) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_FWALK;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ffs_dent_walk: Error reading directory %" PRIuINUM, inode);
-	tsk_errstr2[0] = '\0';
 	free(dirbuf);
 	fs_inode_free(fs_inode);
 	return 1;
@@ -406,8 +432,8 @@ ffs_dent_walk_lcl(FS_INFO * fs, FFS_DINFO * dinfo, INUM_T inode, int flags,
 	int len = (FFS_DIRBLKSIZ < size) ? FFS_DIRBLKSIZ : (int) size;
 
 	retval =
-	    ffs_dent_parse_block(ffs, dinfo, dirbuf + cidx * FFS_DIRBLKSIZ,
-	    len, flags, action, ptr);
+	    ffs_dent_parse_block(ffs, dinfo, list_seen,
+	    dirbuf + cidx * FFS_DIRBLKSIZ, len, flags, action, ptr);
 
 	/* one is returned when the action wants to stop */
 	if ((retval == 1) || (retval == -1))
