@@ -2,7 +2,7 @@
 ** fls
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/09/20 20:16:01 $
 **
 ** Given an image and directory inode, display the file names and 
 ** directories that exist (both active and deleted)
@@ -25,13 +25,13 @@
 #include <time.h>
 #include "fs_tools.h"
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 void
 usage()
 {
     TFPRINTF(stderr,
-	_T
+	_TSK_T
 	("usage: %s [-adDFlpruvV] [-f fstype] [-i imgtype] [-m dir/] [-o imgoffset] [-z ZONE] [-s seconds] image [images] [inode]\n"),
 	progname);
     tsk_fprintf(stderr,
@@ -65,10 +65,10 @@ usage()
 }
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
-    TCHAR *fstype = NULL;
-    TCHAR *imgtype = NULL;
+    TSK_TCHAR *fstype = NULL;
+    TSK_TCHAR *imgtype = NULL;
     INUM_T inode;
     int flags = FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC;
     int ch;
@@ -77,7 +77,7 @@ MAIN(int argc, TCHAR ** argv)
     IMG_INFO *img;
     int lclflags;
     int32_t sec_skew = 0;
-    static TCHAR *macpre = NULL;
+    static TSK_TCHAR *macpre = NULL;
     SSIZE_T imgoff = 0;
 
     progname = argv[0];
@@ -85,76 +85,77 @@ MAIN(int argc, TCHAR ** argv)
 
     lclflags = FLS_DIR | FLS_FILE;
 
-    while ((ch = getopt(argc, argv, _T("adDf:Fi:m:lo:prs:uvVz:"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("adDf:Fi:m:lo:prs:uvVz:"))) > 0) {
 	switch (ch) {
-	case _T('?'):
+	case _TSK_T('?'):
 	default:
-	    TFPRINTF(stderr, _T("Invalid argument: %s\n"), argv[optind]);
+	    TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+		argv[optind]);
 	    usage();
-	case _T('a'):
+	case _TSK_T('a'):
 	    lclflags |= FLS_DOT;
 	    break;
-	case _T('d'):
+	case _TSK_T('d'):
 	    flags &= ~FS_FLAG_NAME_ALLOC;
 	    break;
-	case _T('D'):
+	case _TSK_T('D'):
 	    lclflags &= ~FLS_FILE;
 	    lclflags |= FLS_DIR;
 	    break;
-	case _T('f'):
+	case _TSK_T('f'):
 	    fstype = optarg;
-	    if (TSTRCMP(fstype, _T("list")) == 0) {
+	    if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
 		fs_print_types(stderr);
 		exit(1);
 	    }
 
 	    break;
-	case _T('F'):
+	case _TSK_T('F'):
 	    lclflags &= ~FLS_DIR;
 	    lclflags |= FLS_FILE;
 	    break;
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('l'):
+	case _TSK_T('l'):
 	    lclflags |= FLS_LONG;
 	    break;
-	case _T('m'):
+	case _TSK_T('m'):
 	    lclflags |= FLS_MAC;
 	    macpre = optarg;
 	    break;
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('p'):
+	case _TSK_T('p'):
 	    lclflags |= FLS_FULL;
 	    break;
-	case _T('r'):
+	case _TSK_T('r'):
 	    flags |= FS_FLAG_NAME_RECURSE;
 	    break;
-	case _T('s'):
+	case _TSK_T('s'):
 	    sec_skew = TATOI(optarg);
 	    break;
-	case _T('u'):
+	case _TSK_T('u'):
 	    flags &= ~FS_FLAG_NAME_UNALLOC;
 	    break;
-	case _T('v'):
+	case _TSK_T('v'):
 	    verbose++;
 	    break;
-	case _T('V'):
+	case _TSK_T('V'):
 	    print_version(stdout);
 	    exit(0);
 	case 'z':
 	    {
-		TCHAR envstr[32];
-		TSNPRINTF(envstr, 32, _T("TZ=%s"), optarg);
+		TSK_TCHAR envstr[32];
+		TSNPRINTF(envstr, 32, _TSK_T("TZ=%s"), optarg);
 		if (0 != PUTENV(envstr)) {
 		    tsk_fprintf(stderr, "error setting environment");
 		    exit(1);
@@ -197,10 +198,10 @@ MAIN(int argc, TCHAR ** argv)
     if (macpre) {
 	size_t len = TSTRLEN(macpre);
 	if (macpre[len - 1] != '/') {
-	    TCHAR *tmp = macpre;
-	    macpre = (TCHAR *) malloc(len + 2 * sizeof(TCHAR));
+	    TSK_TCHAR *tmp = macpre;
+	    macpre = (TSK_TCHAR *) malloc(len + 2 * sizeof(TSK_TCHAR));
 	    TSTRNCPY(macpre, tmp, len + 1);
-	    TSTRNCAT(macpre, _T("/"), len + 2);
+	    TSTRNCAT(macpre, _TSK_T("/"), len + 2);
 	}
     }
 
@@ -212,7 +213,7 @@ MAIN(int argc, TCHAR ** argv)
 	/* Not an inode at the end */
 	if ((img =
 		img_open(imgtype, argc - optind,
-		    (const TCHAR **) &argv[optind])) == NULL) {
+		    (const TSK_TCHAR **) &argv[optind])) == NULL) {
 	    tsk_error_print(stderr);
 	    exit(1);
 	}
@@ -229,7 +230,7 @@ MAIN(int argc, TCHAR ** argv)
     else {
 	if ((img =
 		img_open(imgtype, argc - optind - 1,
-		    (const TCHAR **) &argv[optind])) == NULL) {
+		    (const TSK_TCHAR **) &argv[optind])) == NULL) {
 	    tsk_error_print(stderr);
 	    exit(1);
 	}
