@@ -2,7 +2,7 @@
 ** fs_dent
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/12/05 21:39:52 $
 **
 ** Display and manipulate directory entries 
 ** This file contains generic functions that call the appropriate function
@@ -51,6 +51,8 @@ fs_dent_alloc(ULONG norm_namelen, ULONG shrt_namelen)
     }
     fs_dent->name_max = norm_namelen;
 
+    //fs_dent->flags = 0;
+
     fs_dent->shrt_name_max = shrt_namelen;
     if (shrt_namelen == 0) {
 	fs_dent->shrt_name = NULL;
@@ -99,6 +101,9 @@ fs_dent_realloc(FS_DENT * fs_dent, ULONG namelen)
 void
 fs_dent_free(FS_DENT * fs_dent)
 {
+    if (!fs_dent)
+	return;
+
     if (fs_dent->fsi)
 	fs_inode_free(fs_dent->fsi);
 
