@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/08/17 17:12:51 $
+ * $Date: 2006/12/05 21:39:52 $
  *
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
  *
@@ -93,6 +93,9 @@ fs_inode_free(FS_INODE * fs_inode)
 {
     FS_NAME *fs_name, *fs_name2;
 
+    if (!fs_inode)
+	return;
+
     if (fs_inode->direct_addr)
 	free((char *) fs_inode->direct_addr);
     fs_inode->direct_addr = NULL;
