@@ -5,7 +5,7 @@
 ** This header file is to be included if file system routines
 ** are used from the library.  
 **
-** $Date: 2006/08/30 21:09:11 $
+** $Date: 2006/12/05 21:39:52 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -20,19 +20,18 @@
 #ifndef _FS_TOOLS_H
 #define _FS_TOOLS_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 
 #include "img_tools.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /****************  FILE SYSTEM TYPES SECTION ********************/
-    extern uint8_t fs_parse_type(const TCHAR *);
+    extern uint8_t fs_parse_type(const TSK_TCHAR *);
     extern void fs_print_types(FILE *);
     extern char *fs_get_type(uint8_t);
 
@@ -79,9 +78,6 @@ extern "C" {
 #define HFS                     0x80
 
 
-
-
-
     typedef struct FS_INFO FS_INFO;
     typedef struct FS_INODE FS_INODE;
     typedef struct FS_DENT FS_DENT;
@@ -142,7 +138,10 @@ extern "C" {
 	/* endian order flag - values defined in misc/tsk_endian.h */
 	uint8_t endian;
 
-
+	TSK_LIST *list_inum_named;	/* list of unallocated inodes that
+					 * are pointed to by a file name -- 
+					 * Used to find orphans
+					 */
 	/* file system specific function pointers */
 	 uint8_t(*block_walk) (FS_INFO *, DADDR_T, DADDR_T, int,
 	    FS_BLOCK_WALK_FN, void *);
@@ -181,12 +180,12 @@ extern "C" {
 #define FS_FLAG_NAME_RECURSE (1<<2)	/* recurse on directories */
 
 /* flags that are used for inode_walk and FS_INODE */
-#define FS_FLAG_META_LINK	(1<<0)	/* link count > 0 */
-#define FS_FLAG_META_UNLINK	(1<<1)	/* link count == 0 */
-#define FS_FLAG_META_ALLOC	(1<<2)	/* allocated */
-#define FS_FLAG_META_UNALLOC    (1<<3)	/* unallocated */
-#define FS_FLAG_META_USED	(1<<4)	/* used */
-#define FS_FLAG_META_UNUSED	(1<<5)	/* never used */
+#define FS_FLAG_META_ALLOC	(1<<0)	/* allocated */
+#define FS_FLAG_META_UNALLOC    (1<<1)	/* unallocated */
+#define FS_FLAG_META_USED	(1<<2)	/* used */
+#define FS_FLAG_META_UNUSED	(1<<3)	/* never used */
+#define FS_FLAG_META_ORPHAN	(1<<4)	/* Orphan Files */
+#define FS_FLAG_META_COMP	(1<<5)	/* The file contains compressed data */
 
 /* flags that are used for block_walk and any data units 
  * Including the data units in the action of file_walk */
@@ -197,6 +196,11 @@ extern "C" {
 #define FS_FLAG_DATA_BAD	(1<<4)	/* marked as bad by the FS */
 #define FS_FLAG_DATA_ALIGN	(1<<5)	/* block align (i.e. send a whole block) */
 #define FS_FLAG_DATA_RES	(1<<6)	/* This data is resident (NTFS ONLY) -- used by ntfs_data_walk */
+#define FS_FLAG_DATA_SPARSE	(1<<7)	/* Used to note that the block addr
+					 * is not be accurate since the block
+					 * is sparse (all zeros) */
+#define FS_FLAG_DATA_COMP	(1<<8)	/* This "block" was stored in compressed form */
+
 
 
 
@@ -363,6 +367,8 @@ extern "C" {
 
 	uint8_t ent_type;	/* dir, file etc FS_DENT_??? */
 
+	//int flags;            /* FS_FLAG_NAME_* */
+
 	char *path;		/* prefix to name when recursing */
 	unsigned int pathdepth;	/* current depth of directories */
     };
@@ -462,7 +468,7 @@ extern "C" {
 /**************************************************************8
  * Generic routines.
  */
-    extern FS_INFO *fs_open(IMG_INFO *, SSIZE_T, const TCHAR *);
+    extern FS_INFO *fs_open(IMG_INFO *, SSIZE_T, const TSK_TCHAR *);
 
 /* fs_io routines */
     extern SSIZE_T fs_read_block(FS_INFO *, DATA_BUF *, OFF_T, DADDR_T);
@@ -473,6 +479,14 @@ extern "C" {
     extern char *fs_load_file(FS_INFO *, FS_INODE *, uint32_t, uint16_t,
 	int);
 
+    extern SSIZE_T
+	fs_read_file(FS_INFO *, FS_INODE *, uint32_t, uint16_t,
+	SSIZE_T, SSIZE_T, char *);
+
+    extern SSIZE_T
+	fs_read_file_noid(FS_INFO *, FS_INODE *, SSIZE_T, SSIZE_T, char *);
+
+
 
 
 
@@ -515,7 +529,7 @@ extern "C" {
 #define FLS_FULL        0x010
 #define FLS_MAC         0x020
     extern uint8_t fs_fls(FS_INFO * fs, uint8_t lclflags, INUM_T inode,
-	int flags, TCHAR * pre, int32_t skew);
+	int flags, TSK_TCHAR * pre, int32_t skew);
 
 
     extern uint8_t fs_icat(FS_INFO * fs, uint8_t lclflags, INUM_T inum,
@@ -527,19 +541,21 @@ extern "C" {
 #define IFIND_DATA      0x08
 #define IFIND_PAR       0x10
 #define IFIND_PAR_LONG  0x20
-    extern uint8_t fs_ifind_path(FS_INFO * fs, uint8_t lclflags,
-	TCHAR * path);
+    extern int8_t fs_ifind_path(FS_INFO * fs, uint8_t lclflags,
+	TSK_TCHAR * path, INUM_T * result);
     extern uint8_t fs_ifind_data(FS_INFO * fs, uint8_t lclflags,
 	DADDR_T blk);
     extern uint8_t fs_ifind_par(FS_INFO * fs, uint8_t lclflags,
 	INUM_T par);
 
 
-#define ILS_OPEN        0x1
-#define ILS_REM 0x2
-#define ILS_MAC 0x4
+#define ILS_OPEN        (1<<0)
+#define ILS_MAC		(1<<1)
+#define ILS_LINK	(1<<2)
+#define ILS_UNLINK	(1<<3)
+
     extern uint8_t fs_ils(FS_INFO * fs, uint8_t lclflags, INUM_T istart,
-	INUM_T ilast, int flags, int32_t skew, TCHAR * img);
+	INUM_T ilast, int flags, int32_t skew, TSK_TCHAR * img);
 
 
 
