@@ -2,7 +2,7 @@
 ** icat 
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:12 $
+** $Date: 2006/09/20 20:16:01 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -28,13 +28,13 @@
 
 /* usage - explain and terminate */
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 static void
 usage()
 {
     TFPRINTF(stderr,
-	_T
+	_TSK_T
 	("usage: %s [-hHsvV] [-f fstype] [-i imgtype] [-o imgoffset] image [images] inum[-typ[-id]]\n"),
 	progname);
     tsk_fprintf(stderr, "\t-h: Do not display holes in sparse files\n");
@@ -55,10 +55,10 @@ usage()
 }
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
-    TCHAR *fstype = NULL;
-    TCHAR *imgtype = NULL;
+    TSK_TCHAR *fstype = NULL;
+    TSK_TCHAR *imgtype = NULL;
     FS_INFO *fs;
     IMG_INFO *img;
     INUM_T inum;
@@ -74,49 +74,50 @@ MAIN(int argc, TCHAR ** argv)
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, _T("f:hi:o:rRsvV"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("f:hi:o:rRsvV"))) > 0) {
 	switch (ch) {
-	case _T('?'):
+	case _TSK_T('?'):
 	default:
-	    TFPRINTF(stderr, _T("Invalid argument: %s\n"), argv[optind]);
+	    TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+		argv[optind]);
 	    usage();
-	case _T('f'):
+	case _TSK_T('f'):
 	    fstype = optarg;
-	    if (TSTRCMP(fstype, _T("list")) == 0) {
+	    if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
 		fs_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('h'):
+	case _TSK_T('h'):
 	    flags |= FS_FLAG_FILE_NOSPARSE;
 	    break;
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('r'):
+	case _TSK_T('r'):
 	    flags |= FS_FLAG_FILE_RECOVER;
 	    break;
-	case _T('R'):
+	case _TSK_T('R'):
 	    flags |= FS_FLAG_FILE_RECOVER;
 	    suppress_recover_error = 1;
 	    break;
-	case _T('s'):
+	case _TSK_T('s'):
 	    flags |= FS_FLAG_FILE_SLACK;
 	    break;
-	case _T('v'):
+	case _TSK_T('v'):
 	    verbose++;
 	    break;
-	case _T('V'):
+	case _TSK_T('V'):
 	    print_version(stdout);
 	    exit(0);
 	}
@@ -130,14 +131,14 @@ MAIN(int argc, TCHAR ** argv)
 
     /* Get the inode address */
     if (parse_inum(argv[argc - 1], &inum, &type, &id, &id_used)) {
-	TFPRINTF(stderr, _T("Invalid inode address: %s\n"),
+	TFPRINTF(stderr, _TSK_T("Invalid inode address: %s\n"),
 	    argv[argc - 1]);
 	usage();
     }
 
     if ((img =
 	    img_open(imgtype, argc - optind - 1,
-		(const TCHAR **) &argv[optind])) == NULL) {
+		(const TSK_TCHAR **) &argv[optind])) == NULL) {
 	tsk_error_print(stderr);
 	exit(1);
     }
