@@ -2,7 +2,7 @@
 ** ifind (inode find)
 ** The Sleuth Kit
 **
-** $Date: 2006/08/30 21:09:12 $
+** $Date: 2006/12/05 21:39:52 $
 **
 ** Given an image  and block number, identify which inode it is used by
 ** 
@@ -25,14 +25,14 @@
 #include <time.h>
 #include "fs_tools.h"
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 static uint8_t localflags;
 
 static void
 usage()
 {
     TFPRINTF(stderr,
-	_T
+	_TSK_T
 	("usage: %s [-alvV] [-f fstype] [-i imgtype] [-o imgoffset] [-d unit_addr] [-n file] [-p par_addr] [-z ZONE] image [images]\n"),
 	progname);
     tsk_fprintf(stderr, "\t-a: find all inodes\n");
@@ -61,18 +61,18 @@ usage()
 
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
-    TCHAR *imgtype = NULL;
-    TCHAR *fstype = NULL;
+    TSK_TCHAR *imgtype = NULL;
+    TSK_TCHAR *fstype = NULL;
     FS_INFO *fs;
     IMG_INFO *img;
     int ch;
-    TCHAR *cp;
+    TSK_TCHAR *cp;
     extern int optind;
     DADDR_T block = 0;		/* the block to find */
     INUM_T parinode = 0;
-    TCHAR *path = NULL;
+    TSK_TCHAR *path = NULL;
     SSIZE_T imgoff = 0;
 
     progname = argv[0];
@@ -80,12 +80,12 @@ MAIN(int argc, TCHAR ** argv)
 
     localflags = 0;
 
-    while ((ch = getopt(argc, argv, _T("ad:f:i:ln:o:p:vVz:"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("ad:f:i:ln:o:p:vVz:"))) > 0) {
 	switch (ch) {
-	case _T('a'):
+	case _TSK_T('a'):
 	    localflags |= IFIND_ALL;
 	    break;
-	case _T('d'):
+	case _TSK_T('d'):
 	    if (localflags & (IFIND_PAR | IFIND_PATH)) {
 		tsk_fprintf(stderr,
 		    "error: only one address type can be given\n");
@@ -94,29 +94,29 @@ MAIN(int argc, TCHAR ** argv)
 	    localflags |= IFIND_DATA;
 	    block = TSTRTOULL(optarg, &cp, 0);
 	    if (*cp || cp == optarg) {
-		TFPRINTF(stderr, _T("Invalid block address: %s\n"),
+		TFPRINTF(stderr, _TSK_T("Invalid block address: %s\n"),
 		    optarg);
 		usage();
 	    }
 	    break;
-	case _T('f'):
+	case _TSK_T('f'):
 	    fstype = optarg;
-	    if (TSTRCMP(fstype, _T("list")) == 0) {
+	    if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
 		fs_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('l'):
+	case _TSK_T('l'):
 	    localflags |= IFIND_PAR_LONG;
 	    break;
-	case _T('n'):
+	case _TSK_T('n'):
 	    {
 		size_t len;
 		if (localflags & (IFIND_PAR | IFIND_DATA)) {
@@ -125,8 +125,8 @@ MAIN(int argc, TCHAR ** argv)
 		    usage();
 		}
 		localflags |= IFIND_PATH;
-		len = (TSTRLEN(optarg) + 1) * sizeof(TCHAR);
-		if ((path = (TCHAR *) mymalloc(len)) == NULL) {
+		len = (TSTRLEN(optarg) + 1) * sizeof(TSK_TCHAR);
+		if ((path = (TSK_TCHAR *) mymalloc(len)) == NULL) {
 		    tsk_error_print(stderr);
 		    exit(1);
 		}
@@ -148,7 +148,7 @@ MAIN(int argc, TCHAR ** argv)
 	    localflags |= IFIND_PAR;
 	    parinode = TSTRTOULL(optarg, &cp, 0);
 	    if (parse_inum(optarg, &parinode, NULL, NULL, NULL)) {
-		TFPRINTF(stderr, _T("Invalid inode address: %s\n"),
+		TFPRINTF(stderr, _TSK_T("Invalid inode address: %s\n"),
 		    optarg);
 		usage();
 	    }
@@ -161,8 +161,8 @@ MAIN(int argc, TCHAR ** argv)
 	    exit(0);
 	case 'z':
 	    {
-		TCHAR envstr[32];
-		TSNPRINTF(envstr, 32, _T("TZ=%s"), optarg);
+		TSK_TCHAR envstr[32];
+		TSNPRINTF(envstr, 32, _TSK_T("TZ=%s"), optarg);
 		if (0 != PUTENV(envstr)) {
 		    tsk_fprintf(stderr, "error setting environment");
 		    exit(1);
@@ -195,7 +195,7 @@ MAIN(int argc, TCHAR ** argv)
 
     if ((img =
 	    img_open(imgtype, argc - optind,
-		(const TCHAR **) &argv[optind])) == NULL) {
+		(const TSK_TCHAR **) &argv[optind])) == NULL) {
 	tsk_error_print(stderr);
 	if (path)
 	    free(path);
@@ -257,12 +257,14 @@ MAIN(int argc, TCHAR ** argv)
 	    fs->close(fs);
 	    img->close(img);
 	    exit(1);
-
 	}
     }
 
     else if (localflags & IFIND_PATH) {
-	if (fs_ifind_path(fs, localflags, path)) {
+	int retval;
+	INUM_T inum;
+
+	if (-1 == (retval = fs_ifind_path(fs, localflags, path, &inum))) {
 	    tsk_error_print(stderr);
 	    fs->close(fs);
 	    img->close(img);
@@ -270,8 +272,11 @@ MAIN(int argc, TCHAR ** argv)
 	    exit(1);
 	}
 	free(path);
+	if (retval == 1)
+	    tsk_printf("File not found\n");
+	else
+	    tsk_printf("%" PRIuINUM "\n", inum);
     }
-
     fs->close(fs);
     img->close(img);
 
