@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:12 $
+** $Date: 2006/09/21 16:47:06 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -27,30 +27,30 @@
 #include <locale.h>
 #include "fs_tools.h"
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 /* usage - explain and terminate */
 static void
 usage()
 {
     TFPRINTF(stderr,
-	_T
-	("usage: %s [-eOmrvV] [-aAlLzZ] [-f fstype] [-i imgtype] [-o imgoffset] [-s seconds] image [images] [inum[-end]]\n"),
+	_TSK_T
+	("usage: %s [-emOpvV] [-aAlLzZ] [-f fstype] [-i imgtype] [-o imgoffset] [-s seconds] image [images] [inum[-end]]\n"),
 	progname);
     tsk_fprintf(stderr, "\t-e: Display all inodes\n");
+    tsk_fprintf(stderr, "\t-m: Display output in the mactime format\n");
     tsk_fprintf(stderr,
-	"\t-O: Display inodes that are removed, but sill open (was -o)\n");
+	"\t-O: Display inodes that are removed, but sill open\n");
     tsk_fprintf(stderr,
-	"\t-m: Display output in the mactime format (replaces ils2mac from TCT)\n");
-    tsk_fprintf(stderr, "\t-r: Display removed inodes (default)\n");
+	"\t-p: Display orphan inodes (unallocated with no file name)\n");
     tsk_fprintf(stderr,
 	"\t-s seconds: Time skew of original machine (in seconds)\n");
-    tsk_fprintf(stderr, "\t-a: Allocated files\n");
-    tsk_fprintf(stderr, "\t-A: Un-Allocated files\n");
-    tsk_fprintf(stderr, "\t-l: Linked files\n");
-    tsk_fprintf(stderr, "\t-L: Un-Linked files\n");
-    tsk_fprintf(stderr, "\t-z: Un-Used files (ctime is 0)\n");
-    tsk_fprintf(stderr, "\t-Z: Used files (ctime is not 0)\n");
+    tsk_fprintf(stderr, "\t-a: Allocated inodes\n");
+    tsk_fprintf(stderr, "\t-A: Unallocated inodes\n");
+    tsk_fprintf(stderr, "\t-l: Linked inodes\n");
+    tsk_fprintf(stderr, "\t-L: Unlinked inodes\n");
+    tsk_fprintf(stderr, "\t-z: Unused inodes (ctime is 0)\n");
+    tsk_fprintf(stderr, "\t-Z: Used inodes (ctime is not 0)\n");
     tsk_fprintf(stderr,
 	"\t-i imgtype: The format of the image file (use '-i list' for supported types)\n");
     tsk_fprintf(stderr,
@@ -65,21 +65,20 @@ usage()
 
 
 /* main - open file system, list inode info */
-
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
-    TCHAR *fstype = NULL;
-    TCHAR *imgtype = NULL, *cp, *dash;
+    TSK_TCHAR *fstype = NULL;
+    TSK_TCHAR *imgtype = NULL, *cp, *dash;
     IMG_INFO *img;
     FS_INFO *fs;
     INUM_T istart = 0, ilast = 0;
     int ch;
-    int flags = 0;
+    int flags = FS_FLAG_META_UNALLOC;
     int argflags = 0;
     SSIZE_T imgoff = 0;
     int set_range = 1;
-    TCHAR *image = NULL;
+    TSK_TCHAR *image = NULL;
     int32_t sec_skew = 0;
 
     progname = argv[0];
@@ -89,81 +88,82 @@ MAIN(int argc, TCHAR ** argv)
      * Provide convenience options for the most commonly selected feature
      * combinations.
      */
-    while ((ch = getopt(argc, argv, _T("aAef:i:lLo:Omrs:vVzZ"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("aAef:i:lLmo:Oprs:vVzZ"))) > 0) {
 	switch (ch) {
-	case _T('?'):
+	case _TSK_T('?'):
 	default:
-	    TFPRINTF(stderr, _T("Invalid argument: %s\n"), argv[optind]);
+	    TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+		argv[optind]);
 	    usage();
-	case _T('e'):
-	    flags |= ~0;
-	    break;
-	case _T('f'):
+	case _TSK_T('f'):
 	    fstype = optarg;
-	    if (TSTRCMP(fstype, _T("list")) == 0) {
+	    if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
 		fs_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('m'):
+	case _TSK_T('e'):
+	    flags |= (FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC);
+	    break;
+	case _TSK_T('m'):
 	    argflags |= ILS_MAC;
 	    break;
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('O'):
-	    flags |= (FS_FLAG_META_ALLOC | FS_FLAG_META_UNLINK);
+	case _TSK_T('O'):
+	    flags |= FS_FLAG_META_UNALLOC;
+	    flags &= ~FS_FLAG_META_ALLOC;
 	    argflags |= ILS_OPEN;
 	    break;
-	case _T('r'):
-	    argflags |= ILS_REM;
+	case _TSK_T('p'):
+	    flags |= (FS_FLAG_META_ORPHAN | FS_FLAG_META_UNALLOC);
+	    flags &= ~FS_FLAG_META_ALLOC;
 	    break;
-	case _T('s'):
+	case _TSK_T('r'):
+	    flags |= FS_FLAG_META_UNALLOC;
+	    flags &= ~FS_FLAG_META_ALLOC;
+	    break;
+	case _TSK_T('s'):
 	    sec_skew = TATOI(optarg);
 	    break;
-	case _T('v'):
+	case _TSK_T('v'):
 	    verbose++;
 	    break;
-	case _T('V'):
+	case _TSK_T('V'):
 	    print_version(stdout);
 	    exit(0);
 
 	    /*
 	     * Provide fine controls to tweak one feature at a time.
 	     */
-	case _T('a'):
+	case _TSK_T('a'):
 	    flags |= FS_FLAG_META_ALLOC;
-	    flags &= ~FS_FLAG_META_UNALLOC;
 	    break;
-	case _T('A'):
+	case _TSK_T('A'):
 	    flags |= FS_FLAG_META_UNALLOC;
-	    flags &= ~FS_FLAG_META_ALLOC;
 	    break;
-	case _T('l'):
-	    flags |= FS_FLAG_META_LINK;
-	    flags &= ~FS_FLAG_META_UNLINK;
+	case _TSK_T('l'):
+	    argflags |= ILS_LINK;
 	    break;
-	case _T('L'):
-	    flags |= FS_FLAG_META_UNLINK;
-	    flags &= ~FS_FLAG_META_LINK;
+	case _TSK_T('L'):
+	    argflags |= ILS_UNLINK;
 	    break;
-	case _T('z'):
+	case _TSK_T('z'):
 	    flags |= FS_FLAG_META_UNUSED;
-	    flags &= ~FS_FLAG_META_USED;
 	    break;
-	case _T('Z'):
+	case _TSK_T('Z'):
 	    flags |= FS_FLAG_META_USED;
-	    flags &= ~FS_FLAG_META_UNUSED;
 	    break;
 	}
     }
@@ -173,8 +173,14 @@ MAIN(int argc, TCHAR ** argv)
 	usage();
     }
 
+    if ((argflags & ILS_LINK) && (argflags & ILS_UNLINK)) {
+	tsk_fprintf(stderr,
+	    "ERROR: Only linked or unlinked should be used\n");
+	usage();
+    }
+
     /* We need to determine if an inode or inode range was given */
-    if ((dash = TSTRCHR(argv[argc - 1], _T('-'))) == NULL) {
+    if ((dash = TSTRCHR(argv[argc - 1], _TSK_T('-'))) == NULL) {
 	/* Check if is a single number */
 	istart = TSTRTOULL(argv[argc - 1], &cp, 0);
 	if (*cp || cp == argv[argc - 1]) {
@@ -182,7 +188,7 @@ MAIN(int argc, TCHAR ** argv)
 	    image = argv[optind];
 	    if ((img =
 		    img_open(imgtype, argc - optind,
-			(const TCHAR **) &argv[optind])) == NULL) {
+			(const TSK_TCHAR **) &argv[optind])) == NULL) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
@@ -194,7 +200,7 @@ MAIN(int argc, TCHAR ** argv)
 	    image = argv[optind];
 	    if ((img =
 		    img_open(imgtype, argc - optind - 1,
-			(const TCHAR **) &argv[optind])) == NULL) {
+			(const TSK_TCHAR **) &argv[optind])) == NULL) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
@@ -207,11 +213,11 @@ MAIN(int argc, TCHAR ** argv)
 	istart = TSTRTOULL(argv[argc - 1], &cp, 0);
 	if (*cp || cp == argv[argc - 1]) {
 	    /* Not a number - consider it a file name */
-	    *dash = _T('-');
+	    *dash = _TSK_T('-');
 	    image = argv[optind];
 	    if ((img =
 		    img_open(imgtype, argc - optind,
-			(const TCHAR **) &argv[optind])) == NULL) {
+			(const TSK_TCHAR **) &argv[optind])) == NULL) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
@@ -226,7 +232,7 @@ MAIN(int argc, TCHAR ** argv)
 		image = argv[optind];
 		if ((img =
 			img_open(imgtype, argc - optind,
-			    (const TCHAR **) &argv[optind])) == NULL) {
+			    (const TSK_TCHAR **) &argv[optind])) == NULL) {
 		    tsk_error_print(stderr);
 		    exit(1);
 		}
@@ -237,7 +243,7 @@ MAIN(int argc, TCHAR ** argv)
 		image = argv[optind];
 		if ((img =
 			img_open(imgtype, argc - optind - 1,
-			    (const TCHAR **) &argv[optind])) == NULL) {
+			    (const TSK_TCHAR **) &argv[optind])) == NULL) {
 		    tsk_error_print(stderr);
 		    exit(1);
 		}
@@ -253,7 +259,6 @@ MAIN(int argc, TCHAR ** argv)
 	exit(1);
     }
 
-
     /* do we need to set the range or just check them? */
     if (set_range) {
 	istart = fs->first_inum;
@@ -283,30 +288,6 @@ MAIN(int argc, TCHAR ** argv)
 	exit(1);
     }
 
-    /* removed inodes (default behavior) */
-    if ((argflags & ILS_REM) || (flags == 0)) {
-	if (((fs->ftype & FSMASK) == NTFS_TYPE) ||
-	    ((fs->ftype & FSMASK) == FATFS_TYPE))
-	    flags |= (FS_FLAG_META_USED | FS_FLAG_META_UNALLOC);
-	else
-	    flags |= (FS_FLAG_META_USED | FS_FLAG_META_UNLINK);
-    }
-
-    /* If neither of the flags in a family are set, then set both 
-     *
-     * Apply rules for default settings. Assume a "don't care" condition when
-     * nothing is explicitly selected from a specific feature category.
-     */
-    if ((flags & (FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC)) == 0)
-	flags |= FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC;
-
-    if ((flags & (FS_FLAG_META_LINK | FS_FLAG_META_UNLINK)) == 0)
-	flags |= FS_FLAG_META_LINK | FS_FLAG_META_UNLINK;
-
-    if ((flags & (FS_FLAG_META_USED | FS_FLAG_META_UNUSED)) == 0)
-	flags |= FS_FLAG_META_USED | FS_FLAG_META_UNUSED;
-
-
     if (fs_ils(fs, argflags, istart, ilast, flags, sec_skew, image)) {
 	tsk_error_print(stderr);
 	fs->close(fs);
