@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:12 $
+** $Date: 2006/09/21 16:47:06 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -31,7 +31,7 @@
 #include <Winsock2.h>
 #endif
 
-static TCHAR *image;
+static TSK_TCHAR *image;
 
 /* number of seconds time skew of system 
  * if the system was 100 seconds fast, the value should be +100 
@@ -115,6 +115,14 @@ print_header_mac()
 static uint8_t
 ils_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 {
+    uint8_t lclflags = *(uint8_t *) ptr;
+
+    if ((fs_inode->nlink == 0) && ((lclflags & ILS_UNLINK) == 0)) {
+	return WALK_CONT;
+    }
+    else if ((fs_inode->nlink == 1) && ((lclflags & ILS_LINK) == 0)) {
+	return WALK_CONT;
+    }
 
     if (sec_skew != 0) {
 	fs_inode->mtime -= sec_skew;
@@ -151,9 +159,17 @@ static uint8_t
 ils_mac_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 {
     char ls[12];
+    uint8_t lclflags = *(uint8_t *) ptr;
+
+    if ((fs_inode->nlink == 0) && ((lclflags & ILS_UNLINK) == 0)) {
+	return WALK_CONT;
+    }
+    else if ((fs_inode->nlink == 1) && ((lclflags & ILS_LINK) == 0)) {
+	return WALK_CONT;
+    }
 
     /* ADD image and file name (if we have one) */
-    TFPRINTF(stdout, _T("0|<%s-"), image);
+    TFPRINTF(stdout, _TSK_T("0|<%s-"), image);
     tsk_printf("%s%s%s-%" PRIuINUM ">|0|%" PRIuINUM "|%d|",
 	(fs_inode->name) ? fs_inode->name->name : "",
 	(fs_inode->name) ? "-" : "",
@@ -189,13 +205,24 @@ ils_mac_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 /* return 1 on error and 0 on success */
 uint8_t
 fs_ils(FS_INFO * fs, uint8_t lclflags, INUM_T istart, INUM_T ilast,
-    int flags, int32_t skew, TCHAR * img)
+    int flags, int32_t skew, TSK_TCHAR * img)
 {
     sec_skew = skew;
 
+    /* If orphan is desired, then make sure LINK flags are set */
+    if (flags & FS_FLAG_META_ORPHAN) {
+	lclflags |= (ILS_LINK | ILS_UNLINK);
+    }
+    else {
+	/* If LINK is not set at all, then set them */
+	if (((lclflags & ILS_LINK) == 0) && ((lclflags & ILS_UNLINK) == 0))
+	    lclflags |= (ILS_LINK | ILS_UNLINK);
+    }
+
+
     /* Print the data */
     if (lclflags & ILS_MAC) {
-	TCHAR *tmpptr;
+	TSK_TCHAR *tmpptr;
 	image = img;
 
 #ifdef TSK_WIN32
@@ -209,12 +236,13 @@ fs_ils(FS_INFO * fs, uint8_t lclflags, INUM_T istart, INUM_T ilast,
 
 	print_header_mac();
 
-	if (fs->inode_walk(fs, istart, ilast, flags, ils_mac_act, NULL))
+	if (fs->inode_walk(fs, istart, ilast, flags, ils_mac_act,
+		&lclflags))
 	    return 1;
     }
     else {
 	print_header(fs);
-	if (fs->inode_walk(fs, istart, ilast, flags, ils_act, NULL))
+	if (fs->inode_walk(fs, istart, ilast, flags, ils_act, &lclflags))
 	    return 1;
     }
 
