@@ -205,8 +205,8 @@ iso9660_get_dentrys(FS_INFO * fs, OFF_T offs, int count, int type,
 	sizeof(iso9660_dentry), offs);
     if (cnt != sizeof(iso9660_dentry)) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L, "iso_get_dentries");
 	return -1;
@@ -238,8 +238,8 @@ iso9660_get_dentrys(FS_INFO * fs, OFF_T offs, int count, int type,
 	cnt = fs_read_random(fs, buf, dentry_bytes, offs);
 	if (cnt != dentry_bytes) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_FS_READ;
-		tsk_errstr[0] = '\0';
 	    }
 	    snprintf(tsk_errstr2, TSK_ERRSTR_L, "iso_get_dentries");
 	    return -1;
@@ -309,8 +309,8 @@ iso9660_get_dentrys(FS_INFO * fs, OFF_T offs, int count, int type,
 	    fs_read_random(fs, (char *) &dd, sizeof(iso9660_dentry), offs);
 	if (cnt != sizeof(iso9660_dentry)) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_FS_READ;
-		tsk_errstr[0] = '\0';
 	    }
 	    snprintf(tsk_errstr2, TSK_ERRSTR_L, "iso_get_dentries");
 	    return -1;
@@ -333,8 +333,8 @@ iso9660_get_dentrys(FS_INFO * fs, OFF_T offs, int count, int type,
 		cnt = fs_read_random(fs, (char *) uni_buf, dd.len, offs);
 		if (cnt != dd.len) {
 		    if (cnt != -1) {
+			tsk_error_reset();
 			tsk_errno = TSK_ERR_FS_READ;
-			tsk_errstr[0] = '\0';
 		    }
 		    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			"iso_get_dentries");
@@ -377,8 +377,8 @@ iso9660_get_dentrys(FS_INFO * fs, OFF_T offs, int count, int type,
 		cnt = fs_read_random(fs, in->inode.fn, dd.len, offs);
 		if (cnt != dd.len) {
 		    if (cnt != -1) {
+			tsk_error_reset();
 			tsk_errno = TSK_ERR_FS_READ;
-			tsk_errstr[0] = '\0';
 		    }
 		    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			"iso_get_dentries");
@@ -417,8 +417,8 @@ iso9660_get_dentrys(FS_INFO * fs, OFF_T offs, int count, int type,
 		cnt = fs_read_random(fs, buf, dentry_bytes, offs);
 		if (cnt != dentry_bytes) {
 		    if (cnt != -1) {
+			tsk_error_reset();
 			tsk_errno = TSK_ERR_FS_READ;
-			tsk_errstr[0] = '\0';
 		    }
 		    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 			"iso_get_dentries");
@@ -553,8 +553,8 @@ iso9660_find_inodes(ISO_INFO * iso)
 		fs_read_random(fs, (char *) &dir, (int) sizeof(dir), offs);
 	    if (cnt != sizeof(dir)) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L, "iso_find_inodes");
 		return -1;
@@ -566,8 +566,8 @@ iso9660_find_inodes(ISO_INFO * iso)
 	    cnt = fs_read_random(fs, (char *) uni_buf, dir.len_di, offs);
 	    if (cnt != sizeof(dir)) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L, "iso_find_inodes");
 		return -1;
@@ -647,8 +647,8 @@ iso9660_find_inodes(ISO_INFO * iso)
 	    cnt = fs_read_random(fs, (char *) &dir, sizeof(dir), offs);
 	    if (cnt != sizeof(dir)) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L, "iso_find_inodes");
 		return -1;
@@ -660,8 +660,8 @@ iso9660_find_inodes(ISO_INFO * iso)
 	    cnt = fs_read_random(fs, fn, dir.len_di, offs);
 	    if (cnt != dir.len_di) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L, "iso_find_inodes");
 		return -1;
@@ -775,6 +775,9 @@ iso9660_inode_lookup(FS_INFO * fs, INUM_T inum)
     ISO_INFO *iso = (ISO_INFO *) fs;
     FS_INODE *fs_inode;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if ((fs_inode =
 	    fs_inode_alloc(ISO9660_NDADDR, ISO9660_NIADDR)) == NULL)
 	return NULL;
@@ -803,6 +806,9 @@ iso9660_inode_walk(FS_INFO * fs, INUM_T start, INUM_T last, int flags,
     FS_INODE *fs_inode;
     int myflags;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if ((fs_inode =
 	    fs_inode_alloc(ISO9660_NDADDR, ISO9660_NIADDR)) == NULL)
 	return 1;
@@ -813,25 +819,45 @@ iso9660_inode_walk(FS_INFO * fs, INUM_T start, INUM_T last, int flags,
 	    " action: %lu ptr: %lu\n",
 	    (ULONG) fs, start, last, flags, (ULONG) action, (ULONG) ptr);
 
-    myflags = FS_FLAG_META_LINK | FS_FLAG_META_USED | FS_FLAG_META_ALLOC;
+    myflags = FS_FLAG_META_ALLOC;
 
     /*
      * Sanity checks.
      */
     if (start < fs->first_inum || start > fs->last_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: Start inode:  %" PRIuINUM "", myname, start);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
     if (last < fs->first_inum || last > fs->last_inum || last < start) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: End inode: %" PRIuINUM "", myname, last);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
+
+    // @@@@ it does not seem that ISO would have any orphans..
+    if (flags & FS_FLAG_META_ORPHAN) {
+	return 0;
+    }
+    else if (((flags & FS_FLAG_META_ALLOC) == 0) &&
+	((flags & FS_FLAG_META_UNALLOC) == 0)) {
+	flags |= (FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC);
+    }
+
+    /* If neither of the USED or UNUSED flags are set, then set them
+     * both
+     */
+    if (((flags & FS_FLAG_META_USED) == 0) &&
+	((flags & FS_FLAG_META_UNUSED) == 0)) {
+	flags |= (FS_FLAG_META_USED | FS_FLAG_META_UNUSED);
+    }
+
+
+
     /*
      * Iterate.
      */
@@ -908,6 +934,9 @@ iso9660_block_walk(FS_INFO * fs, DADDR_T start, DADDR_T last, int flags,
     int myflags = 0;
     SSIZE_T cnt;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if (verbose)
 	tsk_fprintf(stderr, "iso9660_block_walk: fs: %lu"
 	    " start: %" PRIuDADDR " last: %" PRIuDADDR " flags: %d"
@@ -918,20 +947,27 @@ iso9660_block_walk(FS_INFO * fs, DADDR_T start, DADDR_T last, int flags,
      * Sanity checks.
      */
     if (start < fs->first_block || start > fs->last_block) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: Start block: %" PRIuDADDR "", myname, start);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
     if (last < fs->first_block || last > fs->last_block) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "%s: End block: %" PRIuDADDR "", myname, last);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
+    /* Sanity check on flags -- make sure at least one ALLOC is set */
+    if (((flags & FS_FLAG_DATA_ALLOC) == 0) &&
+	((flags & FS_FLAG_DATA_UNALLOC) == 0)) {
+	flags |= (FS_FLAG_DATA_ALLOC | FS_FLAG_DATA_UNALLOC);
+    }
+
+
     if ((data_buf = data_buf_alloc(fs->block_size)) == NULL) {
 	return 1;
     }
@@ -951,8 +987,8 @@ iso9660_block_walk(FS_INFO * fs, DADDR_T start, DADDR_T last, int flags,
 	    cnt = fs_read_block(fs, data_buf, fs->block_size, addr);
 	    if (cnt != fs->block_size) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L, "iso_block_walk");
 		return 1;
@@ -990,6 +1026,9 @@ iso9660_file_walk(FS_INFO * fs, FS_INODE * inode, uint32_t type,
     size_t bytes_read;
     DADDR_T addr;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if (verbose)
 	tsk_fprintf(stderr,
 	    "iso9660_file_walk: inode: %" PRIuINUM " type: %" PRIu32
@@ -1024,8 +1063,8 @@ iso9660_file_walk(FS_INFO * fs, FS_INODE * inode, uint32_t type,
 	    bytes_read = fs_read_random(fs, data_buf, size, offs);
 	    if (bytes_read != size) {
 		if (bytes_read != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "iso9660_file_walk: Error reading block: %"
@@ -1057,10 +1096,10 @@ iso9660_file_walk(FS_INFO * fs, FS_INODE * inode, uint32_t type,
 static uint8_t
 iso9660_fscheck(FS_INFO * fs, FILE * hFile)
 {
+    tsk_error_reset();
     tsk_errno = TSK_ERR_FS_FUNC;
     snprintf(tsk_errstr, TSK_ERRSTR_L,
 	"fscheck not implemented for iso9660 yet");
-    tsk_errstr2[0] = '\0';
     return 1;
 }
 
@@ -1078,6 +1117,9 @@ iso9660_fsstat(FS_INFO * fs, FILE * hFile)
     pvd_node *p = iso->pvd;
     svd_node *s = iso->svd;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if (verbose)
 	tsk_fprintf(stderr, "iso9660_fsstat: fs: %lu \
 			hFile: %lu\n", (ULONG) fs, (ULONG) hFile);
@@ -1420,6 +1462,9 @@ iso9660_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
     FS_INODE *fs_inode;
     iso9660_dentry dd;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     fs_inode = iso9660_inode_lookup(fs, inum);
 
     tsk_fprintf(hFile, "Entry: %lu\n", (ULONG) inum);
@@ -1533,9 +1578,9 @@ iso9660_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
 uint8_t
 iso9660_jopen(FS_INFO * fs, INUM_T inum)
 {
+    tsk_error_reset();
     tsk_errno = TSK_ERR_FS_FUNC;
     snprintf(tsk_errstr, TSK_ERRSTR_L, "ISO9660 does not have a journal");
-    tsk_errstr2[0] = '\0';
     return 1;
 }
 
@@ -1543,9 +1588,9 @@ uint8_t
 iso9660_jentry_walk(FS_INFO * fs, int flags, FS_JENTRY_WALK_FN action,
     void *ptr)
 {
+    tsk_error_reset();
     tsk_errno = TSK_ERR_FS_FUNC;
     snprintf(tsk_errstr, TSK_ERRSTR_L, "ISO9660 does not have a journal");
-    tsk_errstr2[0] = '\0';
     return 1;
 }
 
@@ -1553,9 +1598,9 @@ uint8_t
 iso9660_jblk_walk(FS_INFO * fs, DADDR_T start, DADDR_T end, int flags,
     FS_JBLK_WALK_FN action, void *ptr)
 {
+    tsk_error_reset();
     tsk_errno = TSK_ERR_FS_FUNC;
     snprintf(tsk_errstr, TSK_ERRSTR_L, "ISO9660 does not have a journal");
-    tsk_errstr2[0] = '\0';
     return 1;
 }
 
@@ -1580,6 +1625,11 @@ iso9660_close(FS_INFO * fs)
 
     free((char *) iso->dinode);
 
+    if (fs->list_inum_named) {
+	tsk_list_free(fs->list_inum_named);
+	fs->list_inum_named = NULL;
+    }
+
     free(fs);
 }
 
@@ -1603,13 +1653,9 @@ get_vol_desc(FS_INFO * fs)
     svd_node *s, *stmp;
     iso_bootrec *b;
     off_t offs;
-    char *myname = "get_vol_desc";
+    char *myname = "iso_get_vol_desc";
     SSIZE_T cnt;
 
-    b = (iso_bootrec *) mymalloc(sizeof(iso_bootrec));
-    if (b == NULL) {
-	return -1;
-    }
 
     iso->pvd = NULL;
     iso->svd = NULL;
@@ -1619,8 +1665,8 @@ get_vol_desc(FS_INFO * fs)
     cnt = fs_read_random(fs, (char *) &vd, sizeof(iso_vd), ISO9660_SBOFF);
     if (cnt != sizeof(iso_vd)) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "iso_get_vol_desc: Error reading");
@@ -1629,8 +1675,13 @@ get_vol_desc(FS_INFO * fs)
 
     if (strncmp(vd.magic, ISO9660_MAGIC, 5)) {
 	if (verbose)
-	    tsk_fprintf(stderr, "%s Bad volume descriptor: \
-			         Magic number is not CD001", myname);
+	    tsk_fprintf(stderr, "%s: Bad volume descriptor: \
+			         Magic number is not CD001\n", myname);
+	return -1;
+    }
+
+    b = (iso_bootrec *) mymalloc(sizeof(iso_bootrec));
+    if (b == NULL) {
 	return -1;
     }
 
@@ -1644,6 +1695,7 @@ get_vol_desc(FS_INFO * fs)
 
 	    p = (pvd_node *) mymalloc(sizeof(pvd_node));
 	    if (p == NULL) {
+		free(b);
 		return -1;
 	    }
 
@@ -1652,11 +1704,12 @@ get_vol_desc(FS_INFO * fs)
 		offs);
 	    if (cnt != sizeof(iso_pvd)) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "iso_get_vol_desc: Error reading");
+		free(b);
 		return -1;
 	    }
 
@@ -1692,6 +1745,7 @@ get_vol_desc(FS_INFO * fs)
 
 	    s = (svd_node *) mymalloc(sizeof(svd_node));
 	    if (s == NULL) {
+		free(b);
 		return -1;
 	    }
 
@@ -1700,11 +1754,12 @@ get_vol_desc(FS_INFO * fs)
 		offs);
 	    if (cnt != sizeof(iso_svd)) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "iso_get_vol_desc: Error reading");
+		free(b);
 		return -1;
 	    }
 
@@ -1743,11 +1798,12 @@ get_vol_desc(FS_INFO * fs)
 		fs_read_random(fs, (char *) b, sizeof(iso_bootrec), offs);
 	    if (cnt != sizeof(iso_bootrec)) {
 		if (cnt != -1) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_FS_READ;
-		    tsk_errstr[0] = '\0';
 		}
 		snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		    "iso_get_vol_desc: Error reading");
+		free(b);
 		return -1;
 	    }
 
@@ -1758,18 +1814,20 @@ get_vol_desc(FS_INFO * fs)
 	cnt = fs_read_random(fs, (char *) &vd, sizeof(iso_vd), offs);
 	if (cnt != sizeof(iso_vd)) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_FS_READ;
-		tsk_errstr[0] = '\0';
 	    }
 	    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		"iso_get_vol_desc: Error reading");
+	    free(b);
 	    return -1;
 	}
 
 	if (strncmp(vd.magic, ISO9660_MAGIC, 5)) {
 	    if (verbose)
-		tsk_fprintf(stderr, "%s Bad volume descriptor: \
+		tsk_fprintf(stderr, "%s: Bad volume descriptor: \
 				         Magic number is not CD001", myname);
+	    free(b);
 	    return -1;
 	}
     }
@@ -1807,10 +1865,10 @@ get_vol_desc(FS_INFO * fs)
     }
 
     if ((iso->pvd == NULL) && (iso->svd == NULL)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "get_vol_desc: primary and secondary volume descriptors null");
-	tsk_errstr2[0] = '\0';
 	return -1;
     }
 
@@ -1890,6 +1948,7 @@ iso9660_open(IMG_INFO * img_info, SSIZE_T offset, unsigned char ftype,
     iso->rr_found = 0;
 
     if ((ftype & FSMASK) != ISO9660_TYPE) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Invalid FS type in iso9660_open");
@@ -1916,6 +1975,7 @@ iso9660_open(IMG_INFO * img_info, SSIZE_T offset, unsigned char ftype,
 	if (test)
 	    return NULL;
 	else {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_MAGIC;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"Invalid FS type in iso9660_open");
@@ -1926,6 +1986,7 @@ iso9660_open(IMG_INFO * img_info, SSIZE_T offset, unsigned char ftype,
     fs->dev_bsize = 512;
     fs->block_size = find_block_sizes(fs);
     if (fs->block_size == 0) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L, "Block size not found");
 	return NULL;
@@ -1972,5 +2033,7 @@ iso9660_open(IMG_INFO * img_info, SSIZE_T offset, unsigned char ftype,
     }
     iso->dinum = -1;
 
+    fs->list_inum_named = NULL;
+
     return fs;
 }
