@@ -85,7 +85,11 @@ iso9660_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
     int retval;
     SSIZE_T cnt;
 
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
     if (inum < fs->first_inum || inum > fs->last_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "iso9660_dent_walk: inode value: %" PRIuINUM "\n", inum);
@@ -105,6 +109,8 @@ iso9660_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
 
     /* walking a directory */
     if ((iso->dinode->dr.flags & ISO9660_FLAG_DIR) == ISO9660_FLAG_DIR) {
+	TSK_LIST *list_seen = NULL;
+
 	/* calculate directory extent location */
 	offs =
 	    (off_t) (fs->block_size *
@@ -119,6 +125,7 @@ iso9660_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
 	cnt = fs_read_random(fs, buf, length, offs);
 	if (cnt != length) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_FS_READ;
 		tsk_errstr[0] = '\0';
 	    }
@@ -220,10 +227,14 @@ iso9660_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
 		    retval = action(fs, fs_dent, myflags, ptr);
 		    if (retval == WALK_ERROR) {
 			fs_dent_free(fs_dent);
+			tsk_list_free(list_seen);
+			list_seen = NULL;
 			return 1;
 		    }
 		    else if (retval == WALK_STOP) {
 			fs_dent_free(fs_dent);
+			tsk_list_free(list_seen);
+			list_seen = NULL;
 			return 0;
 		    }
 		}
@@ -232,32 +243,43 @@ iso9660_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
 		    && (flags & FS_FLAG_NAME_RECURSE)) {
 		    int depth_added = 0;
 
-		    if ((depth < MAX_DEPTH) &&
-			(DIR_STRSZ >
-			    strlen(dirs) + strlen(fs_dent->name))) {
-			didx[depth] = &dirs[strlen(dirs)];
-			strncpy(didx[depth], fs_dent->name,
-			    DIR_STRSZ - strlen(dirs));
-			strncat(dirs, "/", DIR_STRSZ);
-			depth_added = 1;
-		    }
-		    depth++;
-		    if (iso9660_dent_walk(fs, in->inum, flags, action,
-			    ptr)) {
-			/* If the directory could not be loaded,
-			 * then move on */
-			if (verbose) {
-			    tsk_fprintf(stderr,
-				"iso_dent_parse_block: error reading directory: %"
-				PRIuINUM "\n", in->inum);
-			    tsk_error_print(stderr);
+		    /* Make sure we do not get into an infinite loop */
+		    if (0 == tsk_list_find(list_seen, fs_dent->inode)) {
+			if (tsk_list_add(&list_seen, fs_dent->inode)) {
+			    fs_dent_free(fs_dent);
+			    tsk_list_free(list_seen);
+			    list_seen = NULL;
+			    return -1;
 			}
-			tsk_error_reset();
-		    }
 
-		    depth--;
-		    if (depth_added)
-			*didx[depth] = '\0';
+
+			if ((depth < MAX_DEPTH) &&
+			    (DIR_STRSZ >
+				strlen(dirs) + strlen(fs_dent->name))) {
+			    didx[depth] = &dirs[strlen(dirs)];
+			    strncpy(didx[depth], fs_dent->name,
+				DIR_STRSZ - strlen(dirs));
+			    strncat(dirs, "/", DIR_STRSZ);
+			    depth_added = 1;
+			}
+			depth++;
+			if (iso9660_dent_walk(fs, in->inum, flags, action,
+				ptr)) {
+			    /* If the directory could not be loaded,
+			     * then move on */
+			    if (verbose) {
+				tsk_fprintf(stderr,
+				    "iso_dent_parse_block: error reading directory: %"
+				    PRIuINUM "\n", in->inum);
+				tsk_error_print(stderr);
+			    }
+			    tsk_error_reset();
+			}
+
+			depth--;
+			if (depth_added)
+			    *didx[depth] = '\0';
+		    }
 		}
 
 		length -= dd->length;
@@ -287,6 +309,8 @@ iso9660_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
 	}
 
 	free(buf);
+	tsk_list_free(list_seen);
+	list_seen = NULL;
 	/* regular file */
     }
     fs_dent_free(fs_dent);
