@@ -2,7 +2,7 @@
 ** istat
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:12 $
+** $Date: 2006/09/20 20:16:01 $
 **
 ** Display all inode info about a given inode.
 **
@@ -24,14 +24,14 @@
 #include <time.h>
 #include "fs_tools.h"
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 /* usage - explain and terminate */
 static void
 usage()
 {
     TFPRINTF(stderr,
-	_T
+	_TSK_T
 	("usage: %s [-b num] [-f fstype] [-i imgtype] [-o imgoffset] [-z zone] [-s seconds] [-vV] image inum\n"),
 	progname);
     tsk_fprintf(stderr,
@@ -53,15 +53,15 @@ usage()
 
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
-    TCHAR *imgtype = NULL;
-    TCHAR *fstype = NULL;
+    TSK_TCHAR *imgtype = NULL;
+    TSK_TCHAR *fstype = NULL;
     IMG_INFO *img;
     FS_INFO *fs;
     INUM_T inum;
     int ch;
-    TCHAR *cp;
+    TSK_TCHAR *cp;
     int32_t sec_skew = 0;
     SSIZE_T imgoff = 0;
 
@@ -71,55 +71,56 @@ MAIN(int argc, TCHAR ** argv)
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, _T("b:f:i:o:s:vVz:"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("b:f:i:o:s:vVz:"))) > 0) {
 	switch (ch) {
-	case _T('?'):
+	case _TSK_T('?'):
 	default:
-	    TFPRINTF(stderr, _T("Invalid argument: %s\n"), argv[optind]);
+	    TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+		argv[optind]);
 	    usage();
-	case _T('b'):
+	case _TSK_T('b'):
 	    numblock = TSTRTOULL(optarg, &cp, 0);
 	    if (*cp || cp == optarg || numblock < 1) {
 		TFPRINTF(stderr,
-		    _T
+		    _TSK_T
 		    ("invalid argument: block count must be positive: %s\n"),
 		    optarg);
 		usage();
 	    }
 	    break;
-	case _T('f'):
+	case _TSK_T('f'):
 	    fstype = optarg;
-	    if (TSTRCMP(fstype, _T("list")) == 0) {
+	    if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
 		fs_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('s'):
+	case _TSK_T('s'):
 	    sec_skew = TATOI(optarg);
 	    break;
-	case _T('v'):
+	case _TSK_T('v'):
 	    verbose++;
 	    break;
-	case _T('V'):
+	case _TSK_T('V'):
 	    print_version(stdout);
 	    exit(0);
-	case _T('z'):
+	case _TSK_T('z'):
 	    {
-		TCHAR envstr[32];
-		TSNPRINTF(envstr, 32, _T("TZ=%s"), optarg);
+		TSK_TCHAR envstr[32];
+		TSNPRINTF(envstr, 32, _TSK_T("TZ=%s"), optarg);
 		if (0 != PUTENV(envstr)) {
 		    tsk_fprintf(stderr, "error setting environment");
 		    exit(1);
@@ -142,7 +143,8 @@ MAIN(int argc, TCHAR ** argv)
      * This will make scripting easier
      */
     if (parse_inum(argv[argc - 1], &inum, NULL, NULL, NULL)) {
-	TFPRINTF(stderr, _T("Invalid inode number: %s"), argv[argc - 1]);
+	TFPRINTF(stderr, _TSK_T("Invalid inode number: %s"),
+	    argv[argc - 1]);
 	usage();
     }
 
@@ -151,7 +153,7 @@ MAIN(int argc, TCHAR ** argv)
      */
     if ((img =
 	    img_open(imgtype, argc - optind - 1,
-		(const TCHAR **) &argv[optind])) == NULL) {
+		(const TSK_TCHAR **) &argv[optind])) == NULL) {
 	tsk_error_print(stderr);
 	exit(1);
     }
