@@ -2,7 +2,7 @@
 ** jcat
 ** The Sleuth Kit 
 **
-** $Date: 2006/08/30 21:09:12 $
+** $Date: 2006/09/20 20:16:01 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -15,14 +15,14 @@
 #include <locale.h>
 #include "fs_tools.h"
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 /* usage - explain and terminate */
 static void
 usage()
 {
     TFPRINTF(stderr,
-	_T
+	_TSK_T
 	("usage: %s [-f fstype] [-i imgtype] [-o imgoffset] [-vV] image [images] [inode] blk\n"),
 	progname);
     tsk_fprintf(stderr, "\tblk: The journal block to view\n");
@@ -41,51 +41,52 @@ usage()
 
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
-    TCHAR *fstype = NULL;
-    TCHAR *imgtype = NULL;
+    TSK_TCHAR *fstype = NULL;
+    TSK_TCHAR *imgtype = NULL;
     FS_INFO *fs;
     IMG_INFO *img;
     INUM_T inum;
     int ch;
     DADDR_T blk;
-    TCHAR *cp;
+    TSK_TCHAR *cp;
     SSIZE_T imgoff = 0;
 
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, _T("f:i:o:vV"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("f:i:o:vV"))) > 0) {
 	switch (ch) {
-	case _T('?'):
+	case _TSK_T('?'):
 	default:
-	    TFPRINTF(stderr, _T("Invalid argument: %s\n"), argv[optind]);
+	    TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+		argv[optind]);
 	    usage();
-	case _T('f'):
+	case _TSK_T('f'):
 	    fstype = optarg;
-	    if (TSTRCMP(fstype, _T("list")) == 0) {
+	    if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
 		fs_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('v'):
+	case _TSK_T('v'):
 	    verbose++;
 	    break;
-	case _T('V'):
+	case _TSK_T('V'):
 	    print_version(stdout);
 	    exit(0);
 	}
@@ -99,7 +100,7 @@ MAIN(int argc, TCHAR ** argv)
 
     blk = TSTRTOULL(argv[argc - 1], &cp, 0);
     if (*cp || cp == argv[argc - 1]) {
-	TFPRINTF(stderr, _T("bad block number: %s"), argv[argc - 1]);
+	TFPRINTF(stderr, _TSK_T("bad block number: %s"), argv[argc - 1]);
 	exit(1);
     }
 
@@ -108,7 +109,7 @@ MAIN(int argc, TCHAR ** argv)
 	/* Not a number therefore an image */
 	if ((img =
 		img_open(imgtype, argc - optind - 1,
-		    (const TCHAR **) &argv[optind])) == NULL) {
+		    (const TSK_TCHAR **) &argv[optind])) == NULL) {
 	    tsk_error_print(stderr);
 	    exit(1);
 	}
@@ -125,7 +126,7 @@ MAIN(int argc, TCHAR ** argv)
     else {
 	if ((img =
 		img_open(imgtype, argc - optind - 2,
-		    (const TCHAR **) &argv[optind])) == NULL) {
+		    (const TSK_TCHAR **) &argv[optind])) == NULL) {
 	    tsk_error_print(stderr);
 	    exit(1);
 	}
