@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit
 **
-** $Date: 2006/08/25 21:42:19 $
+** $Date: 2006/11/07 19:57:22 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -512,7 +512,7 @@ extern "C" {
  */
     typedef struct {
 	char *buffer;
-	unsigned int size;
+	size_t size;
     } NTFS_SXX_BUFFER;
 
 
