@@ -2,7 +2,7 @@
 ** ntfs_dent
 ** The Sleuth Kit
 **
-** $Date: 2006/08/30 21:09:12 $
+** $Date: 2006/12/05 21:39:52 $
 **
 ** name layer support for the NTFS file system
 **
@@ -42,7 +42,7 @@ typedef struct {
 
 
 static uint8_t
-ntfs_dent_walk_lcl(FS_INFO *, NTFS_DINFO *, INUM_T, int,
+ntfs_dent_walk_lcl(FS_INFO *, NTFS_DINFO *, TSK_LIST **, INUM_T, int,
     FS_DENT_WALK_FN, void *);
 /* 
  * copy the index (directory) entry into the generic structure
@@ -103,11 +103,15 @@ ntfs_dent_copy(NTFS_INFO * ntfs, NTFS_DINFO * dinfo, ntfs_idxentry * idxe,
     if (fs_dent->fsi != NULL)
 	fs_inode_free(fs_dent->fsi);
 
-    fs_dent->fsi = fs->inode_lookup(fs, fs_dent->inode);
-    if (verbose)
-	tsk_fprintf(stderr,
-	    "ntfs_dent_copy: error looking up inode: %" PRIuINUM "\n",
-	    fs_dent->inode);
+    if (NULL == (fs_dent->fsi = fs->inode_lookup(fs, fs_dent->inode))) {
+	if (verbose) {
+	    tsk_fprintf(stderr,
+		"ntfs_dent_copy: error looking up inode: %" PRIuINUM "\n",
+		fs_dent->inode);
+	    tsk_error_print(stderr);
+	    tsk_error_reset();
+	}
+    }
 
     if (getu64(fs->endian, fname->flags) & NTFS_FNAME_FLAGS_DIR)
 	fs_dent->ent_type = FS_DENT_DIR;
@@ -156,8 +160,8 @@ is_time(uint64_t t)
  */
 static int
 ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
-    ntfs_idxentry * idxe, uint32_t size, uint32_t len,
-    int flags, FS_DENT_WALK_FN action, void *ptr)
+    TSK_LIST ** list_seen, ntfs_idxentry * idxe, uint32_t size,
+    uint32_t len, int flags, FS_DENT_WALK_FN action, void *ptr)
 {
     uintptr_t endaddr, endaddr_alloc;
     int myflags = 0;
@@ -170,9 +174,9 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
 
     if (verbose)
 	tsk_fprintf(stderr,
-	    "ntfs_dent_idxentry: Processing entry: %" PRIu64
-	    "  Size: %" PRIu32 "  Len: %" PRIu32 "\n",
-	    (uint64_t) ((uintptr_t) idxe), size, len);
+	    "ntfs_dent_idxentry: Processing index entry: %" PRIu64
+	    "  Size: %" PRIu32 "  Len: %" PRIu32 "  Flags: %x\n",
+	    (uint64_t) ((uintptr_t) idxe), size, len, flags);
 
     /* where is the end of the buffer */
     endaddr = ((uintptr_t) idxe + size);
@@ -223,6 +227,9 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
 		(fname->nspace != NTFS_FNAME_DOS) &&
 		(fname->nspace != NTFS_FNAME_WINDOS)) {
 		idxe = (ntfs_idxentry *) ((uintptr_t) idxe + 4);
+		if (verbose)
+		    tsk_fprintf(stderr,
+			"ntfs_dent_idxentry: Skipping because of invalid name space\n");
 		continue;
 	    }
 
@@ -232,6 +239,9 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
 		|| (*(uint8_t *) & fname->name == 0)) {
 
 		idxe = (ntfs_idxentry *) ((uintptr_t) idxe + 4);
+		if (verbose)
+		    tsk_fprintf(stderr,
+			"ntfs_dent_idxentry: Skipping because of reported file sizes, name length, or NULL name\n");
 		continue;
 	    }
 
@@ -240,6 +250,9 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
 		(is_time(getu64(fs->endian, fname->mtime)) == 0)) {
 
 		idxe = (ntfs_idxentry *) ((uintptr_t) idxe + 4);
+		if (verbose)
+		    tsk_fprintf(stderr,
+			"ntfs_dent_idxentry: Skipping because of invalid times\n");
 		continue;
 	    }
 	}
@@ -269,15 +282,12 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
 	}
 
 	/* Copy it into the generic form */
-	ntfs_dent_copy(ntfs, dinfo, idxe, fs_dent);
-
-	if (verbose)
-	    tsk_fprintf(stderr,
-		"ntfs_dent_idxentry: Deletion Check Details of %s: Str Len: %"
-		PRIu16 "  Len to end after current: %" PRIu64 "\n",
-		fs_dent->name, getu16(fs->endian, idxe->strlen),
-		(uint64_t) (endaddr_alloc - (uintptr_t) idxe -
-		    getu16(fs->endian, idxe->idxlen)));
+	if (ntfs_dent_copy(ntfs, dinfo, idxe, fs_dent)) {
+	    if (verbose)
+		tsk_fprintf(stderr,
+		    "ntfs_dent_idxentry: Skipping because error copying dent_entry\n");
+	    goto incr_entry;
+	}
 
 	/* 
 	 * Check if this entry is deleted
@@ -303,6 +313,15 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
 	    myflags = FS_FLAG_NAME_ALLOC;
 	}
 
+	if (verbose)
+	    tsk_fprintf(stderr,
+		"ntfs_dent_idxentry: Entry Details of %s: Str Len: %"
+		PRIu16 "  Len to end after current: %" PRIu64
+		"  flags: %x\n", fs_dent->name, getu16(fs->endian,
+		    idxe->strlen),
+		(uint64_t) (endaddr_alloc - (uintptr_t) idxe -
+		    getu16(fs->endian, idxe->idxlen)), myflags);
+
 
 	if ((flags & myflags) == myflags) {
 	    int retval = action(fs, fs_dent, myflags, ptr);
@@ -325,29 +344,38 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
 	    (fs_dent->inode)) {
 	    int depth_added = 0;
 
-	    if ((dinfo->depth < MAX_DEPTH) &&
-		(DIR_STRSZ >
-		    strlen(dinfo->dirs) + strlen(fs_dent->name))) {
-		dinfo->didx[dinfo->depth] =
-		    &dinfo->dirs[strlen(dinfo->dirs)];
-		strncpy(dinfo->didx[dinfo->depth], fs_dent->name,
-		    DIR_STRSZ - strlen(dinfo->dirs));
-		strncat(dinfo->dirs, "/", DIR_STRSZ);
-		depth_added = 1;
-	    }
-	    dinfo->depth++;
+	    /* Make sure we do not get into an infinite loop */
+	    if (0 == tsk_list_find(*list_seen, fs_dent->inode)) {
+		if (tsk_list_add(list_seen, fs_dent->inode)) {
+		    fs_dent_free(fs_dent);
+		    return -1;
+		}
 
-	    if (ntfs_dent_walk_lcl(&(ntfs->fs_info), dinfo, fs_dent->inode,
-		    flags, action, ptr)) {
-		if (verbose)
-		    tsk_fprintf(stderr,
-			"Error recursing into directory\n");
-		tsk_error_reset();
-	    }
 
-	    dinfo->depth--;
-	    if (depth_added)
-		*dinfo->didx[dinfo->depth] = '\0';
+		if ((dinfo->depth < MAX_DEPTH) &&
+		    (DIR_STRSZ >
+			strlen(dinfo->dirs) + strlen(fs_dent->name))) {
+		    dinfo->didx[dinfo->depth] =
+			&dinfo->dirs[strlen(dinfo->dirs)];
+		    strncpy(dinfo->didx[dinfo->depth], fs_dent->name,
+			DIR_STRSZ - strlen(dinfo->dirs));
+		    strncat(dinfo->dirs, "/", DIR_STRSZ);
+		    depth_added = 1;
+		}
+		dinfo->depth++;
+
+		if (ntfs_dent_walk_lcl(&(ntfs->fs_info), dinfo, list_seen,
+			fs_dent->inode, flags, action, ptr)) {
+		    if (verbose)
+			tsk_fprintf(stderr,
+			    "Error recursing into directory\n");
+		    tsk_error_reset();
+		}
+
+		dinfo->depth--;
+		if (depth_added)
+		    *dinfo->didx[dinfo->depth] = '\0';
+	    }
 
 	}			/* end of recurse */
 
@@ -407,6 +435,7 @@ ntfs_fix_idxrec(NTFS_INFO * ntfs, ntfs_idxrec * idxrec, uint32_t len)
     /* sanity check so we don't run over in the next loop */
     if ((unsigned int) ((getu16(fs->endian, idxrec->upd_cnt) - 1) *
 	    ntfs->ssize_b) > len) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_INODE_INT;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "fix_idxrec: More Update Sequence Entries than idx record size");
@@ -436,6 +465,7 @@ ntfs_fix_idxrec(NTFS_INFO * ntfs, ntfs_idxrec * idxrec, uint32_t len)
 	    uint16_t cur_repl =
 		getu16(fs->endian, &upd->upd_seq + (i - 1) * 2);
 
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_INODE_INT;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"fix_idxrec: Incorrect update sequence value in index buffer\nUpdate Value: 0x%"
@@ -475,16 +505,32 @@ ntfs_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
     FS_DENT_WALK_FN action, void *ptr)
 {
     NTFS_DINFO dinfo;
+    TSK_LIST *list_seen = NULL;
+    uint8_t retval;
+
+    // clean up any error messages that are lying around
+    tsk_error_reset();
+
+    /* Sanity check on flags -- make sure at least one ALLOC is set */
+    if (((flags & FS_FLAG_NAME_ALLOC) == 0) &&
+	((flags & FS_FLAG_NAME_UNALLOC) == 0)) {
+	flags |= (FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC);
+    }
+
 
     memset(&dinfo, 0, sizeof(NTFS_DINFO));
+    retval = ntfs_dent_walk_lcl(fs, &dinfo, &list_seen, inum,
+	flags, action, ptr);
 
-    return ntfs_dent_walk_lcl(fs, &dinfo, inum, flags, action, ptr);
+    tsk_list_free(list_seen);
+    list_seen = NULL;
+    return retval;
 }
 
 
 static uint8_t
-ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
-    int flags, FS_DENT_WALK_FN action, void *ptr)
+ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, TSK_LIST ** list_seen,
+    INUM_T inum, int flags, FS_DENT_WALK_FN action, void *ptr)
 {
     NTFS_INFO *ntfs = (NTFS_INFO *) fs;
     FS_INODE *fs_inode;
@@ -501,6 +547,7 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
 
     /* sanity check */
     if (inum < fs->first_inum || inum > fs->last_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ntfs_dent_walk: inode value: %" PRIuINUM "\n", inum);
@@ -509,7 +556,8 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
 
     if (verbose)
 	tsk_fprintf(stderr,
-	    "ntfs_dent_walk: Processing directory %" PRIuINUM "\n", inum);
+	    "ntfs_dent_walk: Processing directory %" PRIuINUM
+	    " with flags %x\n", inum, flags);
 
     /* Get the inode and verify it has attributes */
     fs_inode = fs->inode_lookup(fs, inum);
@@ -519,6 +567,7 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
 	return 1;
     }
     if (!fs_inode->attr) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_INODE_INT;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "dent_walk: Error: Directory address %" PRIuINUM
@@ -541,6 +590,7 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
     }
 
     if (fs_data_root->flags & FS_DATA_NONRES) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_INODE_INT;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "dent_walk: $IDX_ROOT is not resident - it should be");
@@ -551,6 +601,7 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
 
     /* Verify that the attribute type is $FILE_NAME */
     if (getu32(fs->endian, idxroot->type) == 0) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_INODE_INT;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "dent_walk: Attribute type in index root is 0");
@@ -558,6 +609,7 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
 	return 1;
     }
     else if (getu32(fs->endian, idxroot->type) != NTFS_ATYPE_FNAME) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_INODE_INT;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ERROR: Directory index is sorted by type: %" PRIu32
@@ -674,7 +726,12 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
     }
 
     /* Now we return to processing the Index Root Attribute */
-    retval = ntfs_dent_idxentry(ntfs, dinfo, idxe,
+    if (verbose)
+	tsk_fprintf(stderr,
+	    "ntfs_dent_walk: Processing $IDX_ROOT of inum %" PRIuINUM "\n",
+	    inum);
+
+    retval = ntfs_dent_idxentry(ntfs, dinfo, list_seen, idxe,
 	getu32(fs->endian,
 	    idxelist->buf_off) -
 	getu32(fs->endian, idxelist->begin_off),
@@ -701,6 +758,7 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
     if (!fs_data_alloc) {
 	int retval;
 	if (getu32(fs->endian, idxelist->flags) & NTFS_IDXELIST_CHILD) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_INODE_INT;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"Error: $IDX_ROOT says there should be children, but there isn't");
@@ -715,6 +773,7 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
 
 
     if (fs_data_alloc->flags & FS_DATA_RES) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_INODE_INT;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "$IDX_ALLOC is Resident - it shouldn't be");
@@ -753,6 +812,7 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
 	free(idxalloc);
 	fs_inode_free(fs_inode);
 
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_FWALK;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Error reading directory contents: %" PRIuINUM "\n", inum);
@@ -830,7 +890,7 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
 	list_len = (uint32_t) ((uintptr_t) idxrec - (uintptr_t) idxe);
 
 	/* process the list of index entries */
-	retval = ntfs_dent_idxentry(ntfs, dinfo, idxe, list_len,
+	retval = ntfs_dent_idxentry(ntfs, dinfo, list_seen, idxe, list_len,
 	    getu32(fs->endian,
 		idxelist->end_off) -
 	    getu32(fs->endian, idxelist->begin_off), flags, action, ptr);
@@ -877,7 +937,7 @@ ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
 	    (uintptr_t) idxe;
 
 	/* process the list of index entries */
-	retval = ntfs_dent_idxentry(ntfs, dinfo, idxe, list_len,
+	retval = ntfs_dent_idxentry(ntfs, dinfo, list_seen, idxe, list_len,
 	    getu32(fs->endian,
 		idxelist->end_off) -
 	    getu32(fs->endian, idxelist->begin_off), flags, action, ptr);
@@ -927,6 +987,7 @@ ntfs_find_file_rec(FS_INFO * fs, NTFS_DINFO * dinfo, FS_DENT * fs_dent,
 
     if (fs_name->par_inode < fs->first_inum ||
 	fs_name->par_inode > fs->last_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "invalid inode value: %" PRIuINUM "\n", fs_name->par_inode);
@@ -1062,6 +1123,7 @@ ntfs_find_file(FS_INFO * fs, INUM_T inode_toid, uint32_t type_toid,
 
     /* sanity check */
     if (inode_toid < fs->first_inum || inode_toid > fs->last_inum) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_FS_ARG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "ntfs_find_file: invalid inode value: %" PRIuINUM "\n",
@@ -1109,6 +1171,7 @@ ntfs_find_file(FS_INFO * fs, INUM_T inode_toid, uint32_t type_toid,
 	    fs_data =
 		fs_data_lookup(fs_dent->fsi->attr, type_toid, id_toid);
 	if (!fs_data) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_FS_INODE_INT;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"find_file: Type %" PRIu32 " Id %" PRIu16
