@@ -1,5 +1,5 @@
 /*
- * $Date: 2006/08/25 18:28:31 $
+ * $Date: 2006/12/07 16:38:18 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -11,6 +11,8 @@
 
 #if defined(USE_LIBAFF)
 
+typedef int bool;
+
 #include "afflib.h"
 #include "aff.h"
 
@@ -31,12 +33,12 @@ aff_read_random(IMG_INFO * img_info, OFF_T vol_offset, char *buf,
     if (aff_info->seek_pos != tot_offset) {
 	if ((off_t) af_seek(aff_info->af_file, tot_offset, SEEK_SET) !=
 	    tot_offset) {
+	    tsk_error_reset();
 	    // @@@ ADD more specific error messages
 	    tsk_errno = TSK_ERR_IMG_SEEK;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"aff_read_random - %" PRIuOFF " - %s", tot_offset,
 		strerror(errno));
-	    tsk_errstr2[0] = '\0';
 	    return -1;
 
 	}
@@ -46,11 +48,11 @@ aff_read_random(IMG_INFO * img_info, OFF_T vol_offset, char *buf,
     cnt = af_read(aff_info->af_file, (unsigned char *) buf, len);
     if (cnt == -1) {
 	// @@@ Add more specific error message
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_IMG_READ;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "aff_read_random - offset: %" PRIuOFF " - len: %"
 	    PRIuOFF " - %s", tot_offset, len, strerror(errno));
-	tsk_errstr2[0] = '\0';
 	return -1;
     }
 
@@ -211,9 +213,9 @@ aff_open(const char **images, IMG_INFO * next)
     int type;
 
     if (next != NULL) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_IMG_LAYERS;
 	snprintf(tsk_errstr, TSK_ERRSTR_L, "AFF must be lowest layer");
-	tsk_errstr2[0] = '\0';
 	return NULL;
     }
 
@@ -231,7 +233,7 @@ aff_open(const char **images, IMG_INFO * next)
     img_info->imgstat = aff_imgstat;
 
 
-    type = af_identify_type(images[0]);
+    type = af_identify_file_type(images[0], 1);
     if ((type == AF_IDENTIFY_ERR) || (type == AF_IDENTIFY_NOEXIST)) {
 	if (verbose) {
 	    tsk_fprintf(stderr,
@@ -239,7 +241,7 @@ aff_open(const char **images, IMG_INFO * next)
 		images[0]);
 	    perror("aff_open");
 	}
-	tsk_errno = TSK_ERR_IMG_OPEN;
+	tsk_error_reset();
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "aff_open file: %s: Error checking type", images[0]);
 	tsk_errstr2[0] = '\0';
@@ -259,10 +261,10 @@ aff_open(const char **images, IMG_INFO * next)
 //      img_info->itype = AFF_AFF;
     //   }
     else {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_IMG_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "aff_open: Not an AFF, AFD, or AFM file");
-	tsk_errstr2[0] = '\0';
 	free(aff_info);
 	if (verbose)
 	    tsk_fprintf(stderr, "Not an AFF/AFD/AFM file\n");
@@ -272,11 +274,11 @@ aff_open(const char **images, IMG_INFO * next)
 
     aff_info->af_file = af_open(images[0], O_RDONLY, 0);
     if (!aff_info->af_file) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_IMG_OPEN;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "aff_open file: %s: Error opening - %s", images[0],
 	    strerror(errno));
-	tsk_errstr2[0] = '\0';
 	free(aff_info);
 	if (verbose) {
 	    tsk_fprintf(stderr, "Error opening AFF/AFD/AFM file\n");
