@@ -1,5 +1,5 @@
 /*
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/12/07 16:38:18 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -19,6 +19,7 @@
 #include "split.h"
 
 #if defined(USE_LIBAFF)
+typedef int bool;
 #include "aff.h"
 #endif
 
@@ -36,15 +37,20 @@
  * The highest layer is returned or NULL if an error occurs
  */
 IMG_INFO *
-img_open(const TCHAR * type, const int num_img, const TCHAR ** images)
+img_open(const TSK_TCHAR * type, const int num_img,
+    const TSK_TCHAR ** images)
 {
     IMG_INFO *img_info = NULL;
-    TCHAR *tp, *next;
-    TCHAR type_lcl[128], *type_lcl_p;
-    const TCHAR **img_tmp;
+    TSK_TCHAR *tp, *next;
+    TSK_TCHAR type_lcl[128], *type_lcl_p;
+    const TSK_TCHAR **img_tmp;
     int num_img_tmp = num_img;
 
+    // Get rid of any old error messages laying around
+    tsk_error_reset();
+
     if ((num_img == 0) || (images[0] == NULL)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_IMG_NOFILE;
 	snprintf(tsk_errstr, TSK_ERRSTR_L, "img_open");
 	tsk_errstr2[0] = '\0';
@@ -53,8 +59,8 @@ img_open(const TCHAR * type, const int num_img, const TCHAR ** images)
 
     if (verbose)
 	TFPRINTF(stderr,
-	    _T("img_open: Type: %s   NumImg: %d  Img1: %s\n"),
-	    (type ? type : _T("n/a")), num_img, images[0]);
+	    _TSK_T("img_open: Type: %s   NumImg: %d  Img1: %s\n"),
+	    (type ? type : _TSK_T("n/a")), num_img, images[0]);
 
     // only the first in list (lowest) layer gets the files
     img_tmp = images;
@@ -73,20 +79,22 @@ img_open(const TCHAR * type, const int num_img, const TCHAR ** images)
 	if (TSTAT(images[0], &stat_buf) == -1) {
 	    // special case to handle windows objects
 #ifdef TSK_WIN32
-	    if ((images[0][0] == _T('\\')) && (images[0][1] == _T('\\')) &&
-		(images[0][2] == _T('.')) && (images[0][3] == _T('\\'))) {
+	    if ((images[0][0] == _TSK_T('\\'))
+		&& (images[0][1] == _TSK_T('\\'))
+		&& (images[0][2] == _TSK_T('.'))
+		&& (images[0][3] == _TSK_T('\\'))) {
 		if (verbose)
 		    TFPRINTF(stderr,
-			_T
+			_TSK_T
 			("img_open: Ignoring stat error because of windows object: %s\n"),
 			images[0]);
 	    }
 	    else {
 #endif
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_IMG_STAT;
 		snprintf(tsk_errstr, TSK_ERRSTR_L, "%s : %s", images[0],
 		    strerror(errno));
-		tsk_errstr2[0] = '\0';
 		return NULL;
 #ifdef TSK_WIN32
 	    }
@@ -116,9 +124,9 @@ img_open(const TCHAR * type, const int num_img, const TCHAR ** images)
 	    else {
 		img_set->close(img_set);
 		img_info->close(img_info);
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_IMG_UNKTYPE;
 		snprintf(tsk_errstr, TSK_ERRSTR_L, "EWF or %s", set);
-		tsk_errstr2[0] = '\0';
 		return NULL;
 	    }
 	}
@@ -160,11 +168,11 @@ img_open(const TCHAR * type, const int num_img, const TCHAR ** images)
     type_lcl_p = type_lcl;
 
     /* We parse this and go up in the layers */
-    tp = TSTRTOK(type_lcl, _T(","));
+    tp = TSTRTOK(type_lcl, _TSK_T(","));
     while (tp != NULL) {
 	uint8_t imgtype;
 
-	next = TSTRTOK(NULL, _T(","));
+	next = TSTRTOK(NULL, _TSK_T(","));
 
 	imgtype = img_parse_type(type);
 	switch (imgtype) {
@@ -212,9 +220,9 @@ img_open(const TCHAR * type, const int num_img, const TCHAR ** images)
 #endif
 
 	default:
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_IMG_UNSUPTYPE;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L, "%s", tp);
-	    tsk_errstr2[0] = '\0';
 	    return NULL;
 	}
 
