@@ -1,5 +1,5 @@
 /*
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/12/07 22:03:05 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -56,10 +56,10 @@ raw_read_random(IMG_INFO * img_info, OFF_T vol_offset, char *buf,
 
 		if (0xFFFFFFFF == SetFilePointer(raw_info->fd, lo, &hi,
 			FILE_BEGIN)) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_IMG_SEEK;
 		    snprintf(tsk_errstr, TSK_ERRSTR_L,
 			"raw_read_random - %" PRIuOFF, tot_offset);
-		    tsk_errstr2[0] = '\0';
 		    return -1;
 		}
 		raw_info->seek_pos = tot_offset;
@@ -67,11 +67,11 @@ raw_read_random(IMG_INFO * img_info, OFF_T vol_offset, char *buf,
 
 	    if (FALSE == ReadFile(raw_info->fd, buf, (DWORD) len,
 		    &nread, NULL)) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_IMG_READ;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "raw_read_random - offset: %" PRIuOFF " - len: %"
 		    PRIuOFF, tot_offset, len);
-		tsk_errstr2[0] = '\0';
 		return -1;
 	    }
 	    cnt = (SSIZE_T) nread;
@@ -79,11 +79,11 @@ raw_read_random(IMG_INFO * img_info, OFF_T vol_offset, char *buf,
 #else
 	if (raw_info->seek_pos != tot_offset) {
 	    if (lseek(raw_info->fd, tot_offset, SEEK_SET) != tot_offset) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_IMG_SEEK;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "raw_read_random - %" PRIuOFF " - %s",
 		    tot_offset, strerror(errno));
-		tsk_errstr2[0] = '\0';
 		return -1;
 	    }
 	    raw_info->seek_pos = tot_offset;
@@ -91,11 +91,11 @@ raw_read_random(IMG_INFO * img_info, OFF_T vol_offset, char *buf,
 
 	cnt = read(raw_info->fd, buf, len);
 	if (cnt == -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_IMG_READ;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"raw_read_random - offset: %" PRIuOFF " - len: %"
 		PRIuOFF " - %s", tot_offset, len, strerror(errno));
-	    tsk_errstr2[0] = '\0';
 	    return -1;
 	}
 #endif
@@ -129,6 +129,7 @@ raw_close(IMG_INFO * img_info)
 #else
     close(raw_info->fd);
 #endif
+    free(raw_info);
 }
 
 
@@ -138,7 +139,7 @@ raw_close(IMG_INFO * img_info)
  *  to test for a raw file
  */
 IMG_INFO *
-raw_open(const TCHAR ** images, IMG_INFO * next)
+raw_open(const TSK_TCHAR ** images, IMG_INFO * next)
 {
     IMG_RAW_INFO *raw_info;
     IMG_INFO *img_info;
@@ -168,18 +169,20 @@ raw_open(const TCHAR ** images, IMG_INFO * next)
 	int is_winobj = 0;
 
 #ifdef TSK_WIN32
-	if ((images[0][0] == _T('\\')) && (images[0][1] == _T('\\')) &&
-	    (images[0][2] == _T('.')) && (images[0][3] == _T('\\'))) {
+	if ((images[0][0] == _TSK_T('\\'))
+	    && (images[0][1] == _TSK_T('\\'))
+	    && (images[0][2] == _TSK_T('.'))
+	    && (images[0][3] == _TSK_T('\\'))) {
 	    is_winobj = 1;
 	}
 #endif
 	if (is_winobj == 0) {
 	    /* Exit if we are given a directory */
 	    if (TSTAT(images[0], &stat_buf) == -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_IMG_STAT;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "raw_open directory check: %s", strerror(errno));
-		tsk_errstr2[0] = '\0';
 		return NULL;
 	    }
 	    else if ((stat_buf.st_mode & S_IFMT) == S_IFDIR) {
@@ -187,10 +190,10 @@ raw_open(const TCHAR ** images, IMG_INFO * next)
 		    tsk_fprintf(stderr,
 			"raw_open: image %s is a directory\n", images[0]);
 
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_IMG_MAGIC;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "raw_open: Image is a directory");
-		tsk_errstr2[0] = '\0';
 		return NULL;
 	    }
 	}
@@ -202,11 +205,11 @@ raw_open(const TCHAR ** images, IMG_INFO * next)
 	    if ((raw_info->fd = CreateFile(images[0], GENERIC_READ,
 			FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0)) ==
 		INVALID_HANDLE_VALUE) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_IMG_OPEN;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "raw_open file: %s msg: %d", images[0],
 		    GetLastError());
-		tsk_errstr2[0] = '\0';
 		return NULL;
 	    }
 
@@ -216,11 +219,11 @@ raw_open(const TCHAR ** images, IMG_INFO * next)
 	    if (is_winobj == 0) {
 		dwLo = GetFileSize(raw_info->fd, &dwHi);
 		if (dwLo == 0xffffffff) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_IMG_OPEN;
 		    snprintf(tsk_errstr, TSK_ERRSTR_L,
 			"raw_open file: %s GetFileSize: %d", images[0],
 			GetLastError());
-		    tsk_errstr2[0] = '\0';
 		    return NULL;
 		}
 		img_info->size = dwLo | ((OFF_T) dwHi << 32);
@@ -233,11 +236,11 @@ raw_open(const TCHAR ** images, IMG_INFO * next)
 			IOCTL_DISK_GET_DRIVE_GEOMETRY,	// operation to perform
 			NULL, 0, &pdg, sizeof(pdg), &junk,
 			(LPOVERLAPPED) NULL)) {
+		    tsk_error_reset();
 		    tsk_errno = TSK_ERR_IMG_OPEN;
 		    snprintf(tsk_errstr, TSK_ERRSTR_L,
 			"raw_open file: %s DeviceIoControl: %d", images[0],
 			GetLastError());
-		    tsk_errstr2[0] = '\0';
 		    return NULL;
 		}
 
@@ -250,10 +253,10 @@ raw_open(const TCHAR ** images, IMG_INFO * next)
 	}
 #else
 	if ((raw_info->fd = open(images[0], O_RDONLY)) < 0) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_IMG_OPEN;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L, "raw_open file: %s msg: %s",
 		images[0], strerror(errno));
-	    tsk_errstr2[0] = '\0';
 	    return NULL;
 	}
 
