@@ -1,5 +1,5 @@
 /*
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/11/29 22:02:15 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -55,20 +55,20 @@ split_read_segment(IMG_SPLIT_INFO * split_info, int idx, char *buf,
 #ifdef TSK_WIN32
 	if ((cimg->fd = CreateFile(split_info->images[idx], GENERIC_READ,
 		    0, 0, OPEN_EXISTING, 0, 0)) == INVALID_HANDLE_VALUE) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_IMG_OPEN;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"split_read_random file: %s msg: %s",
 		split_info->images[idx], strerror(errno));
-	    tsk_errstr2[0] = '\0';
 	    return -1;
 	}
 #else
 	if ((cimg->fd = open(split_info->images[idx], O_RDONLY)) < 0) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_IMG_OPEN;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"split_read_random file: %s msg: %s",
 		split_info->images[idx], strerror(errno));
-	    tsk_errstr2[0] = '\0';
 	    return -1;
 	}
 #endif
@@ -95,21 +95,21 @@ split_read_segment(IMG_SPLIT_INFO * split_info, int idx, char *buf,
 
 	    if (0xFFFFFFFF == SetFilePointer(cimg->fd, lo, &hi,
 		    FILE_BEGIN)) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_IMG_SEEK;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "split_read_random - %" PRIuOFF, rel_offset);
-		tsk_errstr2[0] = '\0';
 		return -1;
 	    }
 	    cimg->seek_pos = rel_offset;
 	}
 
 	if (FALSE == ReadFile(cimg->fd, buf, (DWORD) len, &nread, NULL)) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_IMG_READ;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"split_read_random - offset: %" PRIuOFF " - len: %"
 		PRIuOFF, rel_offset, len);
-	    tsk_errstr2[0] = '\0';
 	    return -1;
 	}
 	cnt = (SSIZE_T) nread;
@@ -117,11 +117,11 @@ split_read_segment(IMG_SPLIT_INFO * split_info, int idx, char *buf,
 #else
     if (cimg->seek_pos != rel_offset) {
 	if (lseek(cimg->fd, rel_offset, SEEK_SET) != rel_offset) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_IMG_SEEK;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"split_read_random - %s - %" PRIuOFF " - %s",
 		split_info->images[idx], rel_offset, strerror(errno));
-	    tsk_errstr2[0] = '\0';
 	    return -1;
 	}
 	cimg->seek_pos = rel_offset;
@@ -129,11 +129,11 @@ split_read_segment(IMG_SPLIT_INFO * split_info, int idx, char *buf,
 
     cnt = read(cimg->fd, buf, len);
     if (cnt == -1) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_IMG_READ;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "split_read_random - offset: %" PRIuOFF
 	    " - len: %" PRIuOFF " - %s", rel_offset, len, strerror(errno));
-	tsk_errstr2[0] = '\0';
 	return -1;
     }
 #endif
@@ -240,11 +240,11 @@ split_read_random(IMG_INFO * img_info, OFF_T vol_offset, char *buf,
 	}
     }
 
+    tsk_error_reset();
     tsk_errno = TSK_ERR_IMG_READ_OFF;
     snprintf(tsk_errstr, TSK_ERRSTR_L,
 	"split_read_random - %" PRIuOFF " - %s", tot_offset,
 	strerror(errno));
-    tsk_errstr2[0] = '\0';
     return -1;
 }
 
@@ -299,16 +299,16 @@ split_close(IMG_INFO * img_info)
  * Return IMG_INFO or NULL if an error occurs
  */
 IMG_INFO *
-split_open(int num_img, const TCHAR ** images, IMG_INFO * next)
+split_open(int num_img, const TSK_TCHAR ** images, IMG_INFO * next)
 {
     IMG_SPLIT_INFO *split_info;
     IMG_INFO *img_info;
     int i;
 
     if (next != NULL) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_IMG_LAYERS;
 	snprintf(tsk_errstr, TSK_ERRSTR_L, "split must be lowest layer");
-	tsk_errstr2[0] = '\0';
 	return NULL;
     }
 
@@ -358,10 +358,10 @@ split_open(int num_img, const TCHAR ** images, IMG_INFO * next)
 
 	split_info->cptr[i] = -1;
 	if (TSTAT(images[i], &sb) == -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_IMG_STAT;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L, "split_open - %s - %s",
 		images[i], strerror(errno));
-	    tsk_errstr2[0] = '\0';
 	    free(split_info->max_off);
 	    free(split_info->cptr);
 	    free(split_info);
@@ -372,10 +372,10 @@ split_open(int num_img, const TCHAR ** images, IMG_INFO * next)
 		tsk_fprintf(stderr,
 		    "split_open: image %s is a directory\n", images[i]);
 
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_IMG_MAGIC;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"split_open: Image is a directory");
-	    tsk_errstr2[0] = '\0';
 	    return NULL;
 	}
 
