@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/12/07 16:38:18 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -106,34 +106,34 @@ bsd_load_table(MM_INFO * mm)
     cnt = mm_read_block_nobuf
 	(mm, (char *) &dlabel, sizeof(dlabel), BSD_PART_SOFFSET);
     if (cnt != sizeof(dlabel)) {
+	if (cnt != -1) {
+	    tsk_error_reset();
+	    tsk_errno = TSK_ERR_MM_READ;
+	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "BSD Disk Label in Sector: %" PRIuDADDR, laddr);
-	if (cnt != -1) {
-	    tsk_errno = TSK_ERR_MM_READ;
-	    tsk_errstr[0] = '\0';
-	}
 	return 1;
     }
 
     /* Check the magic  */
     if (mm_guessu32(mm, dlabel.magic, BSD_MAGIC)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "BSD partition table (magic #1) (Sector: %"
 	    PRIuDADDR ") %" PRIx32, laddr, getu32(mm->endian,
 		dlabel.magic));
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
     /* Check the second magic value */
     if (getu32(mm->endian, dlabel.magic2) != BSD_MAGIC) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "BSD disk label (magic #2) (Sector: %"
 	    PRIuDADDR ")  %" PRIx32, laddr, getu32(mm->endian,
 		dlabel.magic2));
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -156,10 +156,10 @@ bsd_load_table(MM_INFO * mm)
 	    continue;
 
 	if (part_start > max_addr) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_BLK_NUM;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"bsd_load_table: Starting sector too large for image");
-	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 
@@ -189,18 +189,18 @@ bsd_part_walk(MM_INFO * mm, PNUM_T start, PNUM_T last, int flags,
     PNUM_T cnt = 0;
 
     if (start < mm->first_part || start > mm->last_part) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "bsd_part_walk: Start partition: %" PRIuPNUM "", start);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
     if (last < mm->first_part || last > mm->last_part) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "bsd_part_walk: End partition: %" PRIuPNUM "", last);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -242,7 +242,12 @@ bsd_close(MM_INFO * mm)
 MM_INFO *
 bsd_open(IMG_INFO * img_info, DADDR_T offset)
 {
-    MM_INFO *mm = (MM_INFO *) mymalloc(sizeof(*mm));
+    MM_INFO *mm;
+
+    // clean up any errors that are lying around
+    tsk_error_reset();
+
+    mm = (MM_INFO *) mymalloc(sizeof(*mm));
     if (mm == NULL)
 	return NULL;
 
