@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/09/02 23:34:04 $
+ * $Date: 2006/04/06 01:15:57 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -70,7 +70,7 @@ extern "C" {
     } bsd_disklabel;
 
 #define BSD_MAGIC	0x82564557
-#define BSD_PART_OFFSET	1
+#define BSD_PART_SOFFSET	1
 
 #ifdef __cplusplus
 }
