@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/08/25 18:28:42 $
+ * $Date: 2006/12/07 16:38:18 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -691,17 +691,18 @@ dos_load_ext_table(MM_INFO * mm, DADDR_T sect_cur, DADDR_T sect_ext_base,
     cnt = mm_read_block_nobuf
 	(mm, (char *) &sect, sizeof(sect), (DADDR_T) sect_cur);
     if (cnt != sizeof(sect)) {
+	if (cnt != -1) {
+	    tsk_error_reset();
+	    tsk_errno = TSK_ERR_MM_READ;
+	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "Extended DOS table sector %" PRIuDADDR, sect_cur);
-	if (cnt != -1) {
-	    tsk_errno = TSK_ERR_MM_READ;
-	    tsk_errstr[0] = '\0';
-	}
 	return 1;
     }
 
     /* Sanity Check */
     if (getu16(mm->endian, sect.magic) != DOS_MAGIC) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Extended DOS partition table in sector %"
@@ -751,10 +752,10 @@ dos_load_ext_table(MM_INFO * mm, DADDR_T sect_cur, DADDR_T sect_ext_base,
 	     * extended partition) */
 
 	    if (sect_ext_base + part_start > max_addr) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_MM_BLK_NUM;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "dos_load_ext_table: Starting sector too large for image");
-		tsk_errstr2[0] = '\0';
 		if (verbose)
 		    tsk_fprintf(stderr,
 			"Starting sector %" PRIuDADDR
@@ -782,10 +783,10 @@ dos_load_ext_table(MM_INFO * mm, DADDR_T sect_cur, DADDR_T sect_ext_base,
 	     * starting location */
 
 	    if (sect_cur + part_start > max_addr) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_MM_BLK_NUM;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "dos_load_ext_table: Starting sector too large for image");
-		tsk_errstr2[0] = '\0';
 		if (verbose)
 		    tsk_fprintf(stderr,
 			"Starting sector %" PRIuDADDR
@@ -833,23 +834,23 @@ dos_load_prim_table(MM_INFO * mm, uint8_t test)
 	(mm, (char *) &sect, sizeof(sect), DOS_PART_SOFFSET);
 
     if (cnt != sizeof(sect)) {
+	if (cnt != -1) {
+	    tsk_error_reset();
+	    tsk_errno = TSK_ERR_MM_READ;
+	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "Primary DOS table sector %" PRIuDADDR, taddr);
-	if (cnt != -1) {
-	    tsk_errno = TSK_ERR_MM_READ;
-	    tsk_errstr[0] = '\0';
-	}
 	return 1;
     }
 
 
     /* Sanity Check */
     if (mm_guessu16(mm, sect.magic, DOS_MAGIC)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "File is not a DOS partition (invalid primary magic) (Sector: %"
 	    PRIuDADDR ")", taddr);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -864,31 +865,31 @@ dos_load_prim_table(MM_INFO * mm, uint8_t test)
 		"dos_load_prim_table: Testing FAT/NTFS conditions\n");
 
 	if (strncmp("MSDOS", sect.oemname, 5) == 0) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_MAGIC;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"dos_load_prim_table: MSDOS OEM name exists");
-	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 	else if (strncmp("MSWIN", sect.oemname, 5) == 0) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_MAGIC;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"dos_load_prim_table: MSWIN OEM name exists");
-	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 	else if (strncmp("NTFS", sect.oemname, 4) == 0) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_MAGIC;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"dos_load_prim_table: NTFS OEM name exists");
-	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 	else if (strncmp("FAT", sect.oemname, 4) == 0) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_MAGIC;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"dos_load_prim_table: FAT OEM name exists");
-	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
     }
@@ -919,10 +920,10 @@ dos_load_prim_table(MM_INFO * mm, uint8_t test)
 	    continue;
 
 	if (part_start > max_addr) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_BLK_NUM;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"dos_load_prim_table: Starting sector too large for image");
-	    tsk_errstr2[0] = '\0';
 	    if (verbose)
 		tsk_fprintf(stderr,
 		    "Starting sector %" PRIu32 " too large for image\n",
@@ -933,10 +934,10 @@ dos_load_prim_table(MM_INFO * mm, uint8_t test)
 #if 0
 // I'm not sure if this is too strict ...
 	else if ((part_start + part_size) > max_addr) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_BLK_NUM
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"dos_load_prim_table: Partition ends after image");
-	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 #endif
@@ -965,10 +966,10 @@ dos_load_prim_table(MM_INFO * mm, uint8_t test)
 	if (verbose)
 	    tsk_fprintf(stderr, "dos_load_prim: No valid entries\n");
 
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "dos_load_prim_table: No valid entries in primary table");
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -989,17 +990,17 @@ dos_part_walk(MM_INFO * mm, PNUM_T start, PNUM_T last, int flags,
     PNUM_T cnt = 0;
 
     if (start < mm->first_part || start > mm->last_part) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "dos_part_walk: Starting partition: %" PRIuPNUM "", start);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
     if (last < mm->first_part || last > mm->last_part) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "dos_part_walk: Ending partition: %" PRIuPNUM "", last);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -1045,7 +1046,12 @@ dos_close(MM_INFO * mm)
 MM_INFO *
 dos_open(IMG_INFO * img_info, DADDR_T offset, uint8_t test)
 {
-    MM_INFO *mm = (MM_INFO *) mymalloc(sizeof(*mm));
+    MM_INFO *mm;
+
+    // clean up any errors that are lying around
+    tsk_error_reset();
+
+    mm = (MM_INFO *) mymalloc(sizeof(*mm));
     if (mm == NULL)
 	return NULL;
 
