@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/11/29 22:02:16 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -46,8 +46,8 @@ gpt_load_table(MM_INFO * mm)
     /* if -1, then tsk_errno is already set */
     if (cnt != sizeof(dos_part)) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "Error reading DOS safety partition table in Sector: %"
@@ -57,20 +57,20 @@ gpt_load_table(MM_INFO * mm)
 
     /* Sanity Check */
     if (mm_guessu16(mm, dos_part.magic, DOS_MAGIC)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Missing DOS safety partition (invalid magic) (Sector: %"
 	    PRIuDADDR ")", taddr);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
     if (dos_part.ptable[0].ptype != GPT_DOS_TYPE) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Missing DOS safety partition (invalid type in table: %d)",
 	    dos_part.ptable[0].ptype);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -88,8 +88,8 @@ gpt_load_table(MM_INFO * mm)
 	(mm, (char *) &head, sizeof(head), GPT_PART_SOFFSET + 1);
     if (cnt != sizeof(head)) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "GPT Header structure in Sector: %" PRIuDADDR, taddr + 1);
@@ -98,10 +98,10 @@ gpt_load_table(MM_INFO * mm)
 
 
     if (getu64(mm->endian, &head.signature) != GPT_HEAD_SIG) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "GPT Header: %" PRIx64, getu64(mm->endian, &head.signature));
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -118,11 +118,11 @@ gpt_load_table(MM_INFO * mm)
     /* Allocate a buffer for each table entry */
     ent_size = getu32(mm->endian, &head.tab_size_b);
     if (ent_size < sizeof(gpt_entry)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Header reports partition entry size of %" PRIu32
 	    " and not %d", ent_size, sizeof(gpt_entry));
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -151,8 +151,8 @@ gpt_load_table(MM_INFO * mm)
 	    getu64(mm->endian, &head.tab_start_lba) + a);
 	if (cnt != mm->block_size) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_MM_READ;
-		tsk_errstr[0] = '\0';
 	    }
 	    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		"Error reading GPT partition table sector : %"
@@ -181,10 +181,10 @@ gpt_load_table(MM_INFO * mm)
 		continue;
 
 	    if (getu64(mm->endian, ent->start_lba) > max_addr) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_MM_BLK_NUM;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "gpt_load_table: Starting sector too large for image");
-		tsk_errstr2[0] = '\0';
 		return 1;
 	    }
 
@@ -235,18 +235,18 @@ gpt_part_walk(MM_INFO * mm, PNUM_T start, PNUM_T last, int flags,
     unsigned int cnt = 0;
 
     if (start < mm->first_part || start > mm->last_part) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Starting partition: %" PRIuPNUM "", start);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
     if (last < mm->first_part || last > mm->last_part) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "Ending partition: %" PRIuPNUM "", last);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -279,7 +279,12 @@ gpt_close(MM_INFO * mm)
 MM_INFO *
 gpt_open(IMG_INFO * img_info, DADDR_T offset)
 {
-    MM_INFO *mm = (MM_INFO *) mymalloc(sizeof(*mm));
+    MM_INFO *mm;
+
+    // clean up any errors that are lying around
+    tsk_error_reset();
+
+    mm = (MM_INFO *) mymalloc(sizeof(*mm));
     if (mm == NULL)
 	return NULL;
 
