@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/12/07 16:38:18 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -58,8 +58,8 @@ mac_load_table(MM_INFO * mm)
 	/* If -1, then tsk_errno is already set */
 	if (cnt != sizeof(part)) {
 	    if (cnt != -1) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_MM_READ;
-		tsk_errstr[0] = '\0';
 	    }
 	    snprintf(tsk_errstr2, TSK_ERRSTR_L,
 		"MAC Partition entry %" PRIuDADDR, taddr + idx);
@@ -71,12 +71,12 @@ mac_load_table(MM_INFO * mm)
 	if (idx == 0) {
 	    /* Set the endian ordering the first time around */
 	    if (mm_guessu16(mm, part.magic, MAC_MAGIC)) {
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_MM_MAGIC;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "Mac partition table entry (Sector: %"
 		    PRIuDADDR ") %" PRIx16,
 		    (taddr + idx), getu16(mm->endian, part.magic));
-		tsk_errstr2[0] = '\0';
 		return 1;
 	    }
 
@@ -84,12 +84,12 @@ mac_load_table(MM_INFO * mm)
 	    max_part = getu32(mm->endian, part.pmap_size);
 	}
 	else if (getu16(mm->endian, part.magic) != MAC_MAGIC) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_MAGIC;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"Mac partition table entry (Sector: %"
 		PRIuDADDR ") %" PRIx16, (taddr + idx),
 		getu16(mm->endian, part.magic));
-	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 
@@ -107,10 +107,10 @@ mac_load_table(MM_INFO * mm)
 	    continue;
 
 	if (part_start > max_addr) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_BLK_NUM;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"mac_load_table: Starting sector too large for image");
-	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 
@@ -151,18 +151,18 @@ mac_part_walk(MM_INFO * mm, PNUM_T start, PNUM_T last, int flags,
     unsigned int cnt = 0;
 
     if (start < mm->first_part || start > mm->last_part) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "mac_part_walk: Start partition: %" PRIuPNUM "", start);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
     if (last < mm->first_part || last > mm->last_part) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "mac_part_walk: Ending partition: %" PRIuPNUM "", last);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -199,7 +199,12 @@ mac_close(MM_INFO * mm)
 MM_INFO *
 mac_open(IMG_INFO * img_info, DADDR_T offset)
 {
-    MM_INFO *mm = (MM_INFO *) mymalloc(sizeof(*mm));
+    MM_INFO *mm;
+
+    // clean up any errors that are lying around
+    tsk_error_reset();
+
+    mm = (MM_INFO *) mymalloc(sizeof(*mm));
     if (mm == NULL)
 	return NULL;
 
