@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/11/29 22:02:16 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -22,7 +22,7 @@
  * Offset is the byte offset to the start of the volume system
  */
 MM_INFO *
-mm_open(IMG_INFO * img_info, DADDR_T offset, const TCHAR * type)
+mm_open(IMG_INFO * img_info, DADDR_T offset, const TSK_TCHAR * type)
 {
     /* Autodetect mode 
      * We need to try all of them in case there are multiple 
@@ -54,6 +54,7 @@ mm_open(IMG_INFO * img_info, DADDR_T offset, const TCHAR * type)
 	       else {
 	       mm_set->close(mm_set);
 	       mm_info->close(mm_info);
+	       tsk_error_reset();
 	       tsk_errno = TSK_ERR_MM_UNKTYPE;
 	       snprintf(tsk_errstr, TSK_ERRSTR_L,
 	       "BSD or %s at %" PRIuDADDR, set, offset);
@@ -73,10 +74,10 @@ mm_open(IMG_INFO * img_info, DADDR_T offset, const TCHAR * type)
 	    else {
 		mm_set->close(mm_set);
 		mm_info->close(mm_info);
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_MM_UNKTYPE;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "GPT or %s at %" PRIuDADDR, set, offset);
-		tsk_errstr2[0] = '\0';
 		return NULL;
 	    }
 	}
@@ -92,10 +93,10 @@ mm_open(IMG_INFO * img_info, DADDR_T offset, const TCHAR * type)
 	    else {
 		mm_set->close(mm_set);
 		mm_info->close(mm_info);
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_MM_UNKTYPE;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "Sun or %s at %" PRIuDADDR, set, offset);
-		tsk_errstr2[0] = '\0';
 		return NULL;
 	    }
 	}
@@ -111,10 +112,10 @@ mm_open(IMG_INFO * img_info, DADDR_T offset, const TCHAR * type)
 	    else {
 		mm_set->close(mm_set);
 		mm_info->close(mm_info);
+		tsk_error_reset();
 		tsk_errno = TSK_ERR_MM_UNKTYPE;
 		snprintf(tsk_errstr, TSK_ERRSTR_L,
 		    "Mac or %s at %" PRIuDADDR, set, offset);
-		tsk_errstr2[0] = '\0';
 		return NULL;
 	    }
 	}
@@ -123,9 +124,8 @@ mm_open(IMG_INFO * img_info, DADDR_T offset, const TCHAR * type)
 	}
 
 	if (mm_set == NULL) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_UNKTYPE;
-	    tsk_errstr[0] = '\0';
-	    tsk_errstr2[0] = '\0';
 	    return NULL;
 	}
 
@@ -150,9 +150,9 @@ mm_open(IMG_INFO * img_info, DADDR_T offset, const TCHAR * type)
 	    return gpt_open(img_info, offset);
 	case MM_UNSUPP:
 	default:
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_UNSUPTYPE;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L, "%s", type);
-	    tsk_errstr2[0] = '\0';
 	    return NULL;
 	}
     }
