@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/08/25 18:28:43 $
+ * $Date: 2006/09/22 21:31:41 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -11,9 +11,6 @@
 #ifndef _MM_TOOLS_H
 #define _MM_TOOLS_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
 
     /*
      * External interface.
@@ -28,6 +25,7 @@ extern "C" {
 
 #include "img_tools.h"
 
+
 #if defined(HAVE_UNISTD)
 #include <unistd.h>
 #endif
@@ -35,6 +33,9 @@ extern "C" {
 #include <sys/param.h>
 #endif
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /* Flags for the return value of walk actions */
 #define WALK_CONT	0x0
@@ -111,7 +112,7 @@ extern "C" {
 /**************************************************************8
  * Generic routines.
  */
-    extern MM_INFO *mm_open(IMG_INFO *, DADDR_T, const TCHAR *);
+    extern MM_INFO *mm_open(IMG_INFO *, DADDR_T, const TSK_TCHAR *);
 //extern SSIZE_T mm_read_block(FS_INFO *, FS_BUF *, int, DADDR_T, const char *);
     extern SSIZE_T mm_read_block_nobuf(MM_INFO *, char *, OFF_T, DADDR_T);
     extern void mm_print_types(FILE *);
@@ -136,7 +137,7 @@ extern "C" {
 
 /***** TYPES *****/
 
-    extern char mm_parse_type(const TCHAR *);
+    extern char mm_parse_type(const TSK_TCHAR *);
     extern char *mm_get_type(char);
 
 #define MM_UNSUPP	0x00
