@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/09/06 20:40:02 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -36,7 +36,7 @@ MM_TYPES mm_open_table[] = {
  * Returns MM_UNSUPP if string cannot be parsed
  * */
 char
-mm_parse_type(const TCHAR * str)
+mm_parse_type(const TSK_TCHAR * str)
 {
     char tmp[16];
     int i;
