@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/12/07 16:38:18 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -17,7 +17,7 @@
 #include "mm_tools.h"
 
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 static uint8_t print_bytes = 0;
 static uint8_t recurse = 0;
@@ -29,7 +29,7 @@ void
 usage()
 {
     TFPRINTF(stderr,
-	_T
+	_TSK_T
 	("%s [-i imgtype] [-o imgoffset] [-brvV] [-t mmtype] image [images]\n"),
 	progname);
     tsk_fprintf(stderr,
@@ -139,56 +139,56 @@ print_header(MM_INFO * mm)
 
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
     MM_INFO *mm;
-    TCHAR *mmtype = NULL;
+    TSK_TCHAR *mmtype = NULL;
     int ch;
     SSIZE_T imgoff = 0;
     uint8_t flags = 0;
-    TCHAR *imgtype = NULL;
+    TSK_TCHAR *imgtype = NULL;
     IMG_INFO *img;
 
     progname = argv[0];
 
-    while ((ch = getopt(argc, argv, _T("bi:o:rt:vV"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("bi:o:rt:vV"))) > 0) {
 	switch (ch) {
-	case _T('b'):
+	case _TSK_T('b'):
 	    print_bytes = 1;
 	    break;
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 
 	    break;
 
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('r'):
+	case _TSK_T('r'):
 	    recurse = 1;
 	    break;
-	case _T('t'):
+	case _TSK_T('t'):
 	    mmtype = optarg;
-	    if (TSTRCMP(mmtype, _T("list")) == 0) {
+	    if (TSTRCMP(mmtype, _TSK_T("list")) == 0) {
 		mm_print_types(stderr);
 		exit(1);
 	    }
 
 	    break;
-	case _T('v'):
+	case _TSK_T('v'):
 	    verbose++;
 	    break;
-	case _T('V'):
+	case _TSK_T('V'):
 	    print_version(stdout);
 	    exit(0);
-	case _T('?'):
+	case _TSK_T('?'):
 	default:
 	    tsk_fprintf(stderr, "Unknown argument\n");
 	    usage();
@@ -202,7 +202,9 @@ MAIN(int argc, TCHAR ** argv)
     }
 
     /* open the image */
-    img = img_open(imgtype, argc - optind, (const TCHAR **) &argv[optind]);
+    img =
+	img_open(imgtype, argc - optind,
+	(const TSK_TCHAR **) &argv[optind]);
 
     if (img == NULL) {
 	tsk_error_print(stderr);
