@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/09/06 20:40:02 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2005 Brian Carrier.  All rights reserved
@@ -15,13 +15,13 @@
 
 #include "mm_tools.h"
 
-static TCHAR *progname;
+static TSK_TCHAR *progname;
 
 void
 usage()
 {
     TFPRINTF(stderr,
-	_T
+	_TSK_T
 	("%s [-i imgtype] [-o imgoffset] [-vV] [-t mmtype] image [images]\n"),
 	progname);
     tsk_fprintf(stderr,
@@ -44,37 +44,37 @@ print_stats(MM_INFO * mm)
 }
 
 int
-MAIN(int argc, TCHAR ** argv)
+MAIN(int argc, TSK_TCHAR ** argv)
 {
     MM_INFO *mm;
-    TCHAR *imgtype = NULL;
-    TCHAR *mmtype = NULL;
+    TSK_TCHAR *imgtype = NULL;
+    TSK_TCHAR *mmtype = NULL;
     int ch;
     SSIZE_T imgoff = 0;
     IMG_INFO *img;
 
     progname = argv[0];
 
-    while ((ch = getopt(argc, argv, _T("i:o:t:vV"))) > 0) {
+    while ((ch = getopt(argc, argv, _TSK_T("i:o:t:vV"))) > 0) {
 	switch (ch) {
-	case _T('i'):
+	case _TSK_T('i'):
 	    imgtype = optarg;
-	    if (TSTRCMP(imgtype, _T("list")) == 0) {
+	    if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
 		img_print_types(stderr);
 		exit(1);
 	    }
 
 	    break;
 
-	case _T('o'):
+	case _TSK_T('o'):
 	    if ((imgoff = parse_offset(optarg)) == -1) {
 		tsk_error_print(stderr);
 		exit(1);
 	    }
 	    break;
-	case _T('t'):
+	case _TSK_T('t'):
 	    mmtype = optarg;
-	    if (TSTRCMP(mmtype, _T("list")) == 0) {
+	    if (TSTRCMP(mmtype, _TSK_T("list")) == 0) {
 		mm_print_types(stderr);
 		exit(1);
 	    }
@@ -101,7 +101,7 @@ MAIN(int argc, TCHAR ** argv)
     /* open the image */
     if ((img =
 	    img_open(imgtype, argc - optind,
-		(const TCHAR **) &argv[optind])) == NULL) {
+		(const TSK_TCHAR **) &argv[optind])) == NULL) {
 	tsk_error_print(stderr);
 	exit(1);
     }
