@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/09/01 15:23:03 $
+ * $Date: 2006/12/07 16:38:18 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -101,10 +101,10 @@ sun_load_table_i386(MM_INFO * mm, sun_dlabel_i386 * dlabel_x86)
 	    continue;
 
 	if (getu32(mm->endian, dlabel_x86->part[idx].start_sec) > max_addr) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_BLK_NUM;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"sun_load_i386: Starting sector too large for image");
-	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 
@@ -162,10 +162,10 @@ sun_load_table_sparc(MM_INFO * mm, sun_dlabel_sparc * dlabel_sp)
 	    continue;
 
 	if (part_start > max_addr) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_BLK_NUM;
 	    snprintf(tsk_errstr, TSK_ERRSTR_L,
 		"sun_load_sparc: Starting sector too large for image");
-	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 
@@ -206,10 +206,10 @@ sun_load_table(MM_INFO * mm)
     /* Sanity check in case label sizes change */
     if ((sizeof(*dlabel_sp) > LABEL_BUF_SIZE) ||
 	(sizeof(*dlabel_x86) > LABEL_BUF_SIZE)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_BUF;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "sun_load_table: Buffer smaller than label sizes");
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -224,8 +224,8 @@ sun_load_table(MM_INFO * mm)
     /* If -1 is returned, tsk_errno is already set */
     if (cnt != LABEL_BUF_SIZE) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "SUN Disk Label in Sector: %" PRIuDADDR, taddr);
@@ -264,8 +264,8 @@ sun_load_table(MM_INFO * mm)
 
     if (cnt != LABEL_BUF_SIZE) {
 	if (cnt != -1) {
+	    tsk_error_reset();
 	    tsk_errno = TSK_ERR_MM_READ;
-	    tsk_errstr[0] = '\0';
 	}
 	snprintf(tsk_errstr2, TSK_ERRSTR_L,
 	    "SUN (Intel) Disk Label in Sector: %" PRIuDADDR, taddr);
@@ -274,20 +274,20 @@ sun_load_table(MM_INFO * mm)
 
     dlabel_x86 = (sun_dlabel_i386 *) buf;
     if (mm_guessu16(mm, dlabel_x86->magic, SUN_MAGIC)) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "SUN (intel) partition table (Sector: %"
 	    PRIuDADDR ") %x", taddr, getu16(mm->endian, dlabel_sp->magic));
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
     if (getu32(mm->endian, dlabel_x86->sanity) != SUN_SANITY) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_MAGIC;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "SUN (intel) sanity value (Sector: %" PRIuDADDR
 	    ") %x", taddr, getu16(mm->endian, dlabel_sp->magic));
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -308,18 +308,18 @@ sun_part_walk(MM_INFO * mm, PNUM_T start, PNUM_T last, int flags,
     unsigned int cnt = 0;
 
     if (start < mm->first_part || start > mm->last_part) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "sun_part_walk: Starting partition: %" PRIuPNUM, start);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
     if (last < mm->first_part || last > mm->last_part) {
+	tsk_error_reset();
 	tsk_errno = TSK_ERR_MM_WALK_RNG;
 	snprintf(tsk_errstr, TSK_ERRSTR_L,
 	    "sun_part_walk: Ending partition: %" PRIuPNUM, last);
-	tsk_errstr2[0] = '\0';
 	return 1;
     }
 
@@ -352,7 +352,12 @@ sun_close(MM_INFO * mm)
 MM_INFO *
 sun_open(IMG_INFO * img_info, DADDR_T offset)
 {
-    MM_INFO *mm = (MM_INFO *) mymalloc(sizeof(*mm));
+    MM_INFO *mm;
+
+    // clean up any errors that are lying around
+    tsk_error_reset();
+
+    mm = (MM_INFO *) mymalloc(sizeof(*mm));
     if (mm == NULL)
 	return NULL;
 
