@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/06/15 17:27:57 $
+ * $Date: 2006/08/17 17:13:33 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2004 Brian Carrier.  All rights reserved
@@ -24,8 +24,7 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "tsk_os.h"
-#include "libauxtools.h"
+#include "aux_tools.h"
 
 extern char *progname;
 
